/************************************************************************/
/*                                                                      */
/*   File name :  libmk5tok5.c                                          */
/*                                                                      */
/*      Version:   2009-06-07                                           */
/*                                                                      */
/*   Copyright (c) 2003-2009 T.Kondo/NICT All Right Reserved            */
/*                                                                      */
/************************************************************************/
/*                                   
 library for Mark5 to K5 conversion program
     2003-03-13 start

2009-06-07
    "mk5b_dat_anal" and "mk5b_sync_detec" are added

2009-02-17
     "mk5_dat_anal_without_sync","pickup_m5_header","parse_m5_header"
     are added.

2009-02-15
     "read_m5_1frame_new" is added, 
     which read 1 frame of Mark 5 data compulsory

2008-09-26
     add k5ch[16] as a member of mk5tok5_info type variable to make
     it possible to assign channels between K5 and Mk5 freely in case
     of k5tom5 operation. "k5tom5_info_setup", "monit_mk5info",
     "make_k5tom5_info_file" were modified.

2008-07-24
     To add 16 track mode to "k5tom5", following update is made.
     "so" is added as a member of m5head type variable.
     "m5_hd_synctime_make16" and "m5_hd_make16" are added.
     16 track mode is added to "m5_hd_make_IV"

2008-07-04
     "mod_func" was added. This function generates modulation pattern
     for VLBA format.
     "make_k5tom5_info_file" was modified to support modulation ON, 
     and arguments were changed to k5m5runp_t *runp.
     "k5tom5_info_setup"  was modified to support modulation ON.

2008-01-08
     sync detection algorithm for Mark5 data in "sync_detect2new64r" 
     and "mk5_sync_detect" is improved to avoid false detection of 
     sync-pattern in case of time-field starting '8' 
     (this problem appears due to year 2008) 
     
2007-07-18
     the number of characters of k5 file name and Mark5 file name was extended 
	 from 80 to 256

2006-09-27
     A bug in 'k5name_make' was fixed. Array size to contain Mark5 name
     except path was insufficient.
     
2006-09-26
     parseMK5aux is updated to support the pick up of videcon # in case
     of VLBA mode.
     parameter (vcntable : video comberter # table)) was added to 
     arguments of 'mk5_dat_anal'

2005-09-17
     use 'LL' like 0xFFFFFFFFFFFFFFFFLL to express int64_t constant 
     to avoid warning message appearing in the latest version of gcc.

2005-06-17
      divided into libmk5tok5.h and libmk5tok5.c
*/

#include "libmk5tok5.h"

void headerchkK5(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
			   int* sfreq, int* numch, long int* seconds, int* hh,
			   int* mm, int* ss)
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; //2003-10-12
                  //0,0,0,0,0,0,0};
  int smplbit0, sfreq0, seco, nch;
    *iok=-1;
    /* sync pattern check */
    if((bit64[0] == 0xFF) && (bit64[1] == 0xFF) && 
        (bit64[2] == 0xFF) && (bit64[3] == 0xFF) && (bit64[7] == 0x8B)){  // this is sync block (new FMT)

        *iok=0;
        /* parse sampling information and get time data */ 
        *spatn=(int)(bit64[7] & 255);       // spatn = 139 means sync pattern
        smplbit0 = (bit64[6] >> 6) & 15;    // 6 times bit shift to right
        *smplbit = sdat[smplbit0];
        sfreq0 = ( bit64[6] >> 2) & 15 ;
        *sfreq=fdat[sfreq0];
        nch = bit64[6] & 2;
        if (nch == 2) {
            *numch = 4; 
        } else {
            *numch = 1;
        }
        seco = bit64[6] & 1;
        *seconds=seco*(long int)65536+
		        (long int)(bit64[5] & 255)*(long int)256+
		        (long int)(bit64[4] & 255);
        *hh = (*seconds)/3600;
        *mm = ((*seconds) % 3600)/60;
        *ss = (*seconds) % 60;
    }
}

int chk_K5_header(ip_vlbi_file_old_t *S)
{
 /*
    Read 64bit Data header check to get sampling parameters 
  
    Return    0:  first 64 bit is sync block
           +VE :  sync found after VE 32bit-word read
           -VE :  Error occurred
 */
      
       int iok, i, n, i32;
       //fpos_t save;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;
       S->bytesread = fread( S->bit32, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 ){
           //printf("chk_K5_header: File EOF or Read Error! (%s)\n",S->fname);
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }
       for (n=0; n<4; n++){
           S->bit64[n]=S->bit32[n];
       }

loop1:

       S->bytesread = fread( S->bit32, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 )
       {
           //printf("chk_K5_header: File EOF or Read Error! (%s)\n",S->fname);
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }
       for (n=0; n<4; n++){
           S->bit64[n+4]=S->bit32[n];
       }
       headerchkK5(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
                        &S->seconds,&S->hh,&S->mm,&S->ss);
       if (iok == 0)   // sync detection
	   {
          if(i32 !=0) printf("  found after %d bits read!\n",i32*32);
          //printf("chk_K5_header: Header (64bits) Sync Detected!!\n");
		  //printf("chk_K5_header:   File : %s\n",S->fname);
          //printf("chk_K5_header:   A/D(bits) %d  CHs %d  SFreq(kHz) %d",
          //          S->adbit,S->numch, S->sfreq);
          //printf("  Time %02d:%02d:%02d  sec %d\n",
          //          S->hh,S->mm,S->ss,S->seconds);
          S->format=1;          
          goto exit_loop;
	   }
       else
       {
           if(i32 ==0) {
               printf("chk_K5_header: First 64bit is not header block!\n");
               printf("chk_K5_header:   Data file name (%s)\n",S->fname);
               printf("chk_K5_header:   searching header.....");
           }
           /* search sync pattern by shifting read position by 32 bits forward */
           for (n=0; n<4; n++){
               S->bit64[n]=S->bit64[n+4];
           }
           i32++;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       //printf("checkheader:   vflag     dtime_vflag     %d %d\n",S->vflag,S->dtime_vflag);
       //printf("checkheader:   vflag_old dtime_vflag_old %d %d\n",S->vflag_old,S->dtime_vflag_old);
       return i32;
}

int fwd_K5_Nsec(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_old_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
	//printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
		        return -1;
            }
		}
        if(chk_K5_header(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}

/* crc subroutine ripoff  by David*/
int crcc(unsigned char* idata, int len, int mask, int cycl) {
	unsigned int istate = 0;
	int idbit;
	int q, ich, icb;

	for (idbit = 1; idbit <= len; idbit++) {
		q = istate & 1;
		ich = (idbit - 1) / 8;
		icb = 7 - (idbit - 1) % 8;
		if ((((idata[ich] >> icb) & 1) ^ q) == 0)
			istate &= -2;
		else {
			istate ^= mask;
			istate |= 1;
		}
		istate = (istate >> 1) | ((istate & 1) << (cycl -1));  
	}

	return ((int) istate);
}

int crc_12(unsigned char* idata, int len) {
/*  CRC-12 code calc
        X^12 + X^11 + X^3 + X^2 + X +1
*/
	unsigned int istate = 0;
	int idbit;
	int q, ich, icb;
    int mask=0x80F;
    int cycl=12;

	for (idbit = 1; idbit <= len; idbit++) {
		q = (istate>>(cycl-1)) & 1;           // pick up cycl'th bit of istate
		ich = (idbit - 1) / 8;     //byte position in idata  
		icb = 7 - (idbit - 1) % 8; //bit position in byte 0-7

        istate = (istate << 1) & 0xFFF;  // shift left and mask 12 bit

		if ((((idata[ich] >> icb) & 1) ^ q) == 0)    // check bit = cycl'th bit of istate
			istate &= -2;     // reset LSB of istate
		else {
			istate ^= mask;   //  exclusive-or with mask
			istate |= 1;      //  set LSB of istate  
		}
	}

	return ((int) istate);
}

int crc_16(unsigned char* idata, int len) {
/*  CRC-16 code (VLBA Format) calc
        X^16 + X^15 + X^2 +1
*/
	unsigned int istate = 0;
	int idbit;
	int q, ich, icb;
    int mask=0x8005;
    int cycl=16;

	for (idbit = 1; idbit <= len; idbit++) {
		q = (istate>>(cycl-1)) & 1;           // pick up cycl'th bit of istate
		ich = (idbit - 1) / 8;     //byte position in idata  
		icb = 7 - (idbit - 1) % 8; //bit position in byte 0-7

        istate = (istate << 1) & 0xFFFF;  // shift left and mask 16 bit

		if ((((idata[ich] >> icb) & 1) ^ q) == 0)    // check bit = cycl'th bit of istate
			istate &= -2;     // reset LSB of istate
		else {
			istate ^= mask;   //  exclusive-or with mask
			istate |= 1;      //  set LSB of istate  
		}
	}

	return ((int) istate);
}


/* return parity  by David */
int parity(unsigned char c) {
	int i;
	int n = 0;

	for (i = 0; i < 8; i++) {
		n += !!(c & (1 << i));
	}

	n = (n + 1) % 2;
	
	return (n);
}


void m5_hd_synctime_make(m5head_t *hd)
{
    /* Mark 5 header block creation */
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    unsigned int i0=0;
    unsigned int ff=0xFFFFFFFF;
    

    // now support only VLBA format and with parity mode


    m=0;

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->io ^= ff;
            hd->ihead[m]=hd->io;
            m++;
            if(k==7){ // insert parity bit
               hd->io ^= 00;
               hd->ihead[m]=hd->io;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->io ^= i0;
           } else {
              hd->io ^= ff;
           } 
           hd->ihead[m]=hd->io;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->io ^= i0;
          } else {
             hd->io ^= ff;
          }
          hd->ihead[m]=hd->io;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->io ^= i0;
        } else {
           hd->io ^= ff;
        }
        hd->ihead[m]=hd->io;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->io ^= i0;
            } else {
               hd->io ^= ff;
            } 
            hd->ihead[m]=hd->io;
            m++;
        }
    }
    
}

void m5_hd_synctime_make64(m5head_t *hd)
{
    /* Mark 5 header block creation (64bit mode)*/
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    //unsigned int i0=0;
    //unsigned int ff=0xFFFFFFFF;
    int64_t l0=0;
#ifdef WIN32
    int64_t lff=0xFFFFFFFFFFFFFFFF;
#else
    int64_t lff=0xFFFFFFFFFFFFFFFFLL;
#endif
    //int64_t lw,lw1;
    

    // now support only VLBA format and with parity mode


    m=0;

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->lo ^= lff;
            hd->lhead[m]=hd->lo;
            m++;
            if(k==7){ // insert parity bit
               hd->lo ^= l0;
               hd->lhead[m]=hd->lo;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->lo ^= l0;
           } else {
              hd->lo ^= lff;
           } 
           hd->lhead[m]=hd->lo;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->lo ^= l0;
          } else {
             hd->lo ^= lff;
          }
          hd->lhead[m]=hd->lo;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->lo ^= l0;
        } else {
           hd->lo ^= lff;
        }
        hd->lhead[m]=hd->lo;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->lo ^= l0;
            } else {
               hd->lo ^= lff;
            } 
            hd->lhead[m]=hd->lo;
            m++;
        }
    }
    
}

void m5_hd_synctime_make16(m5head_t *hd)  // 2008-07-24
{
    /* Mark 5 header block creation */
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    unsigned short int s0=0;
    unsigned short int sff=0xFFFF;
    

    // now support only VLBA format and with parity mode


    m=0;

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->so ^= sff;
            hd->shead[m]=hd->so;
            m++;
            if(k==7){ // insert parity bit
               hd->so ^= 0;
               hd->shead[m]=hd->so;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->so ^= s0;
           } else {
              hd->so ^= sff;
           } 
           hd->shead[m]=hd->so;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->so ^= s0;
          } else {
             hd->so ^= sff;
          }
          hd->shead[m]=hd->so;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->so ^= s0;
        } else {
           hd->so ^= sff;
        }
        hd->shead[m]=hd->so;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->so ^= s0;
            } else {
               hd->so ^= sff;
            } 
            hd->shead[m]=hd->so;
            m++;
        }
    }
    
}

void m5_hd_make16(m5head_t *hd)  // 2008-07-24
{
    /* Mark 5 header block creation */
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    unsigned short int s0=0;
    unsigned short int sff=0xFFFF;
    

    // now support only VLBA format and with parity mode

    /* AUX field   SSRHPPPPTGBSCD00 */
    // set all zero

    m=0;
    for(i=0; i<8; i++){
        for(k=0; k<8; k++){
            hd->so ^= s0;
            hd->shead[m]=hd->so;
            m++;
            if(k==7){   // insert parity bit 
               hd->so ^= sff;
               hd->shead[m]=hd->so;
               m++;
            }
        }
    }

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->so ^= sff;
            hd->shead[m]=hd->so;
            m++;
            if(k==7){ // insert parity bit
               hd->so ^= 0;
               hd->shead[m]=hd->so;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->so ^= s0;
           } else {
              hd->so ^= sff;
           } 
           hd->shead[m]=hd->so;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->so ^= s0;
          } else {
             hd->so ^= sff;
          }
          hd->shead[m]=hd->so;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->so ^= s0;
        } else {
           hd->so ^= sff;
        }
        hd->shead[m]=hd->so;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->so ^= s0;
            } else {
               hd->so ^= sff;
            } 
            hd->shead[m]=hd->so;
            m++;
        }
    }
    
}


void m5_hd_make(m5head_t *hd)
{
    /* Mark 5 header block creation */
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    unsigned int i0=0;
    unsigned int ff=0xFFFFFFFF;
    

    // now support only VLBA format and with parity mode

    /* AUX field   SSRHPPPPTGBSCD00 */
    // set all zero

    m=0;
    for(i=0; i<8; i++){
        for(k=0; k<8; k++){
            hd->io ^= i0;
            hd->ihead[m]=hd->io;
            m++;
            if(k==7){   // insert parity bit 
               hd->io ^= ff;
               hd->ihead[m]=hd->io;
               m++;
            }
        }
    }

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->io ^= ff;
            hd->ihead[m]=hd->io;
            m++;
            if(k==7){ // insert parity bit
               hd->io ^= 00;
               hd->ihead[m]=hd->io;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->io ^= i0;
           } else {
              hd->io ^= ff;
           } 
           hd->ihead[m]=hd->io;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->io ^= i0;
          } else {
             hd->io ^= ff;
          }
          hd->ihead[m]=hd->io;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->io ^= i0;
        } else {
           hd->io ^= ff;
        }
        hd->ihead[m]=hd->io;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->io ^= i0;
            } else {
               hd->io ^= ff;
            } 
            hd->ihead[m]=hd->io;
            m++;
        }
    }
    
}

void m5_hd_make64(m5head_t *hd)
{
    /* Mark 5 header block creation for 64bit mode*/
    int i,n,a,d,k;
    int m;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char p;
    unsigned char bdat[6];
    //unsigned int i0=0;
    //unsigned int ff=0xFFFFFFFF;
    int64_t l0=0;
#ifdef WIN32
    int64_t lff=0xFFFFFFFFFFFFFFFF;
#else
    int64_t lff=0xFFFFFFFFFFFFFFFFLL;
#endif    

    // now support only VLBA format and with parity mode

    /* AUX field   SSRHPPPPTGBSCD00 */
    // set all zero

    m=0;
    for(i=0; i<8; i++){
        for(k=0; k<8; k++){
            hd->lo ^= l0;
            hd->lhead[m]=hd->lo;
            m++;
            if(k==7){   // insert parity bit 
               hd->lo ^= lff;
               hd->lhead[m]=hd->lo;
               m++;
            }
        }
    }

    /* Sync word    FFFFFFFF  */
    for(i=0; i<4; i++){
        for(k=0; k<8; k++){
            hd->lo ^= lff;
            hd->lhead[m]=hd->lo;
            m++;
            if(k==7){ // insert parity bit
               hd->lo ^= l0;
               hd->lhead[m]=hd->lo;
               m++;
            }
        }
    }

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(hd->mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(hd->seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(hd->ifsec/a) % 10;
        a/=10;
    }

    //for(n=0; n<12; n++) printf("%d",idat[n]);
    //printf("\n");


    for(n=0; n<12; n++){  // 4 bit step
        a=idat[n];
        for(k=0; k<4; k++){
           d= a & 0x8;
           if(d==0){
              hd->lo ^= l0;
           } else {
              hd->lo ^= lff;
           } 
           hd->lhead[m]=hd->lo;
           a <<= 1;
           m++;
        }
        if( (n % 2) == 1){  // insert parity bit 
          p=(idat[n-1] << 4) + idat[n];
          // parity set
          if(parity(p)==0){
             hd->lo ^= l0;
          } else {
             hd->lo ^= lff;
          }
          hd->lhead[m]=hd->lo;
          m++;
        } 
    }
   
    //CRC16
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

    a=crc16;
    for(k=0; k<16; k++){
        d= a & 0x1;
        if(d==0){
           hd->lo ^= l0;
        } else {
           hd->lo ^= lff;
        }
        hd->lhead[m]=hd->lo;
        a >>= 1;
        m++;
        if(k==7 || k==15){ // insert parity
            if(k==7) p= (unsigned char)(crc16 & 0xFF);
            if(k==15) p= (unsigned char)((crc16 >> 8) & 0xFF);
            // parity set
            if(parity(p)==0){
               hd->lo ^= l0;
            } else {
               hd->lo ^= lff;
            } 
            hd->lhead[m]=hd->lo;
            m++;
        }
    }
    
}

void m5_hd_make_IV(m5head_t *hd)
{
    /* Mark 5 header block (MarkIV format) creation */
    //   2008-07-24   16 track mode is added
    int i,n,a,d,k;
    int m,mb;
    int crc12;
    //int64_t lw1;
    int idat[13];
    unsigned int intdat[64];
    unsigned char bytedat[64];
    unsigned char bw,bw1,bw2;
    unsigned char bdat[19*64];  // for crc calc  148bit/trk is used
    unsigned int i0=0;
    unsigned int ff=0xFFFFFFFF;
    unsigned int iw;
    unsigned short int s0=0;        // 2008-07-24
    unsigned short int sff=0xFFFF;  // 2008-07-24
    unsigned short int sw;          // 2008-07-24
    int64_t l0=0;
#ifdef WIN32
    int64_t lff=0xFFFFFFFFFFFFFFFF;
#else
    int64_t lff=0xFFFFFFFFFFFFFFFFLL;
#endif    
    int64_t lw;
    int set_table[]={ 0x00000001,0x00000002,0x00000004,0x00000008,
                      0x00000010,0x00000020,0x00000040,0x00000080,
                      0x00000100,0x00000200,0x00000400,0x00000800,
                      0x00001000,0x00002000,0x00004000,0x00008000,
                      0x00010000,0x00020000,0x00040000,0x00080000,
                      0x00100000,0x00200000,0x00400000,0x00800000,
                      0x01000000,0x02000000,0x04000000,0x08000000,
                      0x10000000,0x20000000,0x40000000,0x80000000};
#ifdef WIN32
       int64_t set_ltabl[]={ 
        0x1,0x2,0x4,0x8,
        0x10,0x20,0x40,0x80,
        0x100,0x200,0x400,0x800,
        0x1000,0x2000,0x4000,0x8000,
        0x10000,0x20000,0x40000,0x80000,
        0x100000,0x200000,0x400000,0x800000,
        0x1000000,0x2000000,0x4000000,0x8000000,
        0x10000000,0x20000000,0x40000000,0x80000000,
        0x100000000,0x200000000,0x400000000,0x800000000,
        0x1000000000,0x2000000000,0x4000000000,0x8000000000,
        0x10000000000,0x20000000000,0x40000000000,0x80000000000,
        0x100000000000,0x200000000000,0x400000000000,0x800000000000,
        0x1000000000000,0x2000000000000,0x4000000000000,0x8000000000000,
        0x10000000000000,0x20000000000000,0x40000000000000,0x80000000000000,
        0x100000000000000,0x200000000000000,0x400000000000000,0x800000000000000,
        0x1000000000000000,0x2000000000000000,0x4000000000000000,0x8000000000000000};
#else 
       int64_t set_ltabl[]={ 
        0x1LL,0x2LL,0x4LL,0x8LL,
        0x10LL,0x20LL,0x40LL,0x80LL,
        0x100LL,0x200LL,0x400LL,0x800LL,
        0x1000LL,0x2000LL,0x4000LL,0x8000LL,
        0x10000LL,0x20000LL,0x40000LL,0x80000LL,
        0x100000LL,0x200000LL,0x400000LL,0x800000LL,
        0x1000000LL,0x2000000LL,0x4000000LL,0x8000000LL,
        0x10000000LL,0x20000000LL,0x40000000LL,0x80000000LL,
        0x100000000LL,0x200000000LL,0x400000000LL,0x800000000LL,
        0x1000000000LL,0x2000000000LL,0x4000000000LL,0x8000000000LL,
        0x10000000000LL,0x20000000000LL,0x40000000000LL,0x80000000000LL,
        0x100000000000LL,0x200000000000LL,0x400000000000LL,0x800000000000LL,
        0x1000000000000LL,0x2000000000000LL,0x4000000000000LL,0x8000000000000LL,
        0x10000000000000LL,0x20000000000000LL,0x40000000000000LL,0x80000000000000LL,
        0x100000000000000LL,0x200000000000000LL,0x400000000000000LL,0x800000000000000LL,
        0x1000000000000000LL,0x2000000000000000LL,0x4000000000000000LL,0x8000000000000000LL};
#endif
   

    // Mark IV format and without parity

    m=0;
    mb=0;  // byte position in bdat calc
    /* AUX field   aaaabbbbrrssttuu */
    // set aaaa and bbbb zero
    for(i=0; i<4; i++){
        switch(hd->nbits){     // 2008-07-24
          case 16: 
                   for(k=0; k<8; k++){
                      hd->shead[m]=s0;
                      m++;
                   }
                   break;
          case 32: 
                   for(k=0; k<8; k++){
                      hd->ihead[m]=i0;
                      m++;
                   }
                   break;
          case 64: 
                   for(k=0; k<8; k++){
                      hd->lhead[m]=l0;
                      m++;
                   }
                   break;
          default: ;
        }
//        if(hd->nbits==64){
//            for(k=0; k<8; k++){
//                hd->lhead[m]=l0;
//                m++;
//            }
//        } else {
//            for(k=0; k<8; k++){
//                hd->ihead[m]=i0;
//                m++;
//            }
//        }
        for(n=0; n<hd->nbits; n++){
            bdat[19*n+mb]=0;  // for CRC12 calc
        }
        mb++;
    }

    //set rr (track info)
    for(i=0; i<hd->nbits; i++){
        //head stack # set
        bw=(hd->headstackinfo[i]-1)<<6;
        //track# set
        bw1= (((int)(hd->trkinfo[i]/10))<<4) & 0x30;  // 10's digit BCD
        bw2= (hd->trkinfo[i] % 10);    // 1's digit BCD
        //
        bw = bw | (bw1 | bw2);
        bytedat[i]=bw;
        bdat[19*i+mb]=bw;
    }
    mb++;
    switch(hd->nbits){     // 2008-07-24
        case 16: 
                for(k=0; k<8; k++){
                   sw=0;
                   for(i=0; i<hd->nbits; i++){
                      sw >>= 1;
                      sw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
                   }
                   hd->shead[m]=sw;
                   m++;
                }
                break;
        case 32: 
                for(k=0; k<8; k++){
                   iw=0;
                   for(i=0; i<hd->nbits; i++){
                      iw >>= 1;
                      iw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
                   }
                   hd->ihead[m]=iw;
                   m++;
                }
                break;
        case 64: 
                for(k=0; k<8; k++){
                   lw=0;
                   for(i=0; i<hd->nbits; i++){
                      /* following code doesn't work properly on Win32 (Why?) */
                      //lw >>= 1;
                      //lw |= (int64_t)((bytedat[i] >> (7-k)) & 0x1 << (hd->nbits-1)) ;  //  start from MSB
                      /* follwoing code works well */
                      if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
                   }
                   hd->lhead[m]=lw;
                   m++;
                }
                break;
        default:;
    }

//    if(hd->nbits==64){
//        for(k=0; k<8; k++){
//            lw=0;
//            for(i=0; i<hd->nbits; i++){
//                /* following code doesn't work properly on Win32 (Why?) */
//                //lw >>= 1;
//                //lw |= (int64_t)((bytedat[i] >> (7-k)) & 0x1 << (hd->nbits-1)) ;  //  start from MSB
//                /* follwoing code works well */
//                if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
//           }
//            hd->lhead[m]=lw;
//            m++;
//        }
//    } else {
//        for(k=0; k<8; k++){
//            iw=0;
//            for(i=0; i<hd->nbits; i++){
//                iw >>= 1;
//                iw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
//            }
//            hd->ihead[m]=iw;
//            m++;
//        }
//    }
    //set ss (identifier of source of data in frame)
    for(i=0; i<hd->nbits; i++){
        //fan-out info set
        bw=(hd->faninfo[i]-1)<<6;
        // sign bit info set
        if(hd->adinfo[i]==1) bw |= 0x20;    //set bit#5 (start bit#=0)
        //side band info set
        if(hd->lsbinfo[i]==1) bw |= 0x10;    //set bit#4 (start bit#=0)
        //Video Converter info set
        if(hd->vcinfo[i] > 0) bw |= (hd->vcinfo[i]-1); 
        bytedat[i]=bw;

        bdat[19*i+mb]=bw;
    }
    mb++;
    switch(hd->nbits){     // 2008-07-24
        case 16: 
                 for(k=0; k<8; k++){
                    sw=0;
                    for(i=0; i<hd->nbits; i++){
                       sw >>= 1;
                       sw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
                    }
                    hd->shead[m]=sw;
                    m++;
                 }
                 break;
        case 32: 
                 for(k=0; k<8; k++){
                    iw=0;
                    for(i=0; i<hd->nbits; i++){
                       iw >>= 1;
                       iw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
                    }
                    hd->ihead[m]=iw;
                    m++;
                 }
                 break;
        case 64: 
                 for(k=0; k<8; k++){
                    lw=0;
                    for(i=0; i<hd->nbits; i++){
                       //lw >>= 1;
                       //lw |= (int64_t)((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
                       if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
                    }
                    hd->lhead[m]=lw;
                    m++;
                 }
                 break;
        default:;
    }
//    if(hd->nbits==64){
//        for(k=0; k<8; k++){
//            lw=0;
//            for(i=0; i<hd->nbits; i++){
//                //lw >>= 1;
//                //lw |= (int64_t)((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
//                if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
//            }
//            hd->lhead[m]=lw;
//            m++;
//        }
//    } else {
//        for(k=0; k<8; k++){
//            iw=0;
//            for(i=0; i<hd->nbits; i++){
//                iw >>= 1;
//                iw |= ((bytedat[i] >> (7-k)) & 0x1) << (hd->nbits-1) ;  //  start from MSB
//            }
//            hd->ihead[m]=iw;
//            m++;
//        }
//    }

    //set tt  (formatter status and error flag)
    //  set all zero
    for(n=0; n<hd->nbits; n++){
        bdat[19*n+mb]=0;
    }
    mb++;
    switch(hd->nbits){     // 2008-07-24
        case 16: 
                 for(k=0; k<8; k++){
                     hd->shead[m]=s0;
                     m++;
                 }
                 break;
        case 32: 
                 for(k=0; k<8; k++){
                     hd->ihead[m]=i0;
                     m++;
                 }
                 break;
        case 64: 
                 for(k=0; k<8; k++){
                     hd->lhead[m]=l0;
                     m++;
                 }
                 break;
        default:;
    }
//    if(hd->nbits==64){
//        for(k=0; k<8; k++){
//            hd->lhead[m]=l0;
//            m++;
//        }
//    } else {
//        for(k=0; k<8; k++){
//            hd->ihead[m]=i0;
//            m++;
//        }
//    }
    //set uu (system id)
    //  set all zero
    for(n=0; n<hd->nbits; n++){
        bdat[19*n+mb]=0;
    }
    mb++;
    switch(hd->nbits){     // 2008-07-24
        case 16: 
                 for(k=0; k<8; k++){
                     hd->shead[m]=s0;
                     m++;
                 }
                 break;
        case 32: 
                 for(k=0; k<8; k++){
                     hd->ihead[m]=i0;
                     m++;
                 }
                 break;
        case 64: 
                 for(k=0; k<8; k++){
                     hd->lhead[m]=l0;
                     m++;
                 }
                 break;
        default:;
    }
//    if(hd->nbits==64){
//        for(k=0; k<8; k++){
//            hd->lhead[m]=l0;
//            m++;
//        }
//    } else {
//        for(k=0; k<8; k++){
//            hd->ihead[m]=i0;
//            m++;
//        }
//    }

    /* Sync word    FFFFFFFF  */
    switch(hd->nbits){     // 2008-07-24
        case 16: 
                 for(i=0; i<4; i++){
                    for(k=0; k<8; k++){
                       hd->shead[m]=sff;
                       m++;
                    }
                    for(n=0; n<hd->nbits; n++){
                       bdat[19*n+mb]=0xFF;
                    }
                    mb++;
                 }
                 break;
        case 32: 
                 for(i=0; i<4; i++){
                    for(k=0; k<8; k++){
                       hd->ihead[m]=ff;
                       m++;
                    }
                    for(n=0; n<hd->nbits; n++){
                       bdat[19*n+mb]=0xFF;
                    }
                    mb++;
                 }
                 break;
        case 64: 
                 for(i=0; i<4; i++){
                    for(k=0; k<8; k++){
                       hd->lhead[m]=lff;
                       m++;
                    }
                    for(n=0; n<hd->nbits; n++){
                       bdat[19*n+mb]=0xFF;
                    }
                    mb++;
                 }
                 break;
        default:;
    }

//    if(hd->nbits==64){
//        for(i=0; i<4; i++){
//            for(k=0; k<8; k++){
//                hd->lhead[m]=lff;
//                m++;
//            }
//            for(n=0; n<hd->nbits; n++){
//                bdat[19*n+mb]=0xFF;
//            }
//            mb++;
//        }
//    } else {
//        for(i=0; i<4; i++){
//            for(k=0; k<8; k++){
//                hd->ihead[m]=ff;
//                m++;
//            }
//            for(n=0; n<hd->nbits; n++){
//                bdat[19*n+mb]=0xFF;
//            }
//            mb++;
//        }
//    }
    /* Time field */

    //  YDDDHHMMSSsss +CRC12
    idat[0]= hd->yyyy % 10;    // Y
    idat[1]= hd->ddd/100;      // 100D
    idat[2]= (hd->ddd % 100)/10;      // 10D
    idat[3]= hd->ddd % 10;      // 1D
    idat[4]= ((int)(hd->seconds/3600))/10; //10H
    idat[5]= ((int)(hd->seconds/3600)) % 10; //1H
    idat[6]= ((int)((hd->seconds % 3600)/60))/10; //10M
    idat[7]= ((int)((hd->seconds % 3600)/60))%10; //1M
    idat[8]= (int)((hd->seconds % 60)/10); //10S
    idat[9]= (hd->seconds % 60) % 10; //1S
    idat[10]= hd->ifsec/1000;     // 100s      (ifsec: 0.1msec unit)
    idat[11]= (hd->ifsec % 1000)/100;     // 10s
    idat[12]= (hd->ifsec % 100)/10;     // 1s
    //for(n=0; n<13; n++) printf("%d",idat[n]);
    //printf("\n");

    bw=0;
    for(n=0; n<13; n++){  // 4 bit step
        a=idat[n];

        if( (n % 2) == 0){
            bw= a<<4;
        } else {
            bw |= (a & 0xF);
            for(k=0; k<hd->nbits; k++){
               bdat[19*k+mb]=bw;
            } 
            mb++;
            bw=0;
        }


        switch(hd->nbits){     // 2008-07-24
            case 16:
                     for(k=0; k<4; k++){
                        d= a & 0x8;
                        if(d==0){
                           hd->shead[m]=s0;
                        } else {
                           hd->shead[m]=sff;
                        } 
                        a <<= 1;
                        m++;
                     }
                     break;
            case 32:
                     for(k=0; k<4; k++){
                        d= a & 0x8;
                        if(d==0){
                           hd->ihead[m]=i0;
                        } else {
                           hd->ihead[m]=ff;
                        } 
                        a <<= 1;
                        m++;
                     }
                     break;
            case 64:
                     for(k=0; k<4; k++){
                        d= a & 0x8;
                        if(d==0){
                           hd->lhead[m]=l0;
                        } else {
                           hd->lhead[m]=lff;
                        } 
                        a <<= 1;
                        m++;
                     }
                     break;
            default:;
        }
//        if(hd->nbits==64){
//            for(k=0; k<4; k++){
//               d= a & 0x8;
//               if(d==0){
//                  hd->lhead[m]=l0;
//               } else {
//                  hd->lhead[m]=lff;
//               } 
//               a <<= 1;
//               m++;
//            }
//        } else {
//            for(k=0; k<4; k++){
//               d= a & 0x8;
//               if(d==0){
//                  hd->ihead[m]=i0;
//               } else {
//                  hd->ihead[m]=ff;
//               } 
//               a <<= 1;
//               m++;
//            }
//        }
    }
    // set 13th idat (upper 4 bit) set
    for(k=0; k<hd->nbits; k++){
        bdat[19*k+mb]=bw;
    } 
    mb++;
    //CRC12
    for(n=0; n<hd->nbits; n++){
	//   crc12 = crcc(bdat+19*n, 148, 0x180B, 12); // Mask should be checked
	   crc12 = crc_12(bdat+19*n, 148); 
       intdat[n]=crc12;
       //printf("Debug CRC12 %08X\n",rvsint(crc16));
       //printf("Debug CRC12 %08X\n",crc16);
    }


    switch(hd->nbits){     // 2008-07-24
        case 16: 
                 for(k=0; k<12; k++){
                    sw=0;
                    for(i=0; i<hd->nbits; i++){
                       sw >>= 1;
                       sw |= ((intdat[i] >> (11-k)) & 0x1) << (hd->nbits-1);  //  start from MSB
                    }
                    hd->shead[m]=sw;
                    m++;
                 }
                 break;
        case 32: 
                 for(k=0; k<12; k++){
                    iw=0;
                    for(i=0; i<hd->nbits; i++){
                       iw >>= 1;
                       iw |= ((intdat[i] >> (11-k)) & 0x1) << (hd->nbits-1);  //  start from MSB
                    }
                    hd->ihead[m]=iw;
                    m++;
                 }
                 break;
        case 64: 
                 for(k=0; k<12; k++){
                    lw=0;
                    for(i=0; i<hd->nbits; i++){
                       //lw >>= 1;
                       //lw |= (int64_t)((intdat[i] >> (11-k)) & 0x1) << (hd->nbits-1);  //  start from MSB
                       if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
                    }
                    hd->lhead[m]=lw;
                    m++;
                 }
                 break;
        default:;
    }
//    if(hd->nbits==64){
//        for(k=0; k<12; k++){
//            lw=0;
//            for(i=0; i<hd->nbits; i++){
//                //lw >>= 1;
//                //lw |= (int64_t)((intdat[i] >> (11-k)) & 0x1) << (hd->nbits-1);  //  start from MSB
//                if(((bytedat[i] >> (7-k)) & 0x1) ==1) lw |= set_ltabl[i];
//            }
//            hd->lhead[m]=lw;
//            m++;
//        }
//    } else {
//        for(k=0; k<12; k++){
//            iw=0;
//            for(i=0; i<hd->nbits; i++){
//                iw >>= 1;
//                iw |= ((intdat[i] >> (11-k)) & 0x1) << (hd->nbits-1);  //  start from MSB
//            }
//            hd->ihead[m]=iw;
//            m++;
//        }
//    }
}

void ip_hd_make_ini(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[ׂč쐬 */
    // 2003-10-12 sampling frequency faster than 16MHz was added
    int n,fid,aid;
    long int w1;
    unsigned char b,c;
    double fsampl;
    /* sync pattern 1 set */
    for (n=0; n<4; n++){
       ipb->hd[n]=0xFF;
    }
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=(unsigned char)((w1 & 0x10000) >> 16);
    /* ch flag set */
    if( ipb->numch == 4){
        b=b | 0x2;
    }
    /* sampling freq index set */
    fid=0;
    //if( ipb->sfreq ==    40) fid=0;
    if( ipb->sfreq ==   100) fid=1;
    if( ipb->sfreq ==   200) fid=2;
    if( ipb->sfreq ==   500) fid=3;
    if( ipb->sfreq ==  1000) fid=4;
    if( ipb->sfreq ==  2000) fid=5;
    if( ipb->sfreq ==  4000) fid=6;
    if( ipb->sfreq ==  8000) fid=7;
    if( ipb->sfreq == 16000) fid=8;
    if( ipb->sfreq == 32000) fid=9;   // 2003-10-12
    if( ipb->sfreq == 64000) fid=10;  // 2003-10-12
    if( ipb->sfreq == 128000) fid=11;  // 2003-10-12
    if( ipb->sfreq == 256000) fid=12;  // 2003-10-12
    if( ipb->sfreq == 512000) fid=13;  // 2003-10-12
    if( ipb->sfreq == 1024000) fid=14;  // 2003-10-12
    if( ipb->sfreq == 2048000) fid=15;  // 2003-10-12
    c=(unsigned char)(fid << 2);
    b=b | c;
    /* A/D bit length set */
    aid=0;
    //if( ipb->adbit == 1 ) aid=0;
    if( ipb->adbit == 2 ) aid=1;
    if( ipb->adbit == 4 ) aid=2;
    if( ipb->adbit == 8 ) aid=3;
    c=(unsigned char)(aid << 6);
    b=b | c;
    ipb->hd[6]=b;
    /* sync pattern 2 set */
    ipb->hd[7]=0x8B;
    /* internal data initialize */
    fsampl=1000.0*(double)ipb->sfreq; // sampling frequency (Hz)

    ipb->nbsec=(((long)fsampl)*ipb->adbit*ipb->numch)/8;  // # of bytes in 1 sec data except header
    ipb->nbsec+=8;
    ipb->bpos=0;
}

void ip_hd_make(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[̂ݍXV */
    long int w1;
    unsigned char b,c;
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=ipb->hd[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    ipb->hd[6]=b;
}

int nrzm2nrzl(int nrzmold, int nrzmnow)
// conversion from NRZM to NRZL
{
    //int k2;
    //k2=1;
    //if(nrzmnow == nrzmold) k2=0;
    //return k2;
    // fast version
    return nrzmold ^ nrzmnow;     //2004-09-06
}

void nrzm2nrzl_a(int *nrzm, int *nrzl, int nkai)
{
    int m,k1;
    k1=0;
    for(m=0; m< nkai; m++){
        nrzl[m]=nrzm2nrzl(k1,nrzm[m]);
        k1=nrzm[m];
    }
}

int rvsint(int i4)
{
    // reverse integer narabi
    int iw, iwork, ipick,i;
    iw=i4;
    iwork=0;
    for(i=0; i<32; i++){
        iwork=iwork << 1;
        ipick=iw & 0x1;
        iwork= ipick | (iwork & 0xFFFFFFFE);
        iw=iw >> 1;
    }
    return iwork;
}

int rvsint_nbit(int i4, int nbit)
{
    // reverse integer narabi 
    // lower nbit only
    int iw, iwork, ipick,i;
    iw=i4;
    iwork=0;
    for(i=0; i<nbit; i++){
        iwork=iwork << 1;
        ipick=iw & 0x1;
        iwork= ipick | (iwork & 0xFFFFFFFE);
        iw=iw >> 1;
    }
    return iwork;
}

void print_intbit(int i4)
{
    // print 32 bit 0 1 data
    int iwork, ipick,i;
    iwork=i4;
    for(i=0; i<32; i++){
        ipick=iwork & 0x1;
        printf("%d ",ipick);
        iwork=iwork >> 1;
    }
    printf("\n");
}


int conv4byte2int(unsigned char *b4)
{
    int i, k, iwork, iw2, ipick, flag, pflag;
    unsigned char iw;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    pflag=0;  // flag to change parity position   not used yet
    for(i=0; i<4; i++){
       iw=b4[i];
       iw2=(int)iw;
       for(k=0; k<8; k++){
           ipick=iw2 & 0x1;

           if(flag==0){
               iwork=iwork >> 1;
               ipick =ipick << 31;
               iwork= ipick | (iwork & 0x7FFFFFFF);
           } else {
               iwork=iwork << 1;
               ipick =ipick & 0x1;
               iwork= ipick | (iwork & 0xFFFFFFFE);
           }
           iw2=iw2 >> 1;
       }
    }
    return iwork;
}


int conv2int(int *idat36, int numch)
{
    //int idat36[36];   // 36 bit data (including parity data) 
    //int i4;   return 
    int i, iwork, ipick, flag, pflag;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    pflag=0;  // flag to change parity position   not used yet
    for(i=0; i<36; i++){
       ipick=idat36[i];
       if(numch ==1){
           //   data sturucture in idat36
           //   01234567p01234567p01234567p01234567p
           //if((i % 9) != 0) {     // to remove parity bit
          if((i % 9) != 8) {     // to remove parity bit
            //  if(flag==0){
                 iwork=iwork >> 1;
                 ipick =ipick << 31;
                 iwork= ipick | (iwork & 0x7FFFFFFF);
            //  } else {
            //     iwork=iwork << 1;
            //     ipick =ipick & 0x1;
            //     iwork= ipick | (iwork & 0xFFFFFFFE);
            //  }
          }
       } else {     // numch =4 mode
          //   data sturucture in idat36
          //   00001111222233334444555566667777pppp
          if( i < 32 ) {     // to remove parity bit
            //  if(flag==0){
                 iwork=iwork >> 1;
                 ipick =ipick << 31;
                 iwork= ipick | (iwork & 0x7FFFFFFF);
            //  } else {
            //     iwork=iwork << 1;
            //     ipick =ipick & 0x1;
            //     iwork= ipick | (iwork & 0xFFFFFFFE);
            //  }
          }
       }
    }
    return iwork;
}

int conv2intn(int *idat36, int numch)  //nonparity version
{
    //int idat[36];   // 36 bit data (including parity data) 
    //int i4;   return 
    int i, iwork, ipick, flag, pflag;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    pflag=0;  // flag to change parity position   not used yet
    //for(i=0; i<36; i++){
    for(i=0; i<32; i++){
       ipick=idat36[i];
       if(numch ==1){
           //if((i % 9) != 0) {     // to remove parity bit
         // if((i % 9) != 8) {     // to remove parity bit
              if(flag==0){
                 iwork=iwork >> 1;
                 ipick =ipick << 31;
                 iwork= ipick | (iwork & 0x7FFFFFFF);
              } else {
                 iwork=iwork << 1;
                 ipick =ipick & 0x1;
                 iwork= ipick | (iwork & 0xFFFFFFFE);
              }
         // }
       } else {     // numch =4 mode
          if( i < 32 ) {     // to remove parity bit
              if(flag==0){
                 iwork=iwork >> 1;
                 ipick =ipick << 31;
                 iwork= ipick | (iwork & 0x7FFFFFFF);
              } else {
                 iwork=iwork << 1;
                 ipick =ipick & 0x1;
                 iwork= ipick | (iwork & 0xFFFFFFFE);
              }
          }
       }
    }
    return iwork;
}



void conv2sint(int *tdat, short int *sint) 
{
    short int iwork;
    int ipick, m, k, ikai;
    for(k=0; k < 4; k++){
        ikai=0;
        iwork=0;
        for(m=0; m<18; m++){
            //  if((m % 9)!= 8){
            if((m % 9)!= 0){
                ikai++;
                ipick=tdat[k*18+m];
                iwork=(short int)ipick | (iwork & 0x7FFFE);
                if(ikai != 16){
                    iwork=iwork << 1;
                }
            }
        }
        sint[k]=iwork;
    }
}

void conv2time(int *idat, int *itime) 
{
    int i1,i2,i3;
    // 1 digit year get
    itime[0]=(idat[0] >> 28) & 0xF;
    // total day get
    i1=(idat[0] >> 24) & 0xF;
    i2=(idat[0] >> 20) & 0xF;
    i3=(idat[0] >> 16) & 0xF;
    itime[1]=i1*100+i2*10+i3;
    // hour get
    i1=(idat[0] >> 12) & 0xF;
    i2=(idat[0] >> 8) & 0xF;
    itime[2]=i1*10+i2;
    //minute get
    i1=(idat[0] >> 4) & 0xF;
    i2=idat[0] & 0xF;
    itime[3]=i1*10+i2;
    // sec get 
    i1=(idat[1] >> 28) & 0xF;
    i2=(idat[1] >> 24) & 0xF;
    itime[4]=i1*10+i2;
    // msec get 
    i1=(idat[1] >> 20) & 0xF;
    i2=(idat[1] >> 16) & 0xF;
    i3=(idat[1] >> 12) & 0xF;
    itime[5]=i1*100+i2*10+i3;
}

void conv2time_vlba(int *idat, int *itime) 
// VLBA mode time info decode       2003-10-28
{
    int i1,i2,i3,i4,i5, mjd3, seconds, msec10;  // msec10=msec*10
    // lower 3 digit mjd get
    i1=(idat[0] >> 28) & 0xF;
    i2=(idat[0] >> 24) & 0xF;
    i3=(idat[0] >> 20) & 0xF;
    mjd3=i1*100+i2*10+i3;

    // integer seconds in day get
    i1=(idat[0] >> 16) & 0xF;
    i2=(idat[0] >> 12) & 0xF;
    i3=(idat[0] >>  8) & 0xF;
    i4=(idat[0] >>  4) & 0xF;
    i5= idat[0] & 0xF;
    seconds=i1*10000+i2*1000+i3*100+i4*10+i5;


    // msec*10 get
    i1=(idat[1] >> 28) & 0xF;
    i2=(idat[1] >> 24) & 0xF;
    i3=(idat[1] >> 20) & 0xF;
    i4=(idat[1] >> 16) & 0xF;
    msec10=i1*1000+i2*100+i3*10+i4;

    // 1 digit year set to -1 to notice this is VLBA format
    itime[0]=-1;
    // 3 digit MJD set
    itime[1]=mjd3;
    // hour 
    itime[2]=seconds/3600;
    //minute
    itime[3]=(seconds % 3600)/60;
    // sec get 
    itime[4]=seconds % 60;
    // msec10 get 
    itime[5]=msec10;
}

void conv2time2(int *tdat, int *itime) 
{
    int iwork, idat[2];

    int ipick, m, k, ikai;
    for(k=0; k < 2; k++){
        ikai=0;
        iwork=0;
        for(m=0; m< 36; m++){
            if((m % 9)!= 8){
            //if((m % 9)!= 0){
                ikai++;
                ipick=tdat[k*36+m];
                iwork=ipick | (iwork & 0xFFFFFFFE);
                if(ikai != 32){
                    iwork=iwork << 1;
                }
            }
        }
        idat[k]=iwork;
    }
    conv2time(idat,itime);
}

void conv2time2n(int *tdat, int *itime) //non parity data
{
    int iwork, idat[2];

    int ipick, m, k, ikai;
    for(k=0; k < 2; k++){
        ikai=0;
        iwork=0;
        for(m=0; m< 32; m++){
            //if((m % 9)!= 8){
                ikai++;
                //ipick=tdat[k*36+m];
                ipick=tdat[k*32+m];
                iwork=ipick | (iwork & 0xFFFFFFFE);
                if(ikai != 32){
                    iwork=iwork << 1;
                }
            //}
        }
        idat[k]=iwork;
    }
    conv2time(idat,itime);
}

void conv2time_full(int *tdat, int *itime, int parity, int vlba) 
//  parity = 1 for with parity mode
//  vlba =1 for VLBA format
//  2003-10-28
{
    int iwork, idat[2];
    int ipick, m, k, ikai;

    for(k=0; k < 2; k++){
        ikai=0;
        iwork=0;
        if(parity == 1){ // with parity
           for(m=0; m< 36; m++){
              if((m % 9)!= 8){
              //if((m % 9)!= 0){
                 ikai++;
                 ipick=tdat[k*36+m];
                 iwork=ipick | (iwork & 0xFFFFFFFE);
                 if(ikai != 32){
                    iwork=iwork << 1;
                 }  
              } 
           } 
        } else {  // non-parity
           for(m=0; m< 32; m++){
                ikai++;
                ipick=tdat[k*32+m];
                iwork=ipick | (iwork & 0xFFFFFFFE);
                if(ikai != 32){
                    iwork=iwork << 1;
                }
           } 
        }
        idat[k]=iwork;
    }
    //printf("%08X %08X\n",idat[0],idat[1]); //debug
    if(vlba==1){  // VLBA mode
        conv2time_vlba(idat,itime);
    } else {
        conv2time(idat,itime);
    }
}

int bcd4digit2int(int bcd4digit)
{   // convert 4 digit BCD (16bit) to integer number
    int i,k,iw[4];
    k=bcd4digit;
    for(i=0; i<4; i++){
        iw[i]=k & 0xF;
        k=k >> 4;
    }
    i=iw[3]*1000+iw[2]*100+iw[1]*10+iw[0];
    return i;
}

int bcd2digit2int(int bcd2digit)
{   // convert 2 digit BCD (16bit) to integer number
    int i,k,iw[2];
    k=bcd2digit;
    for(i=0; i<2; i++){
        iw[i]=k & 0xF;
        k=k >> 4;
    }
    i=iw[1]*10+iw[0];
    return i;
}


void parseMK4aux(int *header, mk5_aux_t *aux)
/*  Parse Mark4 AUX field data in the header */ 
{
     int i,iw1,iw2,iw;

     iw1=rvsint(header[0]);
     iw2=rvsint(header[1]);
     iw =iw1 >> 16;
     aux->h_pos1 = bcd4digit2int(iw);
     iw =iw1 & 0xFFFF;
     aux->h_pos2 = bcd4digit2int(iw);
     aux->h_stack= ((iw2 >> 30 ) & 0x3)+1;
     iw       = (iw2 >> 24 ) & 0x3F;
     aux->trk = bcd2digit2int(iw);
     aux->adid   = (iw2 >> 16 ) & 0xFF;
     iw=iw2 >> 8;
     for(i=0; i<8; i++){
         aux->eflag[i]=iw & 0x1;
         iw=iw >> 1;
     }
     aux->id     = iw2 & 0xFF;
}

void get_trk_from_header(int *header, int *trk, int *adid)
/*  Get Track number and AD identifier from header  */ 
{
    /* so far Mark IV mode only */
    /* VLBA mode is not supported */
     int iw2,iw;

     iw2=rvsint(header[1]);
     iw       = (iw2 >> 24 ) & 0x3F;
     *trk = bcd2digit2int(iw);
     *adid   = (iw2 >> 16 ) & 0xFF;
}

void get_trk_from_header_a(int *header, int *trk, int *adid, int vlba)
/*  Get Track number and AD identifier from header  */ 
// 2004-09-03
{
    /* both Mark IV and VLBA modes are supported */
    // not completed yet
     int iw2,iw;
     int gr, tk;
     //   Gr#(0-4), Trk#(0-7) in VLBA AUX field
     //   vttabl[Gr#][Trk#] -- Track# (0-35)
     int vttabl[][8]=  {{ 3, 5, 7, 9,11,13,15,17},   //Gr# 0
                        {19,21,23,25,27,29,31,33},    //Gr# 1
                        { 2, 4, 6, 8,10,12,14,16},    //Gr# 2
                        {18,20,22,24,26,28,30,32},    //Gr# 3
                        { 1,35, 0,34,-1,-1,-1,-1}};   //Gr# 4  -1 means "not assigned"
     /*  AUX DATA FIELD
         64bit

        Mark-IV
           aaaabbbbrrssttuu        (in 16 BCD dgits of 4-bits each)
                 rr   bits 5-0  Track number

        VLBA
           SSRHPPPPTGBSCD00      (note:  TGBSCD are bit-reversed)
                 T    Tr#
                 G    Gr#
     */
              
     iw2=rvsint(header[1]);
     if(vlba==1){     // VLBA mode
         iw = (iw2 >> 28) & 0xF;
         tk=rvsint_nbit(iw,4);  //  lower 4 bit reverse
         iw = (iw2 >> 24) & 0xF;
         gr=rvsint_nbit(iw,4);  //  lower 4 bit reverse
         if((tk >=0 && tk < 8) && (gr >=0 && gr < 5)){
              *trk=vttabl[gr][tk];
         } else {
             *trk =-1;
         }
         *adid=0;
     } else {       // Mark-IV mode
        iw       = (iw2 >> 24 ) & 0x3F;
        *trk = bcd2digit2int(iw);
        *adid   = (iw2 >> 16 ) & 0xFF;
     }
}

void parseMK5aux(int *header, int vlba, mk5_aux_t *mk5aux)
/*  Get Track information from header  
  [in]  
     header  --  int * 5 array
     vlba    --  0: Mark-IV mode   1: VLBA mode
  [out]
     mk5aux  --  mk5 aux type structual variable

     trk  -----  track number
     hs   -----  head stack #
     foid  ----  fan-out sub-channel number in Mark-IV mode
                   0,1,2,3
     smid  ----  sign-bit magnitude-bit id
                 0: sign bit,   1: magnitude bit
     ulid  ----  usb lsb id
                 0: USB,    1: LSB
     vcn   ----  video converter number
    
   2004-09-13
   2006-09-26
*/
{
    /* both Mark IV and VLBA modes are supported but not perfect*/
    // not completed yet
     int i,iw2,iw;
     int gr, tk;
     //   Gr#(0-4), Trk#(0-7) in VLBA AUX field
     //   vttabl[Gr#][Trk#] -- Track# (0-35)
     int vttabl[][8]=  {{ 3, 5, 7, 9,11,13,15,17},   //Gr# 0
                        {19,21,23,25,27,29,31,33},    //Gr# 1
                        { 2, 4, 6, 8,10,12,14,16},    //Gr# 2
                        {18,20,22,24,26,28,30,32},    //Gr# 3
                        { 1,35, 0,34,-1,-1,-1,-1}};   //Gr# 4  -1 means "not assigned"
     /*  AUX DATA FIELD
         64bit

        Mark-IV
           aaaabbbbrrssttuu        (in 16 BCD dgits of 4-bits each)
                 rr   bits 5-0  Track number

        VLBA
           SSRHPPPPTGBSCD00      (note:  TGBSCD are bit-reversed)
                 T    Tr#
                 G    Gr#
     */
              


     iw2=rvsint(header[1]);
     if(vlba==1){     // VLBA mode
         iw = (iw2 >> 28) & 0xF;
         tk=rvsint_nbit(iw,4);  //  lower 4 bit reverse
         iw = (iw2 >> 24) & 0xF;
         gr=rvsint_nbit(iw,4);  //  lower 4 bit reverse
         if((tk >=0 && tk < 8) && (gr >=0 && gr < 5)){
              mk5aux->trk=vttabl[gr][tk];
         } else {
              mk5aux->trk =-1;
         }
         for(i=0; i<8; i++) mk5aux->eflag[i]=0;

         iw = (iw2 >> 16) & 0xF;            // add on 2006-09-26
         mk5aux->vcn= rvsint_nbit(iw,4) + 1; // add on 2006-09-26

     } else {       // Mark-IV mode
        iw= iw2 & 0xFF;
        mk5aux->id=iw;
        iw2 >>= 8;

        for(i=0; i<8; i++){
            mk5aux->eflag[i]= iw2 & 0x1;
            iw2 >>= 1;
        }
        mk5aux->vcn= ( iw2 & 0xF ) + 1;
        iw2 >>= 4;
        mk5aux->uorlid=iw2 & 0x1;
        iw2 >>= 1;
        mk5aux->sormid=iw2 & 0x1;
        iw2 >>= 1;
        mk5aux->foid=iw2 & 0x3;
        iw2 >>= 2;
        iw = iw2 & 0x3F;
        mk5aux->trk = bcd2digit2int(iw);
        iw2 >>= 6;
        mk5aux->h_stack = ( iw2 & 0x3 ) + 1 ;

     }
}

void header2int(int *head180, int *head5)
{
     int sync[36];
     int i,k;
       /* convert header to int data */
     for(i=0; i<5; i++){
         for(k=0; k<36; k++){
               sync[k]=head180[i*36+k];
         }
           //header[i]=conv2int(*(head+i*36));
          head5[i]=conv2int(sync,1);
     }
}

int read_ndata(FILE *fin, int bitpos, int ndata, int *idata, int *ko)
/*
    read requested number of data at bit position including parity bit
	decode is made

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	int ibuf[1024];
	int inum=1024;
	int iread, nkai, hasu, iwork, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;

    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( ibuf, sizeof( int ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              iwork= ibuf[i] >> bitpos;
              ipick= iwork & 0x1;
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( ibuf, sizeof( int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           iwork= ibuf[i] >> bitpos;
           ipick= iwork & 0x1;
           //know=nrzm2nrzl(kold,ipick);
           know=kold ^ ipick;    // NRZM -> NRZL
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndatas(FILE *fin, int bitpos, int ndata, int *idata, int *ko)
/*
    read requested number of data at bit position including parity bit
	decode is made
     
       16bit-woed mode and NRZL

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	short int ibuf[1024];
	int inum=1024;
	int iread, nkai, hasu, iwork, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;

    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( ibuf, sizeof( short int ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              iwork= ibuf[i] >> bitpos;
              ipick= iwork & 0x1;
              //know=nrzm2nrzl(kold,ipick);
              know=ipick;
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( ibuf, sizeof( short int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           iwork= ibuf[i] >> bitpos;
           ipick= iwork & 0x1;
           //know=nrzm2nrzl(kold,ipick);
           know=ipick;
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata_np(FILE *fin, int bitpos, int ndata, int *idata, int *ko, int ntrack)
/*   2003-07-12
    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	//int ibuf[1024];
	//int ibuf[2048];  //2003-10-10
	unsigned int ibuf[2048];  //2003-10-11
    unsigned short *sbuf;
    unsigned char *cbuf;
    int64_t *lbuf;  //2003-10-10
	int inum=1024;
	//int iread, nkai, hasu, iwork, iwork1,ipick, know;
	int iread, nkai, hasu, ipick, know;
    int64_t iwork1;
	int kold;
	int n,i,icount;

    sbuf=(unsigned short *)ibuf; 
    cbuf=(unsigned char *)ibuf;
    lbuf=(int64_t *)ibuf;
	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
           if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), inum, fin );
           } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), inum, fin );
           } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), inum, fin );
           } else {
              iread = fread( lbuf, sizeof( int64_t ), inum, fin );
           }
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              if(ntrack==8) iwork1=(int64_t)cbuf[i];
              else if(ntrack==16) iwork1=(int64_t)sbuf[i];
              else if(ntrack==32) iwork1=(int64_t)ibuf[i];
              else iwork1=lbuf[i];
              //iwork= iwork1 >> bitpos;
              ipick= (int)((iwork1 >> bitpos) & 0x1);
              //know=nrzm2nrzl(kold,ipick);
              know=ipick;
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), hasu, fin );
        } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), hasu, fin );
        } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), hasu, fin );
        } else {
              iread = fread( lbuf, sizeof( int64_t ), hasu, fin );
        }
        //iread = fread( ibuf, sizeof( short int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           if(ntrack==8) iwork1=(int64_t)cbuf[i];
           else if(ntrack==16) iwork1=(int64_t)sbuf[i];
           else if(ntrack==32) iwork1=(int64_t)ibuf[i];
           else iwork1=lbuf[i];
           //iwork= iwork1 >> bitpos;
           ipick= (int)((iwork1 >> bitpos) & 0x1);
           //know=nrzm2nrzl(kold,ipick);
           know=ipick;
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}


int read_ndata_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int ntrack, int nrzm)
/*   2003-10-30
    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	//int ibuf[1024];
	//int ibuf[2048];  //2003-10-10
	unsigned int ibuf[2048];  //2003-10-11
    unsigned short *sbuf;
    unsigned char *cbuf;
    int64_t *lbuf;  //2003-10-10
	int inum=1024;
	//int iread, nkai, hasu, iwork, iwork1,ipick, know;
	int iread, nkai, hasu, ipick, know;
    int64_t iwork1;
	int kold;
	int n,i,icount;

    sbuf=(unsigned short *)ibuf; 
    cbuf=(unsigned char *)ibuf;
    lbuf=(int64_t *)ibuf;
	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
           if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), inum, fin );
           } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), inum, fin );
           } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), inum, fin );
           } else {
              iread = fread( lbuf, sizeof( int64_t ), inum, fin );
           }
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              if(ntrack==8) iwork1=(int64_t)cbuf[i];
              else if(ntrack==16) iwork1=(int64_t)sbuf[i];
              else if(ntrack==32) iwork1=(int64_t)ibuf[i];
              else iwork1=lbuf[i];
              //iwork= iwork1 >> bitpos;
              ipick= (int)((iwork1 >> bitpos) & 0x1);
              if (nrzm==1) {
                  //know=nrzm2nrzl(kold,ipick);
                  know=kold ^ ipick;    // NRZM -> NRZL
              } else {
                  know=ipick;
              }
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), hasu, fin );
        } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), hasu, fin );
        } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), hasu, fin );
        } else {
              iread = fread( lbuf, sizeof( int64_t ), hasu, fin );
        }
        //iread = fread( ibuf, sizeof( short int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           if(ntrack==8) iwork1=(int64_t)cbuf[i];
           else if(ntrack==16) iwork1=(int64_t)sbuf[i];
           else if(ntrack==32) iwork1=(int64_t)ibuf[i];
           else iwork1=lbuf[i];
           //iwork= iwork1 >> bitpos;
           ipick= (int)((iwork1 >> bitpos) & 0x1);
           if(nrzm==1){
               //know=nrzm2nrzl(kold,ipick);
               know=kold ^ ipick;    // NRZM -> NRZL
           } else {
               know=ipick;
           }
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

/* for all Mk5 data */

int read_ndata8_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm)
/*   2004-09-01  read_ndata_r for ntrack=8
     This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    unsigned char cbuf[1024];
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( cbuf, sizeof( unsigned char ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)(((int)cbuf[i] >> bitpos) & 0x1);
              if (nrzm==1) {
                  //know=nrzm2nrzl(kold,ipick);
                  know=kold ^ ipick;    // NRZM -> NRZL
              } else {
                  know=ipick;
              }
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( cbuf, sizeof( unsigned char ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)(((int)cbuf[i] >> bitpos) & 0x1);
           if(nrzm==1){
               //know=nrzm2nrzl(kold,ipick);
               know=kold ^ ipick;    // NRZM -> NRZL
           } else {
               know=ipick;
           }
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata16_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm)
/*   2004-09-01  read_ndata_r for ntrack=16

    This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned short sbuf[1024]; 
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( sbuf, sizeof( unsigned short ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)(((int)sbuf[i] >> bitpos) & 0x1);
              if (nrzm==1) {
                  //know=nrzm2nrzl(kold,ipick);
                  know=kold ^ ipick;    // NRZM -> NRZL
              } else {
                  know=ipick;
              }
	          idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
       iread = fread( sbuf, sizeof( unsigned short ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)(((int)sbuf[i] >> bitpos) & 0x1);
           if(nrzm==1){
               //know=nrzm2nrzl(kold,ipick);
               know=kold ^ ipick;    // NRZM -> NRZL
           } else {
               know=ipick;
           }
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata32_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm)
/*   2004-09-01  read_ndata_r for ntrack=32

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned int ibuf[1024];  
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( ibuf, sizeof( unsigned int ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)((ibuf[i] >> bitpos) & 0x1);
              if (nrzm==1) {
                  //know=nrzm2nrzl(kold,ipick);
                  know=kold ^ ipick;    // NRZM -> NRZL
              } else {
                  know=ipick;
              }
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( ibuf, sizeof( unsigned int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)((ibuf[i] >> bitpos) & 0x1);
           if(nrzm==1){
               //know=nrzm2nrzl(kold,ipick);
               know=kold ^ ipick;    // NRZM -> NRZL
           } else {
               know=ipick;
           }
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata64_r(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko, int nrzm)
/*   2004-09-01  read_ndata_r for ntrack=64
    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    int64_t lbuf[1024];
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( lbuf, sizeof( int64_t ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)((lbuf[i] >> bitpos) & 0x1);
              if (nrzm==1) {
                  //know=nrzm2nrzl(kold,ipick);
                  know=kold ^ ipick;    // NRZM -> NRZL
              } else {
                  know=ipick;
              }
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( lbuf, sizeof( int64_t ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)((lbuf[i] >> bitpos) & 0x1);
           if(nrzm==1){
               //know=nrzm2nrzl(kold,ipick);
               know=kold ^ ipick;    // NRZM -> NRZL
           } else {
               know=ipick;
           }
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

/* for NRZM Mk5 data */

int read_ndata8_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko)
/*   2004-09-01  read_ndata_r for ntrack=8 NRZM data
     This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    unsigned char cbuf[1024];
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( cbuf, sizeof( unsigned char ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)(((int)cbuf[i] >> bitpos) & 0x1);
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( cbuf, sizeof( unsigned char ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)(((int)cbuf[i] >> bitpos) & 0x1);
           //know=nrzm2nrzl(kold,ipick);
           know=kold ^ ipick;    // NRZM -> NRZL
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata16_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko)
/*   2004-09-01  read_ndata_r for ntrack=16  NRZM data

    This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned short sbuf[1024]; 
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( sbuf, sizeof( unsigned short ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)(((int)sbuf[i] >> bitpos) & 0x1);
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
	          idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
       iread = fread( sbuf, sizeof( unsigned short ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)(((int)sbuf[i] >> bitpos) & 0x1);
           //know=nrzm2nrzl(kold,ipick);
           know=kold ^ ipick;    // NRZM -> NRZL
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata32_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko)
/*   2004-09-01  read_ndata_r for ntrack=32  NRZM data

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned int ibuf[1024];  
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( ibuf, sizeof( unsigned int ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)((ibuf[i] >> bitpos) & 0x1);
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( ibuf, sizeof( unsigned int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)((ibuf[i] >> bitpos) & 0x1);
           //know=nrzm2nrzl(kold,ipick);
           know=kold ^ ipick;    // NRZM -> NRZL
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}

int read_ndata64_m(FILE *fin, int bitpos, int ndata, int *idata, 
                 int *ko)
/*   2004-09-01  read_ndata_r for ntrack=64  NRZM data
    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    int64_t lbuf[1024];
	int inum=1024;
	int iread, nkai, hasu, ipick, know;
	int kold;
	int n,i,icount;

	icount=0;
	kold=*ko;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( lbuf, sizeof( int64_t ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              ipick= (int)((lbuf[i] >> bitpos) & 0x1);
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
			  idata[icount]=know;
			  kold=ipick;
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( lbuf, sizeof( int64_t ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           ipick= (int)((lbuf[i] >> bitpos) & 0x1);
           //know=nrzm2nrzl(kold,ipick);
           know=kold ^ ipick;    // NRZM -> NRZL
		   idata[icount]=know;
		   kold=ipick;
		   icount++;
		}
	}
	*ko=kold;
    return icount;
}


/* for NRZL Mk5 data */

int read_ndata8_l(FILE *fin, int bitpos, int ndata, int *idata) 
/*   2004-09-01  read_ndata_r for ntrack=8  NRZL data (decode not necessary)
     This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    unsigned char cbuf[1024];
	int inum=1024;
	int iread, nkai, hasu;
	int n,i,icount;

	icount=0;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( cbuf, sizeof( unsigned char ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              idata[icount]= (int)(((int)cbuf[i] >> bitpos) & 0x1);
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( cbuf, sizeof( unsigned char ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           idata[icount]= (int)(((int)cbuf[i] >> bitpos) & 0x1);
		   icount++;
		}
	}
    return icount;
}

int read_ndata16_l(FILE *fin, int bitpos, int ndata, int *idata)
/*   2004-09-01  read_ndata_r for ntrack=16  NRZL data (decode not necessary)

    This is faster than read_ndata_r

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned short sbuf[1024]; 
	int inum=1024;
	int iread, nkai, hasu;
	int n,i,icount;

	icount=0;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( sbuf, sizeof( unsigned short ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              idata[icount]= (int)(((int)sbuf[i] >> bitpos) & 0x1);
			  icount++;
			}
		}
	}
	if(hasu >0 ){
       iread = fread( sbuf, sizeof( unsigned short ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           idata[icount]= (int)(((int)sbuf[i] >> bitpos) & 0x1);
		   icount++;
		}
	}
    return icount;
}

int read_ndata32_l(FILE *fin, int bitpos, int ndata, int *idata) 
/*   2004-09-01  read_ndata_r for ntrack=32  NRZL data (decode not necessary)

    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
	unsigned int ibuf[1024];  
	int inum=1024;
	int iread, nkai, hasu;
	int n,i,icount;

	icount=0;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( ibuf, sizeof( unsigned int ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              idata[icount]= (int)((ibuf[i] >> bitpos) & 0x1);
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( ibuf, sizeof( unsigned int ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           idata[icount]= (int)((ibuf[i] >> bitpos) & 0x1);
		   icount++;
		}
	}
    return icount;
}

int read_ndata64_l(FILE *fin, int bitpos, int ndata, int *idata) 
/*   2004-09-01  read_ndata_r for ntrack=64  NRZL data (decode not necessary)
    read requested number of data at bit position including parity bit
	decode is made
     

   Return
    read_ndata -- valid data number
	if read_ndata < ndata,  reach EOF
*/
{
    int64_t lbuf[1024];
	int inum=1024;
	int iread, nkai, hasu;
	int n,i,icount;

	icount=0;

	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug bitpos ntrack %d %d\n",bitpos,ntrack);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
            iread = fread( lbuf, sizeof( int64_t ), inum, fin );
            if (iread ==0 ) { // EOF
                return icount;
			}
            for (i=0; i< iread; i++){
              idata[icount]= (int)((lbuf[i] >> bitpos) & 0x1);
			  icount++;
			}
		}
	}
	if(hasu >0 ){
        iread = fread( lbuf, sizeof( int64_t ), hasu, fin );
        if (iread ==0 ) { // EOF
           return icount;
		}
        for (i=0; i< iread; i++){
           idata[icount]= (int)((lbuf[i] >> bitpos) & 0x1);
		   icount++;
		}
	}
    return icount;
}

void set_idata(unsigned int *idata, int xmax, int track, int nbit)
/*
    set bit data in idata

    IN
       idata  
       xmax  ---  # of bits / track  
       track ---  Track# in idata starting from 0
       nbit  ---  bit position in a frame starting from 0


*/
{
    int tbpos, ipos, bpos;
    int bitsize=32;  // =sizeof(int)*8
    unsigned int btab[]={0x00000001,0x00000002,0x00000004,0x00000008,
                         0x00000010,0x00000020,0x00000040,0x00000080,
                         0x00000100,0x00000200,0x00000400,0x00000800,
                         0x00001000,0x00002000,0x00004000,0x00008000,
                         0x00010000,0x00020000,0x00040000,0x00080000,
                         0x00100000,0x00200000,0x00400000,0x00800000,
                         0x01000000,0x02000000,0x04000000,0x08000000,
                         0x10000000,0x20000000,0x40000000,0x80000000};
    tbpos=track*xmax+nbit;
    ipos=tbpos/bitsize;
    bpos=tbpos % bitsize;

    //idata[ipos]=(idata[ipos] & ~btab[bpos]) | btab[bpos] ;
    idata[ipos]=idata[ipos] | btab[bpos] ;

}


void reset_idata(unsigned int *idata, int xmax, int track, int nbit)
/*
    reset bit data in idata  (set 0)

    IN
       idata  
       xmax  ---  # of bits / track  
       track ---  Track# in idata starting from 0
       nbit  ---  bit position in a frame starting from 0


*/
{
    int tbpos, ipos, bpos;
    int bitsize=32;  // =sizeof(int)*8
    unsigned int btab[]={0x00000001,0x00000002,0x00000004,0x00000008,
                         0x00000010,0x00000020,0x00000040,0x00000080,
                         0x00000100,0x00000200,0x00000400,0x00000800,
                         0x00001000,0x00002000,0x00004000,0x00008000,
                         0x00010000,0x00020000,0x00040000,0x00080000,
                         0x00100000,0x00200000,0x00400000,0x00800000,
                         0x01000000,0x02000000,0x04000000,0x08000000,
                         0x10000000,0x20000000,0x40000000,0x80000000};
    tbpos=track*xmax+nbit;
    ipos=tbpos/bitsize;
    bpos=tbpos % bitsize;

    idata[ipos]= idata[ipos] & ~btab[bpos] ;

}


int get_idata(unsigned int *idata, int xmax, int track, int nbit)
/*
    set bit data in idata

    IN
       idata  
       xmax  ---  # of bits / track  
       track ---  Track# in idata starting from 0
       nbit  ---  bit position in a frame starting from 0


*/
{
    int tbpos, ipos, bpos;
    int bitsize=32;  // =sizeof(int)*8
    unsigned int btab[]={0x00000001,0x00000002,0x00000004,0x00000008,
                         0x00000010,0x00000020,0x00000040,0x00000080,
                         0x00000100,0x00000200,0x00000400,0x00000800,
                         0x00001000,0x00002000,0x00004000,0x00008000,
                         0x00010000,0x00020000,0x00040000,0x00080000,
                         0x00100000,0x00200000,0x00400000,0x00800000,
                         0x01000000,0x02000000,0x04000000,0x08000000,
                         0x10000000,0x20000000,0x40000000,0x80000000};
    tbpos=track*xmax+nbit;
    ipos=tbpos/bitsize;
    bpos=tbpos % bitsize;

    if((idata[ipos] & btab[bpos]) == btab[bpos]) {
        return 1;
    } else {
        return 0;
    }

}



int read_ndata_r_r(mk5_data_r_t *mk5r, int ndata)
/*   2004-08-31
   In
     mk5r
     ndata  ---  read requirement (number of bits in each track)
                 ndata <= 22500

   Return
*/
{
	//int ibuf[1024];
	//int ibuf[2048];  //2003-10-10
	unsigned int ibuf[2048];  //2003-10-11
    unsigned short *sbuf;
    unsigned char *cbuf;
    int64_t *lbuf;  //2003-10-10
	int inum=1024;
	//int iread, nkai, hasu, iwork, iwork1,ipick, know;
	int iread, nkai, hasu, ipick, know;
    int ntrack, nrzm, tot_track;
    int64_t iwork1;
	//int kold;
	int n,i,k;

    sbuf=(unsigned short *)ibuf; 
    cbuf=(unsigned char *)ibuf;
    lbuf=(int64_t *)ibuf;
	//icount=0;
	//kold=*ko;

    ntrack=mk5r->mk5info.ntrack;
    nrzm  =mk5r->mk5info.nrzm;
    tot_track=mk5r->tot_track;

    //printf("wmax = %d\n",mk5r->wmax);
    for(n=0; n<mk5r->wmax; n++) mk5r->workd[n]=0;  // reset workd
    for(n=0; n<tot_track; n++) mk5r->wp[n]=0;  // icount data
    
	nkai=ndata/inum;
	hasu=ndata % inum;
    
    //printf("Debug nrzm %d\n",nrzm);
    //printf("Debug ntrack tot_track %d %d\n",ntrack,tot_track);
    if(nkai > 0) {
		for(n=0; n<nkai; n++){
           if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), inum, mk5r->fin );
           } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), inum, mk5r->fin );
           } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), inum, mk5r->fin );
           } else {
              iread = fread( lbuf, sizeof( int64_t ), inum, mk5r->fin );
           }
            if (iread ==0 ) { // EOF
                return mk5r->wp[0];
			}
            for (i=0; i< iread; i++){
              if(ntrack==8) iwork1=(int64_t)cbuf[i];
              else if(ntrack==16) iwork1=(int64_t)sbuf[i];
              else if(ntrack==32) iwork1=(int64_t)ibuf[i];
              else iwork1=lbuf[i];
              //iwork= iwork1 >> bitpos;
              for(k=0; k< tot_track; k++){
                  ipick= (int)((iwork1 >> mk5r->bitpos[k]) & 0x1);
                  if (nrzm==1) {
                     //know=nrzm2nrzl(mk5r->ko[k],ipick);
                     know=mk5r->ko[k] ^ ipick;    // NRZM -> NRZL
                  } else {
                     know=ipick;
                  }
                  if(know==1) set_idata(mk5r->workd, mk5r->wmax, k, mk5r->wp[k]);
			      mk5r->ko[k]=ipick;
			      mk5r->wp[k]++;
              }
			}
		}
	}
	if(hasu >0 ){
        if(ntrack==8){
              iread = fread( cbuf, sizeof( unsigned char ), hasu, mk5r->fin );
        } else if (ntrack==16){
              iread = fread( sbuf, sizeof( unsigned short ), hasu, mk5r->fin );
        } else if (ntrack==32){
              iread = fread( ibuf, sizeof( unsigned int ), hasu, mk5r->fin );
        } else {
              iread = fread( lbuf, sizeof( int64_t ), hasu, mk5r->fin );
        }
        //iread = fread( ibuf, sizeof( short int ), hasu, fin );
        if (iread ==0 ) { // EOF
            return mk5r->wp[0];
		}
        for (i=0; i< iread; i++){
           if(ntrack==8) iwork1=(int64_t)cbuf[i];
           else if(ntrack==16) iwork1=(int64_t)sbuf[i];
           else if(ntrack==32) iwork1=(int64_t)ibuf[i];
           else iwork1=lbuf[i];
           //iwork= iwork1 >> bitpos;
           for(k=0; k< tot_track; k++){
               ipick= (int)((iwork1 >> mk5r->bitpos[k]) & 0x1);
               if(nrzm==1){
                  //know=nrzm2nrzl(mk5r->ko[k],ipick);
                  know=mk5r->ko[k] ^ ipick;    // NRZM -> NRZL
               } else {
                  know=ipick;
               }
               if(know==1) set_idata(mk5r->workd, mk5r->wmax, k, mk5r->wp[k]);
			   mk5r->ko[k]=ipick;
			   mk5r->wp[k]++;
           }
		}
	}
    //for(k=0; k< tot_track; k++) printf(" %d",mk5r->wp[k]);printf("\n"); //debug

    return mk5r->wp[0];
}

void init_mk5_dat(mk5_data_t *mk5, mk5tok5_info_t *mk5info)
/*   
    initialize  mk5_data type variable
*/
{
       int mk5data_mode; // Mk5 data mode
                         //  1: NRZM with Parity (NRZM->NRZL necessary)
                         //  2: NRZM without parity   (NRZM->NRZL necessary)
                         //  3: NRZL with parity
                         //  4: NRZL without parity
       /* Mark5 data mode set */
       mk5data_mode=4; //   default
       if(mk5info->nrzm==1){
           if(mk5info->parity==1) {
               mk5data_mode=1;
           } else {
               mk5data_mode=2;
           }
       } else {
           if(mk5info->parity==1) {
               mk5data_mode=3;
           } else {
               mk5data_mode=4;
           }
       }
       mk5->data_mode=mk5data_mode;  // flag to idendity data mode 
       //printf("mk5data_mode = %d\n",mk5data_mode);
       mk5->parity=mk5info->parity;
       mk5->nrzm=mk5info->nrzm;
	   mk5->fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate;       // sampling rate of this track
       mk5->vlba=mk5info->vlba; // flag to idendity VLBA or Mark-IV 
       mk5->bitsframe=mk5info->bitsframe;  // 20000 for Mark-IV 20160 for VLBA
       mk5->ntrack=mk5info->ntrack;  
}

int read_1frame(mk5_data_t *mk5)
/*

     

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
	 ko --   to decode NRZL
	 read_1frame -- valid data number in idata


*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync[36];
       int head[180];  // header 180bits including parity
	   int workd[22500];
	   int ndata=22500;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, kold, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread,ierr;

	   ierr=0;
       icount=0;
       imax=36;
       ihead=180;
       tc=-1;

       ikai=0;
       //kold=0;
       kold=mk5->ko;
	   bitpos=mk5->bitpos;
	   ndata=22500;
	   mk5->ndata=0;
       iread=read_ndata(mk5->fin, bitpos, ndata, workd, &kold);
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          iread=read_ndata(mk5->fin, bitpos, ndata, workd, &kold);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
	   for(i=0; i<icount; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i+nstart];
			   ih++;
		   }
           if((m % 9) != 8) {     // to remove parity bit
			   mk5->idata[ii]=workd[i+nstart];
			   ii++;
		   }
		   m++;
	   }
	   nread=ndata-icount;
       iread=read_ndata(mk5->fin, bitpos, nread, workd, &kold);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i];
			   ih++;
		   }
           if((m % 9) != 8) {     // to remove parity bit
			   mk5->idata[ii]=workd[i];
			   ii++;
		   }
		   m++;
	   }
       // now idata contains full one frame without parity
       //printf("debug ih,ii,m %d %d %d\n",ih,ii,m);
       mk5->ko=kold;
       /* convert header to int data */
       for(i=0; i<5; i++){
           for(k=0; k<36; k++){
               sync[k]=head[i*36+k];
           }
           mk5->header[i]=conv2int(sync,1);
       }
	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       conv2time(idat,itime);
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}

int read_1frame_np(mk5_data_t *mk5)  // for non-parity version
/*


  IN
    mk5
    ntrack
     

  Return
     mk5->itime[6]  --  Time at BOFRAME
	 mk5->header[5] --  Header at BOFRAME
	 mk5->idata[20160] -- data stream  1 frame
                       Mark-IV  : 20000
                       VLBA format : 20160
     


*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int sync[36];
       int head[180];  // header 180bits including parity
	   int workd[22500];
	   int ndata=22500;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, kold, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread,ierr;
       int ntrack;

	   ierr=0;
       icount=0;
       /* non parity */
       imax=32;
       ihead=160;
       /* with parity */
       //imax=36;
       //ihead=180;

       tc=-1;

       ikai=0;
       //kold=0;
       kold=mk5->ko;
	   bitpos=mk5->bitpos;
       ntrack=mk5->ntrack;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
	   mk5->ndata=0;
       iread=read_ndata_np(mk5->fin, bitpos, ndata, workd, &kold, ntrack); //2003-07-12
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          iread=read_ndata_np(mk5->fin, bitpos, ndata, workd, &kold,ntrack);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
	   for(i=0; i<icount; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i+nstart];
			   ih++;
		   }
           //if((m % 9) != 8) {     // to remove parity bit
			   mk5->idata[ii]=workd[i+nstart];
			   ii++;
		   //}
		   m++;
	   }
	   nread=ndata-icount;
       iread=read_ndata_np(mk5->fin, bitpos, nread, workd, &kold,ntrack);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i];
			   ih++;
		   }
           //if((m % 9) != 8) {     // to remove parity bit
			   mk5->idata[ii]=workd[i];
			   ii++;
		   //}
		   m++;
	   }
       // now idata contains full one frame without parity
       //printf("debug ih,ii,m %d %d %d\n",ih,ii,m);
       mk5->ko=kold;
       /* convert header to int data */
       for(i=0; i<5; i++){
           //for(k=0; k<36; k++){
           for(k=0; k<32; k++){
              //sync[k]=head[i*36+k];
              sync[k]=head[i*32+k];
           }
           mk5->header[i]=conv2intn(sync,1);
       }
	   //debug
       //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5->header[i]));
	   //printf("\n");

	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       if(mk5->vlba == 1) {
          conv2time_vlba(idat,itime);
       } else {
          conv2time(idat,itime);
       }
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       //get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       get_trk_from_header_a(mk5->header, &mk5->aux.trk, &mk5->aux.adid,mk5->vlba);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}


int read_1frame_r(mk5_data_t *mk5)  // for all mode
/*
  2004-09-03

  IN
    mk5
     

  Return
     mk5->itime[6]  --  Time at BOFRAME
	 mk5->header[5] --  Header at BOFRAME
	 mk5->idata[20160] -- data stream  1 frame
                       Mark-IV  : 20000
                       VLBA format : 20160


*/
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36];
       //int head[180];  // header 180bits including parity
	   int workd[22680];  // to allow VLBA mode 1frame
	   int ndata;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, kold, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread,ierr;
       int ntrack,parity,nrzm;

	   ierr=0;
       icount=0;
       ntrack=mk5->ntrack;
       parity=mk5->parity;
       nrzm=mk5->nrzm;
       if(parity==1){
          imax=36;
          ihead=180;
          sync_pat=sync_pat1;
       } else {
          imax=32;
          ihead=160;
          sync_pat=sync_pat0;
       }
       tc=-1;

       ikai=0;
       //kold=0;
       kold=mk5->ko;
	   bitpos=mk5->bitpos;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
       if(parity == 1) ndata=(ndata*9)/8;
	   //ndata=22500;
	   //ndata=20000;
	   mk5->ndata=0;

       //printf("parity,nrzm,ndata %d %d %d\n",parity,nrzm,ndata);

       //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
       if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm); 
       if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
       if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
       if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
          if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
       //printf("Debug ndata,nstart,icount %d %d %d\n",ndata,nstart,icount);  //debug
	   for(i=0; i<icount; i++){
		   //if(ih<ihead) {
		   //   head[ih]=workd[i+nstart];
		   //   ih++;
		   //}
           if(parity==1){
              if((m % 9) != 8) {     // to remove parity bit
			     mk5->idata[ii]=workd[i+nstart];
			     ii++;
              }
           } else {
			  mk5->idata[ii]=workd[i+nstart];
			  ii++;
           }
		   m++;
	   }
	   nread=ndata-icount;
    //   iread=read_ndata_r(mk5->fin, bitpos, nread, workd, &kold, ntrack, nrzm);
       if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   //if(ih<ihead) {
		   //  head[ih]=workd[i];
		   //  ih++;
		   //}
           if(parity==1){
              if((m % 9) != 8) {     // to remove parity bit
			     mk5->idata[ii]=workd[i];
			     ii++;
              }
           } else {
			  mk5->idata[ii]=workd[i];
			  ii++;
           }
		   m++;
	   }
       mk5->ko=kold;
       /*  now idata contains full one frame without parity */
       /* convert header to int data */
       for(i=0; i<5; i++){
           //for(k=0; k<36; k++){
           //for(k=0; k<imax; k++){
           for(k=0; k<32; k++){
              //sync[k]=head[i*imax+k];
              sync[k]=mk5->idata[i*32+k];
           }
           mk5->header[i]=conv2intn(sync,1);
       }
	   //debug
       //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5->header[i]));
	   //printf("\n");

	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       if(mk5->vlba == 1) {
          conv2time_vlba(idat,itime);
       } else {
          conv2time(idat,itime);
       }
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       //get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       get_trk_from_header_a(mk5->header, &mk5->aux.trk, &mk5->aux.adid,mk5->vlba);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}

int read_1frame_m_p(mk5_data_t *mk5)  
/*
  for NRZM, with parity data   
     (NRZM->NRZL decode and parity removal are necessary)

  2004-09-01

  IN
    mk5
    mk5info
     

  Return
     mk5->itime[6]  --  Time at BOFRAME
	 mk5->header[5] --  Header at BOFRAME
	 mk5->idata[20160] -- data stream  1 frame
                       Mark-IV  : 20000
                       VLBA format : 20160


*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync[36];
       int head[180];  // header 180bits including parity
	   //int workd[22500];
	   int workd[22680];  // to allow VLBA mode 1frame
	   int ndata;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, kold, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread;
       int ntrack;

       icount=0;
       ntrack=mk5->ntrack;
       /* parity = 1 */
       imax=36;
       ihead=180;

       tc=-1;

       ikai=0;
       kold=mk5->ko;
	   bitpos=mk5->bitpos;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
       //if(parity == 1) ndata=(ndata*9)/8;
       ndata=(ndata*9)/8;  // parity is supposed
	   //ndata=22500;
	   //ndata=20000;
	   mk5->ndata=0;

       //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
       if(ntrack==8)  iread=read_ndata8_m(mk5->fin, bitpos, ndata, workd, &kold); 
       if(ntrack==16) iread=read_ndata16_m(mk5->fin, bitpos, ndata, workd, &kold);
       if(ntrack==32) iread=read_ndata32_m(mk5->fin, bitpos, ndata, workd, &kold);
       if(ntrack==64) iread=read_ndata64_m(mk5->fin, bitpos, ndata, workd, &kold);
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
          if(ntrack==8)  iread=read_ndata8_m(mk5->fin, bitpos, ndata, workd, &kold);
          if(ntrack==16) iread=read_ndata16_m(mk5->fin, bitpos, ndata, workd, &kold);
          if(ntrack==32) iread=read_ndata32_m(mk5->fin, bitpos, ndata, workd, &kold);
          if(ntrack==64) iread=read_ndata64_m(mk5->fin, bitpos, ndata, workd, &kold);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
       //printf("Debug ndata,nstart,icount %d %d %d\n",ndata,nstart,icount);
	   for(i=0; i<icount; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i+nstart];
			   ih++;
		   }
           /* remove parity bit */
           if((m % 9) != 8) {     // to remove parity bit
		     mk5->idata[ii]=workd[i+nstart];
		     ii++;
           }
		   m++;
	   }
	   nread=ndata-icount;
       if(ntrack==8)  iread=read_ndata8_m(mk5->fin, bitpos, nread, workd, &kold);
       if(ntrack==16) iread=read_ndata16_m(mk5->fin, bitpos, nread, workd, &kold);
       if(ntrack==32) iread=read_ndata32_m(mk5->fin, bitpos, nread, workd, &kold);
       if(ntrack==64) iread=read_ndata64_m(mk5->fin, bitpos, nread, workd, &kold);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i];
			   ih++;
		   }
           /* remove parity bit */
           if((m % 9) != 8) {     // to remove parity bit
		     mk5->idata[ii]=workd[i];
		     ii++;
           }
		   m++;
	   }
       mk5->ko=kold;
       /* Now idata does not include any parity bit
       /* convert header to int data */
       for(i=0; i<5; i++){
           for(k=0; k<32; k++){
              sync[k]=mk5->idata[i*32+k];
           }
           mk5->header[i]=conv2intn(sync,1);
       }

	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       if(mk5->vlba == 1) {
          conv2time_vlba(idat,itime);
       } else {
          conv2time(idat,itime);
       }
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       //get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       get_trk_from_header_a(mk5->header, &mk5->aux.trk, &mk5->aux.adid, mk5->vlba);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}

int read_1frame_l_np(mk5_data_t *mk5)  
/*
   for NRZL, without parity data  (any decode is not necessary)
  2004-09-01

  IN
    mk5
    mk5info
     

  Return
     mk5->itime[6]  --  Time at BOFRAME
	 mk5->header[5] --  Header at BOFRAME
	 mk5->idata[20160] -- data stream  1 frame
                       Mark-IV  : 20000
                       VLBA format : 20160


*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int sync[36];
       int head[180];  // header 180bits including parity
	   //int workd[22500];
	   int workd[22680];  // to allow VLBA mode 1frame
	   int ndata;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread;
       int ntrack;

	   //ierr=0;
       icount=0;
       ntrack=mk5->ntrack;
       /* parity = 0 */
       imax=32;
       ihead=160;

       tc=-1;

       ikai=0;
	   bitpos=mk5->bitpos;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
	   mk5->ndata=0;

       if(ntrack==8)  iread=read_ndata8_l(mk5->fin, bitpos, ndata, workd); 
       if(ntrack==16) iread=read_ndata16_l(mk5->fin, bitpos, ndata, workd);
       if(ntrack==32) iread=read_ndata32_l(mk5->fin, bitpos, ndata, workd);
       if(ntrack==64) iread=read_ndata64_l(mk5->fin, bitpos, ndata, workd);
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          if(ntrack==8)  iread=read_ndata8_l(mk5->fin, bitpos, ndata, workd);
          if(ntrack==16) iread=read_ndata16_l(mk5->fin, bitpos, ndata, workd);
          if(ntrack==32) iread=read_ndata32_l(mk5->fin, bitpos, ndata, workd);
          if(ntrack==64) iread=read_ndata64_l(mk5->fin, bitpos, ndata, workd);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
       //printf("Debug ndata,nstart,icount %d %d %d\n",ndata,nstart,icount);
	   for(i=0; i<icount; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i+nstart];
			   ih++;
		   }
  		   mk5->idata[ii]=workd[i+nstart];
 		   ii++;
		   m++;
	   }
	   nread=ndata-icount;
       if(ntrack==8)  iread=read_ndata8_l(mk5->fin, bitpos, nread, workd);
       if(ntrack==16) iread=read_ndata16_l(mk5->fin, bitpos, nread, workd);
       if(ntrack==32) iread=read_ndata32_l(mk5->fin, bitpos, nread, workd);
       if(ntrack==64) iread=read_ndata64_l(mk5->fin, bitpos, nread, workd);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   if(ih<ihead) {
			   head[ih]=workd[i];
			   ih++;
		   }
		   mk5->idata[ii]=workd[i];
		   ii++;
		   m++;
	   }
       // now idata contains full one frame data 
       //printf("debug ih,ii,m %d %d %d\n",ih,ii,m);
       /* convert header to int data */
       for(i=0; i<5; i++){
           //for(k=0; k<36; k++){
           for(k=0; k<imax; k++){
              //sync[k]=head[i*36+k];
              sync[k]=head[i*imax+k];
           }
           mk5->header[i]=conv2intn(sync,1);
       }
	   //debug
       //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5->header[i]));
	   //printf("\n");

	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       if(mk5->vlba == 1) {
          conv2time_vlba(idat,itime);
       } else {
          conv2time(idat,itime);
       }
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       //get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       get_trk_from_header_a(mk5->header, &mk5->aux.trk, &mk5->aux.adid, mk5->vlba);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}

int read_1frame_r_r(mk5_data_r_t *mk5r) // no more used any where
/*

     

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
                      including header for Mark-IV
                      without header for VLBA format
	 ko --   to decode NRZL
	 read_1frame_r -- valid data number in idata


*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int sync[36];
       //int head[180];  // header 180bits including parity
       int head[180*32];  // header 180bits including parity
	   //int workd[22500];
	   //int workd[22680];  // to allow VLBA mode 1frame
	   //int workd[22680];  // to allow VLBA mode 1frame
	   int ndata;
       int ma[32],iia[32],iha[32];

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih,iread, tc, imax, ikai, k, m,ihan,n;
       //int kold;
	   //int npos,nstart,isync,bitpos;
	   int npos,nstart,isync;
	   int icount,nread,ierr;
       int ntrack,parity,nrzm;


	   ierr=0;
       icount=0;
       ntrack=mk5r->mk5info.ntrack;
       parity=mk5r->mk5info.parity;
       nrzm=mk5r->mk5info.nrzm;
       if(parity==1){
          imax=36;
          ihead=180;
       } else {
          imax=32;
          ihead=160;
       }
       tc=-1;

       ikai=0;
       //kold=0;

       //kold=mk5->ko;
	   //bitpos=mk5->bitpos;
      // if(mk5r->mk5info.vlba == 1) {
      //     ndata=20160;
      // } else {
      //     ndata=20000;
      // }
       ndata=mk5r->mk5info.bitsframe;
       if(parity == 1) ndata=(ndata*9)/8;
	   //ndata=22500;
	   //ndata=20000;


	   //for(k=0; k<tot_track; k++) mk5[k].ndata=0;

       //iread=read_ndata_np(mk5->fin, bitpos, ndata, workd, &kold, ntrack); 
       iread=read_ndata_r_r(mk5r, ndata); 
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection for 1st track*/
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     //if(mk5[0].idata[k+i]!=sync_pat[i]) ihan=1;
		     if(get_idata(mk5r->workd, mk5r->wmax, 0, k+i) !=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=get_idata(mk5r->workd, mk5r->wmax, 0, k+i);
		  }

          //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm); //2003-10-30
          iread=read_ndata_r_r(mk5r, ndata); 
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=get_idata(mk5r->workd, mk5r->wmax, 0, k);
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(get_idata(mk5r->workd, mk5r->wmax, 0, k+i)!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(get_idata(mk5r->workd, mk5r->wmax, 0, k+i)!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   printf("Sync not detected in 2 frames!\n");
		   return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	  // m=0;
	  // ii=0;
	  // ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
       //printf("Debug ndata,nstart,icount %d %d %d\n",ndata,nstart,icount);
       //printf("Debug mk5r->tot_track %d\n",mk5r->tot_track);
       for(k=0; k< mk5r->tot_track; k++){
          m=0;
          ii=0;
          ih=0;
          //printf("  k= %d\n",k);
	      for(i=0; i<icount; i++){
		     if(ih<ihead) {
			     head[k*180+ih]=get_idata(mk5r->workd, mk5r->wmax, k, i+nstart);
			     ih++;
             } 
             //reset_idata(mk5r->idata, mk5r->xmax, k, ii);  // reset bit data at k, ii
             if(parity==1){
                if((m % 9) != 8) {     // to remove parity bit
                    if(get_idata(mk5r->workd, mk5r->wmax, k, i+nstart)==1){
                        set_idata(mk5r->idata, mk5r->xmax, k, ii);
                    } else {
                        reset_idata(mk5r->idata, mk5r->xmax, k, ii);
                    }
			       ii++;
                } 

             } else {
                if(get_idata(mk5r->workd, mk5r->wmax, k, i+nstart)==1){
                    set_idata(mk5r->idata, mk5r->xmax, k, ii);
                } else {
                    reset_idata(mk5r->idata, mk5r->xmax, k, ii);
                }
			    ii++;
             } 
		     m++;
          }
          ma[k]=m;
          iia[k]=ii;
          iha[k]=ih;
       }

       //printf("ma,iia,iha\n");
       //for(k=0; k< mk5r->tot_track; k++) printf(" %d",ma[k]); printf("\n");
       //for(k=0; k< mk5r->tot_track; k++) printf(" %d",iia[k]); printf("\n");
       //for(k=0; k< mk5r->tot_track; k++) printf(" %d",iha[k]); printf("\n");

	   nread=ndata-icount;
       //iread=read_ndata_np(mk5->fin, bitpos, nread, workd, &kold,ntrack);
       //iread=read_ndata_r(mk5->fin, bitpos, nread, workd, &kold, ntrack, nrzm); //2003-10-30
       iread=read_ndata_r_r(mk5r, nread); 
       //printf("Debug iread, nread %d %d\n",iread,nread);
	   if(iread < nread) {  // eof detection
	      return 0;
	   }

       for(k=0; k< mk5r->tot_track; k++){
           //printf("k= %d  tot_track= %d\n",k,mk5r->tot_track);
           m=ma[k];
           ii=iia[k];
           ih=iha[k];
           //printf("m,ii,ih %d %d %d\n",m,ii,ih);

	       for(i=0; i<nread; i++){
		       if(ih<ihead) {
			      head[k*180+ih]=get_idata(mk5r->workd, mk5r->wmax, k, i);
			      ih++;
               } 
               if(parity==1){
                  if((m % 9) != 8) {     // to remove parity bit
                     if(get_idata(mk5r->workd, mk5r->wmax, k, i)==1){
                        set_idata(mk5r->idata, mk5r->xmax, k, ii);
                     } else {
                        reset_idata(mk5r->idata, mk5r->xmax, k, ii);
                     } 
			         ii++;
                  } 
               } else {
                  if(get_idata(mk5r->workd, mk5r->wmax, k, i)==1){
                     set_idata(mk5r->idata, mk5r->xmax, k, ii);
                  } else {
                     reset_idata(mk5r->idata, mk5r->xmax, k, ii);
                  } 
			      ii++;
               } 
		       m++;
           }
           ma[k]=m;
           iia[k]=ii;
           iha[k]=ih;
           //printf("m,ii,ih %d %d %d\n",m,ii,ih);
           // now idata contains full one frame without parity
           //printf("debug ih,ii,m %d %d %d\n",ih,ii,m);
           //mk5->ko=kold;
           /* convert header to int data */
           for(i=0; i<5; i++){
               //for(k=0; k<36; k++){
               for(n=0; n<imax; n++){
                  //sync[k]=head[i*36+k];
                  sync[n]=head[k*180+i*imax+n];
               } 
               mk5r->header[5*k+i]=conv2intn(sync,1);
           } 
	       //debug
           //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5r->header[5*k+i]));
	       //printf("\n");


	       idat[0]=rvsint(mk5r->header[5*k+3]);
	       idat[1]=rvsint(mk5r->header[5*k+4]);
           //conv2time(mk5->header[3],mk5->itime);
           conv2time(idat,itime);
           for(i=0; i<6; i++) mk5r->itime[6*k+i]=itime[i];
	       mk5r->ndata[k]=ii;
           mk5r->dtime[k]=3600.0*(double)itime[2]+60.0*(double)itime[3]
              +(double)itime[4]+(double)itime[5]/1000.0;

           get_trk_from_header(mk5r->header+5*k, &mk5r->aux[k].trk, &mk5r->aux[k].adid);
           //printf("#3 k,tot_track %d %d\n",k,mk5r->tot_track); //debug
           //printf("TRK %d  ADID %d\n",mk5r->aux[k].trk,mk5r->aux[k].adid);// debug
           //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
           //printf("#4 k,tot_track %d %d\n",k,mk5r->tot_track); //debug
       }
       //printf("TRK ");
       //for(k=0; k<mk5r->tot_track; k++) printf(" %d",mk5r->aux[k].trk); printf("\n"); // debug
	   return iia[0];
}

int fast_atamadashi(mk5_data_t *mk5, int soffset)
/*
  2004-04-07

  Do just second atamadashi
  
  [in] -- sec offset 

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
	 ko --   to decode NRZL
	 fast_atamadashi -- valid data number in idata   0 means EOF


*/
{
	   int ibuf[1024];
	   int nbuf=1024;
	   int ndata=22500;   // sample per frame with parity  20000*9/8

       int itime[6];
       int i,m,nkai,iread,khan;
	   double fsampl,diffsec,dbits;

	   fsampl=mk5->fsampl;

       if(soffset==0 && mk5->itime[5] ==0 ) return mk5->ndata; // nothing to do  (already at just second position)

       for(m=0; m<6; m++) itime[m]=mk5->itime[m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }
       printf("Atamadashi time is %d/%03d %02d:%02d:%02d.%03d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       // fast forward
       diffsec=1.0-(double)(mk5->itime[5])/1000.0 + (double)soffset;
       dbits=diffsec*fsampl*9.0/8.0;   // 9/8 is to include parity bits
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-2;
       nkai=(int)((dbits-(double)ndata)/(double)nbuf)-20;  // to keep enough margin
	   //printf("diffsec,dbits,nkai %f %f  %d\n",diffsec,dbits,nkai);  //debug

       if(nkai > 0) {
            for (i=0; i< nkai; i++){
                iread = fread( ibuf, sizeof( int ), nbuf, mk5->fin );
                if (iread ==0 ) { // EOF
				   mk5->ndata=0;
				   return 0;
				}
			}
	   }

	   i=0;
	   while(1){
		   iread=read_1frame(&(*mk5));
           if (iread ==0 ) { // EOF
		       mk5->ndata=0;
			   return 0;
		   }
           khan=0;
           for(m=0; m<6; m++){
               //printf(" %d",mk5->itime[m]);
               if(itime[m] != mk5->itime[m]) khan=1;

           }
           //printf(" %d\n",mk5->ndata);
		   i++;
           if(khan ==0) break;    // atamadashi finish
	   }
	   //printf("Frame number %d\n",i);
	   return mk5->ndata;
}


int fast_atamadashi_np(mk5_data_t *mk5, int soffset) //non parity version
/*  2004-09-03

  Do just second atamadashi
  
  [in] -- sec offset 

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
	 ko --   to decode NRZL
	 fast_atamadashi -- valid data number in idata   0 means EOF


*/
{
	   //short int ibuf[1024];
	   //int ibuf[1024];
	   unsigned int ibuf[2048];
	   int nbuf=1024;
	   int ndata;   // sample per frame without parity  

       int itime[6];
       int i,m,nkai,iread,khan;
       int ntrack;
	   double fsampl,diffsec,dbits;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

	   fsampl=mk5->fsampl;
       ntrack=mk5->ntrack;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }

       if(soffset==0 && mk5->itime[5] ==0 ) return mk5->ndata; // nothing to do  (already at just second position)

       for(m=0; m<6; m++) itime[m]=mk5->itime[m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }
    // check
       printf("Atamadashi time is %d/%03d %02d:%02d:%02d.%03d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       // fast forward
       diffsec=1.0-(double)(mk5->itime[5])/1000.0 + (double)soffset;
       //dbits=diffsec*fsampl*9.0/8.0;   // 9/8 is to include parity bits
       dbits=diffsec*fsampl;   // 
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-2;
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-10;  //2003-07-16 for get margin
       nkai=(int)((dbits-(double)ndata)/(double)nbuf)-20;  //2004-04-07 for get more margin
	   //printf("diffsec,dbits,nkai %f %f  %d\n",diffsec,dbits,nkai);  //debug
       //printf("ntrack= %d\n",ntrack);
       if(nkai > 0) {
            for (i=0; i< nkai; i++){
                //iread = fread( ibuf, sizeof( short int ), nbuf, mk5->fin );
                if(ntrack==8){
                   iread = fread( cbuf, sizeof( unsigned char ), nbuf, mk5->fin );
                } else if(ntrack==16){
                   iread = fread( sbuf, sizeof( unsigned short ), nbuf, mk5->fin );
                } else if(ntrack==32){
                   iread = fread( ibuf, sizeof( unsigned int ), nbuf, mk5->fin );
                } else {
                   iread = fread( lbuf, sizeof( int64_t ), nbuf, mk5->fin );
                }
                if (iread ==0 ) { // EOF
				   mk5->ndata=0;
				   return 0;
				}
			}
	   }

	   i=0;
	   while(1){
		   iread=read_1frame_np(mk5);
           if (iread ==0 ) { // EOF
		       mk5->ndata=0;
			   return 0;
		   }
           /* if(i==0) printf("Data time is %d/%03d %02d:%02d:%02d.%03d\n",
               mk5->itime[0],mk5->itime[1],mk5->itime[2],
               mk5->itime[3],mk5->itime[4],mk5->itime[5]); */

           khan=0;
           for(m=0; m<6; m++){
               //printf(" %d",mk5->itime[m]);
               if(itime[m] != mk5->itime[m]) khan=1;

           }
           //printf(" %d\n",mk5->ndata);
		   i++;
           if(khan ==0) break;    // atamadashi finish
	   }
	   //printf("Frame number %d\n",i);
	   return mk5->ndata;
}


int fast_atamadashi_r(mk5_data_t *mk5, int soffset) 
// support both non parity and parity modes
/*  2004-09-03

  Do just second atamadashi
  
  [in] 
     soffset -- sec offset 

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
	 ko --   to decode NRZL
	 fast_atamadashi -- valid data number in idata   0 means EOF


*/
{
	   //short int ibuf[1024];
	   //int ibuf[1024];
	   unsigned int ibuf[2048];
	   int nbuf=1024;
	   //int ndata=22500;   // sample per frame with parity  20000*9/8
	   //int ndata=20000;   // sample per frame without parity  20000
       int ndata;

       int itime[6];
       int i,m,nkai,iread,khan;
       int ntrack,parity,nrzm;
	   double fsampl,diffsec,dbits;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

	   fsampl=mk5->fsampl;

       ntrack=mk5->ntrack;
       parity=mk5->parity;
       nrzm=mk5->nrzm;

       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
       if(parity == 1) ndata=(ndata*9)/8;

       //printf("debug ntrack,parity,nrzm %d %d %d\n",ntrack,parity,nrzm);

       if(soffset==0 && mk5->itime[5] ==0 ) return mk5->ndata; // nothing to do  (already at just second position)

       for(m=0; m<6; m++) itime[m]=mk5->itime[m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }

       //printf("              Atamadashi time : %d/%03d %02d:%02d:%02d.%03d\n",
       //        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       // fast forward
       diffsec=1.0-(double)(mk5->itime[5])/1000.0 + (double)soffset;
       if(parity==1){
           dbits=diffsec*fsampl*9.0/8.0;   // 9/8 is to include parity bits
       } else {
           dbits=diffsec*fsampl;   // 
       }
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-2;
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-10;  //2003-07-16 for get margin
       nkai=(int)((dbits-(double)ndata)/(double)nbuf)-20;  //2004-04-07 for get more margin
	   //printf("diffsec,dbits,nkai %f %f  %d\n",diffsec,dbits,nkai);  //debug
       //printf("ntrack= %d\n",ntrack);
       if(nkai > 0) {
            for (i=0; i< nkai; i++){
                //iread = fread( ibuf, sizeof( short int ), nbuf, mk5->fin );
                if(ntrack==8){
                   iread = fread( cbuf, sizeof( unsigned char ), nbuf, mk5->fin );
                } else if(ntrack==16){
                   iread = fread( sbuf, sizeof( unsigned short ), nbuf, mk5->fin );
                } else if(ntrack==32){
                   iread = fread( ibuf, sizeof( unsigned int ), nbuf, mk5->fin );
                } else {
                   iread = fread( lbuf, sizeof( int64_t ), nbuf, mk5->fin );
                }
                if (iread ==0 ) { // EOF
				   mk5->ndata=0;
				   return 0;
				}
			}
	   }

	   i=0;
	   while(1){
		   //iread=read_1frame_np(&(*mk5), ntrack);
		   iread=read_1frame_r(mk5);  
           if (iread ==0 ) { // EOF
		       mk5->ndata=0;
			   return 0;
		   }
           /* if(i==0) printf("Data time is %d/%03d %02d:%02d:%02d.%03d\n",
               mk5->itime[0],mk5->itime[1],mk5->itime[2],
               mk5->itime[3],mk5->itime[4],mk5->itime[5]); */

           khan=0;
           for(m=0; m<6; m++){
               //printf(" %d",mk5->itime[m]);
               if(itime[m] != mk5->itime[m]) khan=1;

           }
           //printf(" %d\n",mk5->ndata);
		   i++;
           if(khan ==0) break;    // atamadashi finish
	   }
	   //printf("Frame number %d\n",i);
	   return mk5->ndata;
}



int fast_atamadashi_r_r(mk5_data_r_t *mk5r, int soffset) 
// support both non parity and parity modes
/*  2004-09-01

  Do just second atamadashi
  
  [in] 
     soffset -- sec offset 

  Return
     itime[6]  --  Time at BOFRAME
	 header[5] --  Header at BOFRAME
	 idata[20000] -- data stream  1 frame
	 ko --   to decode NRZL
	 fast_atamadashi -- valid data number in idata   0 means EOF


*/
{
	   //short int ibuf[1024];
	   //int ibuf[1024];
	   unsigned int ibuf[2048];
	   int nbuf=1024;
	   //int ndata=22500;   // sample per frame with parity  20000*9/8
	   //int ndata=20000;   // sample per frame without parity  20000
       int ndata;

       int itime[6];
       int i,m,nkai,iread,khan;
       int ntrack,parity,nrzm;
       int kt;
	   double fsampl,diffsec,dbits;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

       printf("DebugB %d\n",mk5r->tot_track);

       for(i=0; i<mk5r->tot_track; i++) printf(" i= %d   %f\n",i,mk5r->fsampl[i]);  //debug
       // using first track data
       kt=0;
	   fsampl=mk5r->fsampl[kt];

       ntrack=mk5r->mk5info.ntrack;
       parity=mk5r->mk5info.parity;
       nrzm=mk5r->mk5info.nrzm;

       //ndata=20000;
       //if(parity==1) ndata=22500;

       ndata=mk5r->mk5info.bitsframe;
       //printf("debug ntrack,parity,nrzm,ndata %d %d %d %d\n",ntrack,parity,nrzm,ndata);
       printf("debug fsampl %f\n",fsampl);

       if(soffset==0 && mk5r->itime[6*kt+5] ==0 ) return mk5r->ndata[kt]; // nothing to do  (already at just second position)

       for(m=0; m<6; m++) itime[m]=mk5r->itime[6*kt+m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }
    // check
       printf("Atamadashi time is %d/%03d %02d:%02d:%02d.%03d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       // fast forward
       diffsec=1.0-(double)(mk5r->itime[6*kt+5])/1000.0;
       if(parity==1){
           dbits=diffsec*fsampl*9.0/8.0;   // 9/8 is to include parity bits
       } else {
           dbits=diffsec*fsampl;   // 
       }
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-2;
       //nkai=(int)((dbits-(double)ndata)/(double)nbuf)-10;  //2003-07-16 for get margin
       nkai=(int)((dbits-(double)ndata)/(double)nbuf)-20;  //2004-04-07 for get more margin
	   printf("diffsec,dbits,nkai %f %f  %d\n",diffsec,dbits,nkai);  //debug
       printf("ntrack= %d\n",ntrack); // debug
       if(nkai > 0) {
            for (i=0; i< nkai; i++){
                //iread = fread( ibuf, sizeof( short int ), nbuf, mk5->fin );
                if(ntrack==8){
                   iread = fread( cbuf, sizeof( unsigned char ), nbuf, mk5r->fin );
                } else if(ntrack==16){
                   iread = fread( sbuf, sizeof( unsigned short ), nbuf, mk5r->fin );
                } else if(ntrack==32){
                   iread = fread( ibuf, sizeof( unsigned int ), nbuf, mk5r->fin );
                } else {
                   iread = fread( lbuf, sizeof( int64_t ), nbuf, mk5r->fin );
                }
                if (iread ==0 ) { // EOF
				   mk5r->ndata[kt]=0;
				   return 0;
				}
			}
	   }

	   i=0;
	   while(1){
		   //iread=read_1frame_np(&(*mk5), ntrack);
		   iread=read_1frame_r_r(mk5r); 
           if (iread ==0 ) { // EOF
		       mk5r->ndata[kt]=0;
			   return 0;
		   }
           //if(i==0) printf("Data time is %d/%03d %02d:%02d:%02d.%03d\n",
            printf("Data time is %d/%03d %02d:%02d:%02d.%03d\n",
               mk5r->itime[6*kt+0],mk5r->itime[6*kt+1],mk5r->itime[6*kt+2],
               mk5r->itime[6*kt+3],mk5r->itime[6*kt+4],mk5r->itime[6*kt+5]); 

           khan=0;
           for(m=0; m<6; m++){
               //printf(" %d",mk5->itime[m]);
               if(itime[m] != mk5r->itime[6*kt+m]) khan=1;

           }
           //printf(" %d\n",mk5->ndata);
		   i++;
           if(khan ==0) break;    // atamadashi finish
	   }
	   //printf("Frame number %d\n",i);
	   return mk5r->ndata[kt];
}



int sync_detect(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko)
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead;
       int i,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr;

	   ierr=0;
       icount=0;
       imax=36;
       ihead=180;
       tc=-1;

       ikai=0;
       kold=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];
              iwork= ibuf[i] >> ibit;
              ipick= iwork & 0x1;
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
              sync[imax-1]=know;
              head[ihead-1]=know;
              kold=ipick;
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  if(tc >= 72){
                      tc=-1;
                      conv2time2(tdat, itime);
              //        for(m=0; m<6; m++){
              //            printf(" %d",itime[m]);
              //        }
              //        printf("\n");
                      goto exit2return;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
          }
		  icount+=iread;   // 
		  if(icount > 30000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	}
exit2return:
       *ipos=i;
       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       for(i=0; i<5; i++){
           for(k=0; k<36; k++){
               sync[k]=head[i*36+k];
           }
           //header[i]=conv2int(*(head+i*36));
           header[i]=conv2int(sync,1);
       }
	   return ierr;
}


int sync_detect2(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl)
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       int i,ii,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
	   double dw1,dw2,diffsec,rsampl;

	   ierr=0;
       icount=0;
       imax=36;
       ihead=180;
       tc=-1;

       

	   nloop=0;
       ikai=0;
       kold=0;
	   sampl=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];
              iwork= ibuf[i] >> ibit;
              ipick= iwork & 0x1;
              //know=nrzm2nrzl(kold,ipick);
              know=kold ^ ipick;    // NRZM -> NRZL
              //know=ipick;  //debug
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;
              head[ihead-1]=know;
              kold=ipick;
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  if(tc >= 72){
                      tc=-1;
                      conv2time2(tdat, itimew);
                      /* convert header to int data */
                      for(ii=0; ii<5; ii++){
                         for(k=0; k<36; k++){
                             sync[k]=head[ii*36+k];
						 }
                         //header[ii]=conv2int(*(head+ii*36));
                         headerw[ii]=conv2int(sync,1);
					  }
					  //debug
                      for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii]));
					  printf("\n");
					  if(nloop == 0) {
                          *ipos=i;
	                      sampl=0;
						  for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
					      //printf("debug1\n");
					  } 
                //      for(ii=0; ii<6; ii++){             //debug
                //          printf(" %d",itimew[ii]);     //debug
                //      }                               //debug
                //      printf("\n");                   //debug
				//	  printf("debug\n");
                      if(nloop==4) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
      //               printf("sync detected at %d\n",ikai);   //debug
      //               printf("%d  ",sampl);                   //debug
                   //  nframe++;

                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
			  sampl++;
          }
		  icount+=iread;   // 
		  if(icount > 200000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;
       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");
	   dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		   +(double)itimew[4]+((double)itimew[5])/1000.0;
	   dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		   +(double)itime[4]+((double)itime[5])/1000.0;
	   diffsec=dw2-dw1;
       rsampl=((double)sampl)*8.0/(9.0*diffsec);
	   /* check fsampl */
       *fsampl=rsampl;
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;
	   return ierr;
}

int sync_detect2s(FILE *fin, short int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl)  //nonparity data and 16bit-word version
{
       //int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
       //            1,1,1,1,1,1,1,1,0,
       //            1,1,1,1,1,1,1,1,0,
       //            1,1,1,1,1,1,1,1,0};
       int sync_pat[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       int i,ii,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
	   double dw1,dw2,diffsec,rsampl;

	   ierr=0;
       icount=0;
       //imax=36;
       imax=32;
       //ihead=180;
       ihead=160;
       tc=-1;

       

	   nloop=0;
       ikai=0;
       kold=0;
	   sampl=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( short int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];
              iwork= ibuf[i] >> ibit;
              ipick= iwork & 0x1;
              //know=nrzm2nrzl(kold,ipick);
              know=ipick;  // in case of non decoding
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;
              head[ihead-1]=know;
              kold=ipick;
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  //if(tc >= 72){
                  if(tc >= 64){
                      tc=-1;
                      conv2time2n(tdat, itimew);
                      /* convert header to int data */
                      for(ii=0; ii<5; ii++){
                         //for(k=0; k<36; k++){
                         for(k=0; k<32; k++){
                             //sync[k]=head[ii*36+k];
                             sync[k]=head[ii*32+k];
						 }
                         //header[ii]=conv2int(*(head+ii*36));
                         //headerw[ii]=conv2int(sync,1);
                         headerw[ii]=conv2intn(sync,1);
					  }
					  //debug
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii]));
					  //printf("\n");
					  if(nloop == 0) {
                          *ipos=i;
	                      sampl=0;
						  for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
					      //printf("debug1\n");
					  } 
              //        for(m=0; m<6; m++){
              //            printf(" %d",itime[m]);
              //        }
              //        printf("\n");
				//	  printf("debug\n");
                      if(nloop==4) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                     //printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
			  sampl++;
          }
		  icount+=iread;   // 
		  if(icount > 200000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;
       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");
	   dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		   +(double)itimew[4]+((double)itimew[5])/1000.0;
	   dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		   +(double)itime[4]+((double)itime[5])/1000.0;
	   diffsec=dw2-dw1;
       //rsampl=((double)sampl)*8.0/(9.0*diffsec);
       rsampl=((double)sampl)/diffsec;
	   /* check fsampl */
       *fsampl=rsampl;
       //printf("%f %f %f\n",rsampl, dw1,dw2);
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;
	   return ierr;
}

int sync_detect2new(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm)  // ver 2003-07-12
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       int i,ii,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
       int nbuf,iwork1, hmax;
       short int *sbuf;
       unsigned char *cbuf;
	   double dw1,dw2,diffsec,rsampl,fact;

       sbuf=(short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
	   ierr=0;
       icount=0;
       if(parity==1){  // with parity
           imax=36;
           ihead=180;
           hmax=72;
           sync_pat=sync_pat1;
           fact=8.0/9.0;
           nbuf=900;
       } else { // nonparity
           imax=32;
           hmax=64;
           ihead=160;
           sync_pat=sync_pat0;
           fact=1.0;
           nbuf=800;
       }
       if(n<nbuf) nbuf=n;

       tc=-1;

       

	   nloop=0;
       ikai=0;
       kold=0;
	   sampl=0;
       /* sync detection */
       while(1){
          if(bitposmax==8){
              iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
          } else if(bitposmax==16){
              iread = fread( sbuf, sizeof( short int ), nbuf, fin );
          } else {
              iread = fread( ibuf, sizeof( int ), nbuf, fin );
          }
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int)cbuf[i];
               else if(bitposmax==16) iwork1=(int)sbuf[i];
               else iwork1=ibuf[i];

              //iwork=ibuf[i];
              //iwork= ibuf[i] >> ibit;
              iwork= iwork1 >> ibit;
              ipick= iwork & 0x1;
              //if(nrzm==1) know=nrzm2nrzl(kold,ipick);
              if(nrzm==1) know=kold ^ ipick;    // NRZM -> NRZL
               else know=ipick;  // in case of NRZL mode
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;
              head[ihead-1]=know;
              kold=ipick;
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  //if(tc >= 72){
                  //if(tc >= 64){
                  if(tc >= hmax){
                      tc=-1;
                      if(parity==1) conv2time2(tdat, itimew);
                      else conv2time2n(tdat, itimew);
                      /* convert header to int data */
                      for(ii=0; ii<5; ii++){
                         //for(k=0; k<36; k++){
                         //for(k=0; k<32; k++){
                         for(k=0; k<imax; k++){
                            //sync[k]=head[ii*36+k];
                             //sync[k]=head[ii*32+k];
                             sync[k]=head[ii*imax+k];
						 }
                         //header[ii]=conv2int(*(head+ii*36));
                         //headerw[ii]=conv2int(sync,1);
                         if(parity==1) headerw[ii]=conv2int(sync,1);
                         else headerw[ii]=conv2intn(sync,1);
					  }
					  //debug
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii]));
					  //printf("\n");
					  if(nloop == 0) {
                          *ipos=i;
	                      sampl=0;
						  for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
					      //printf("debug1\n");
					  } 
              //        for(m=0; m<6; m++){
              //            printf(" %d",itime[m]);
              //        }
              //        printf("\n");
				//	  printf("debug\n");
                      if(nloop==3) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                     //printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
			  sampl++;
          }
		  icount+=iread;   // 
		  if(icount > 100000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;

       /**** added on 2003-07-12 ****/
       // header check again   if AUX field has sync pattern, it may be no sync
       if(header[1] == 0xffffffff ) { // no sync
           ierr=-1;
           return ierr;
       }
       /***** add end ****/

       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");
	   dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		   +(double)itimew[4]+((double)itimew[5])/1000.0;
	   dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		   +(double)itime[4]+((double)itime[5])/1000.0;
	   diffsec=dw2-dw1;
       //rsampl=((double)sampl)*8.0/(9.0*diffsec);
       rsampl=fact*((double)sampl)/diffsec;
	   /* check fsampl */
       *fsampl=rsampl;
       //printf("%f %f %f\n",rsampl, dw1,dw2);
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;
	   return ierr;
}

int sync_detect2new64(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm)  // ver 2003-07-12
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       //int i,ii,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
       int i,ii, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
       int nbuf,hmax;
       int64_t iwork1;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;
       //int64_t lbuf[1024];
	   double dw1,dw2,diffsec,rsampl,fact;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;
	   ierr=0;
       icount=0;
       if(parity==1){  // with parity
           imax=36;
           ihead=180;
           hmax=72;
           sync_pat=sync_pat1;
           fact=8.0/9.0;
           nbuf=900;
       } else { // nonparity
           imax=32;
           hmax=64;
           ihead=160;
           sync_pat=sync_pat0;
           fact=1.0;
           nbuf=800;
       }
       if(n<nbuf) nbuf=n;

       tc=-1;

       

	   nloop=0;
       ikai=0;
       kold=0;
	   sampl=0;
       /* sync detection */
       while(1){
          if(bitposmax==8){
              iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
          } else if(bitposmax==16){
              iread = fread( sbuf, sizeof( short int ), nbuf, fin );
          } else if(bitposmax==32){
              iread = fread( ibuf, sizeof( int ), nbuf, fin );
          } else {
              iread = fread( lbuf, sizeof( int64_t ), nbuf, fin );
          }
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int64_t)cbuf[i];
               else if(bitposmax==16) iwork1=(int64_t)sbuf[i];
               else if(bitposmax==32) iwork1=(int64_t)ibuf[i];
               else iwork1=lbuf[i];

              //iwork=ibuf[i];
              //iwork= ibuf[i] >> ibit;
           //   iwork= (int)(iwork1 >> ibit);
           //   ipick= (int)(iwork & 0x1);
              ipick= (int)((iwork1 >> ibit) & 0x1);
              //if(nrzm==1) know=nrzm2nrzl(kold,ipick);
              if(nrzm==1) know=kold ^ ipick;    // NRZM -> NRZL
               else know=ipick;  // in case of NRZL mode
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;
              head[ihead-1]=know;
              kold=ipick;
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  //if(tc >= 72){
                  //if(tc >= 64){
                  if(tc >= hmax){
                      tc=-1;
                      if(parity==1) conv2time2(tdat, itimew);
                      else conv2time2n(tdat, itimew);
                      /* convert header to int data */
                      for(ii=0; ii<5; ii++){
                         //for(k=0; k<36; k++){
                         //for(k=0; k<32; k++){
                         for(k=0; k<imax; k++){
                            //sync[k]=head[ii*36+k];
                             //sync[k]=head[ii*32+k];
                             sync[k]=head[ii*imax+k];
						 }
                         //header[ii]=conv2int(*(head+ii*36));
                         //headerw[ii]=conv2int(sync,1);
                         if(parity==1) headerw[ii]=conv2int(sync,1);
                         else headerw[ii]=conv2intn(sync,1);
					  }
					  //debug
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii]));
					  //printf("\n");
					  if(nloop == 0) {
                          *ipos=i;
	                      sampl=0;
						  for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
					      //printf("debug1\n");
					  } 
              //        for(m=0; m<6; m++){
              //            printf(" %d",itime[m]);
              //        }
              //        printf("\n");
				//	  printf("debug\n");
                      if(nloop==3) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                     //printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
			  sampl++;
          }
		  icount+=iread;   // 
		  if(icount > 100000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;

       /**** added on 2003-07-12 ****/
       // header check again   if AUX field has sync pattern, it may be no sync
       if(header[1] == 0xffffffff ) { // no sync
           ierr=-1;
           return ierr;
       }
       /***** add end ****/

       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");
	   dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		   +(double)itimew[4]+((double)itimew[5])/1000.0;
	   dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		   +(double)itime[4]+((double)itime[5])/1000.0;
	   diffsec=dw2-dw1;
       //rsampl=((double)sampl)*8.0/(9.0*diffsec);
       rsampl=fact*((double)sampl)/diffsec;
	   /* check fsampl */
       *fsampl=rsampl;
       //printf("%f %f %f\n",rsampl, dw1,dw2);
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;
	   return ierr;
}

void headbit2int4(int *headbit, int *headint4, int parity)
// convert bit image data to 4byte integer
{
    int sync[36];
    // int headbit[180], headint4[5];
    int i,k,imax;
    if(parity==1) imax=36;
    else imax=32;

    for(i=0; i<5; i++){
        for(k=0; k<imax; k++){   // imax is 32 for non-parity data
                                                   //         36 for with parity data
            sync[k]=headbit[i*imax+k];
        } 
        if(parity==1) headint4[i]=conv2int(sync,1);
           else headint4[i]=conv2intn(sync,1);
    }
}

void readM5_8byte(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *ko, int bitposmax, int parity, int nrzm, 
                 int *data1st)
// read 8 byte time series data from current position at given bit position
{
       int i,ipick, kold, know, iread,imax, ikai;
       int iw,i1st,is;
       int nbuf;
       int64_t iwork1;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;
       //int64_t lbuf[1024];
       int data[36];

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;
       /* constants set */
       if(parity==1){  // with parity
           imax=36;
           nbuf=900;
       } else { // nonparity
           imax=32;
           nbuf=800;
       }
       if(n<nbuf) nbuf=n;

	   iw=0;  //word position in out
       ikai=0;
       //kold=0;
       kold=*ko;
       i1st=1;
       //printf(" ipos, ireadr %d %d\n",*ipos, *ireadr);
       while(1){

          is=0;
          if(i1st==1){
               // data still remain in buffer
               i1st=0;
               if(*ipos < *ireadr-1){
                  is= *ipos+1;
                  iread= *ireadr;
               }  
          } else {
               
             if(bitposmax==8){            // 8 track mode
                iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
             } else if(bitposmax==16){  // 16 track mode
                iread = fread( sbuf, sizeof( short int ), nbuf, fin );
             } else if(bitposmax==32){  // 32 track mode
                iread = fread( ibuf, sizeof( int ), nbuf, fin );
             } else {                   // 64 track mode
                iread = fread( lbuf, sizeof( int64_t ), nbuf, fin );
             }
             if (iread ==0 ) { // EOF
               exit(1);
             } 
          }  
//printf("debug 1  ibit %d\n",ibit);

          /* parse track data */
          for (i=is; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int64_t)cbuf[i];
               else if(bitposmax==16) iwork1=(int64_t)sbuf[i];
               else if(bitposmax==32) iwork1=(int64_t)ibuf[i];
               else iwork1=lbuf[i];

              /* pick up "ibit" position data */
              ipick= (int)((iwork1 >> ibit) & 0x1);

              if(nrzm==1) know=kold ^ ipick;  // decode NRZM data
               else know=ipick;  // in case of NRZL mode
              kold=ipick;

              data[ikai]=know;
              ikai++;
              if(ikai==imax){
                  ikai=0;
                  if(parity==1){
                      data1st[iw]=conv2int(data,1);
                  } else {
                      data1st[iw]=conv2intn(data,1);
                  }
                  iw++;
                  if(iw==2) goto exit2end;
              }
          }
      }
exit2end:;
}


int read_m5_1frame_new(FILE *fin, unsigned int *ibuf, int vlba, 
                 int bitposmax, int parity, int nrzm, int64_t *iwo,
                 int skpbits, unsigned int *dat1frame) 
/*
   read Mark5 1 frame data compulsory

       Ver. 2009-02-15

      [in] 
           fin  ---- File descripter for Mark V file
           ibuf ---- data read buffer with a size >= 1440 bytes
           vlba ----  =0 for Mark-IV format, =1 for VLBA format
           bitposmax --  # of tracks (8, 16, 32, 64)
           parity ----  =1 with parity, =0 without parity
           nrzm ------  =1 NRZM data,  =0 non NRZM data
           iwo  ------  initially set to zero (used for NRZM decoding
                         in case of succeeding read)
           skpbits ---  # of bits (samples) to be skipped
      [out]
           dat1frame[630*64] -- 1 frame data 

      [return]
           read_1frame_new ---  samples/frame  
                             (20000 for Mark-IV, 20160 for VLBA)

      Note:  Mark-IV data   set vlba=0, parity=0, nrzm=0
             VLBA data          vlba=1, parity=0, nrzm=0

     format in dat1frame
        (Trk#1)W0,W1,...,W629,(Trk#2)W0,W1,...,W629,(Trk#3)W0,....
         
        where W0,W1,.. are 4byte word data

        for Mark-IV format, W0,...,W624 are valid data (i.e. 32x625=20000 samples)
        for VLBA format, W0,...,W629 (all) are valid data (i.e. 32x630=20160 samples)
       
        Contents of each Word (4bytes)
           b0(LSB),.....,b31(MSB)  --- time runs from LSB to MSB

*/
{
       int i,k,iread,ipick;
	   int sampleframe;
       int sample; // sample counter in a frame
       int nbuf;
       int c160max,c160;
       int c32; // 32 counter
       int c8;  // 8 counter
       int c4b; // 4 bytes unit number
       //unsigned int frame[630*64]; // to store frame data by track
       //                                                 // but 1bit sample data assigned each bit
       //                                                 // of 4 byte data (LSB 1st sample)
       unsigned int work32[64]; // to cotain 32 sample (LSB 1st sample) data for 64 track 
       int64_t iwork, iwork1, iwork0;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

       /* constants set */
       if(parity==1){  // with parity
           nbuf=180;
       } else { // nonparity
           nbuf=160;
       }

       //vlba=0; // Mark-IV mode is temporary set
       
       sampleframe=20000;
       if(vlba==1) sampleframe=20160;
       c160max=sampleframe/160;   // =125 for Mark-IV, =126 for VLBA
       iwork0=*iwo;
	   sample=0;
       c32=0;
       c8=0;
       c4b=0;

       /* skip first skpbits */
       if(skpbits > 0){
           for(i=0; i<skpbits; i++){
              if(bitposmax==8){            // 8 track mode
                  iread = fread( cbuf, sizeof( unsigned char ), 1, fin );
              } else if(bitposmax==16){  // 16 track mode
                  iread = fread( sbuf, sizeof( short int ), 1, fin );
              } else if(bitposmax==32){  // 32 track mode
                  iread = fread( ibuf, sizeof( int ), 1, fin );
              } else {                   // 64 track mode
                  iread = fread( lbuf, sizeof( int64_t ), 1, fin );
              }
              if (iread ==0 ) { // EOF
                 exit(1);
              }
              /* keep current data in case of NRZM mode */
              if(nrzm==1){
                  if(bitposmax==8) iwork1=(int64_t)cbuf[0];
                   else if(bitposmax==16) iwork1=(int64_t)sbuf[0];
                   else if(bitposmax==32) iwork1=(int64_t)ibuf[0];
                   else iwork1=lbuf[0];

                  iwork0=iwork1;                 // save current data
              }
           }
       }


       /* frame data read loop without checkin synch block  */
       for(c160=0; c160<c160max; c160++){
          if(bitposmax==8){            // 8 track mode
              iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
          } else if(bitposmax==16){  // 16 track mode
              iread = fread( sbuf, sizeof( short int ), nbuf, fin );
          } else if(bitposmax==32){  // 32 track mode
              iread = fread( ibuf, sizeof( int ), nbuf, fin );
          } else {                   // 64 track mode
              iread = fread( lbuf, sizeof( int64_t ), nbuf, fin );
          }
          if (iread ==0 ) { // EOF
             exit(1);
          }
          /* parse track data */
          for (i=0; i< iread; i++){  // iread should equal to nbuf
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int64_t)cbuf[i];
               else if(bitposmax==16) iwork1=(int64_t)sbuf[i];
               else if(bitposmax==32) iwork1=(int64_t)ibuf[i];
               else iwork1=lbuf[i];
              
              if(nrzm==1){
                   iwork = iwork1 ^ iwork0;  // decode NRZM data
                   iwork0=iwork1;                 // save current data
              } else {
                   iwork=iwork1;
              }

              c8++;

              //if(c4b < 1) printf("c8 = %d \n",c8); //debug
              if(c8 <= 8){
                 /* pick up each track (bit position) data */
                 for (k=0; k< bitposmax; k++){
                     work32[k] >>=1;
                     ipick= (int)(iwork & 0x1);
                     work32[k]=(work32[k] & 0x7FFFFFFF) | (ipick << 31);  // set data from MSB as LSB 1st come
                     iwork >>= 1;
                 }
                 //if(c4b < 1) printf("work32 %08X \n",rvsint(work32[0])); //debugc8 = %d \n",c8); //debug
                 c32++;
                 if(c32==32){ // subsutitute to frame data array
                     //if(c4b < 5) printf("%08X \n",rvsint(work32[0])); //debug
                     for (k=0; k< bitposmax; k++) dat1frame[630*k+c4b]=work32[k];
                     c4b++;
                     c32=0;
                 }
                 sample++;
              }
              /* byte step counter reset */
              if(parity==1){  
                  if(c8 == 9) c8=0; // this is suposed parity data
              } else {
                  if(c8 == 8) c8=0;
              }
          }
       }
       *iwo=iwork0;
	   return sample;
}

void parse_m5_header(int *header, int vlba, 
                     int *itime, mk5_aux_t *mk5aux) 
{
/*
   Parse Mark 5 haeder data to each item
     2009-02-16

     [in]
        header[5] ----  header 20 byte data
        vlba      ----  1: VLBA format  0: Mark IV
     
     [out]
        itime[6]  ----   time information as follows
                                 VLBA            Mark-IV
                         [0] =  -1           1 digit year
                         [1] = 3digit MJD        total day
                         [2] =    hour             hour
                         [3] =    minute           minute
                         [4] =    second           second
                         [5] =    10*msec          msec
         m5aux    -----   AUX field data
*/
    int idat[2];
    idat[0]=rvsint(header[3]);
    idat[1]=rvsint(header[4]);
    //printf("%08X %08X\n",idat[0],idat[1]); //debug
    if(vlba==1){
        conv2time_vlba(idat, itime);
    } else {
        conv2time(idat, itime);
    }
    parseMK5aux(header, vlba, mk5aux);
}



void pickup_m5_header(unsigned int *dat1frame, 
                      int track, int vlba, int mode_in_vlba, 
                      int *header, int *dat1st)
{
/*
      pick up header block and 1st 64 sample data
          2009-02-16

     [in] 
         dat1frame[630*64] ---- 1 frame data of Mark5
         track             ---- track# to pick up 0-63
         vlba              ---- 1: VLBA format   0: Mark IV
         mode_in_vlba      ---- 1: frame start is middle of headder
                                0: frame start is the beggining of headder
     [out]
         header[5]         ---- 160 bit (20 bytes) headder data
         dat1st[2]         ---- first 64 bit (8 bytes) data in a sampled data field  
*/
    int i;
    if(vlba == 1){ // VLBA format
        if(mode_in_vlba == 0){
            for(i=0; i<5; i++){
                header[i]=dat1frame[630*track+i];
            }
            for(i=0; i<2; i++){
                dat1st[i]=dat1frame[630*track+5+i];
            }
        } else {
            for(i=0; i<2; i++){
                header[i]=dat1frame[630*track+628+i];
            }
            for(i=2; i<5; i++){
                header[i]=dat1frame[630*track+i-2];
            }
            for(i=0; i<2; i++){
                dat1st[i]=dat1frame[630*track+3+i];
            }
        }
    } else {    // Mark-IV
        for(i=0; i<5; i++){
            header[i]=dat1frame[630*track+i];
        }
        for(i=0; i<2; i++){
            dat1st[i]=dat1frame[630*track+5+i];
        }
    }
}



int sync_detect2new64r(FILE *fin, unsigned int *ibuf, int n, int ibit, int *ireadr, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl, int *bitsframe,
                 int bitposmax, int parity, int nrzm, int *data1st) 
{
/*
      Ver. 2008-01-08 to make robust against 2008 year data
           2009-02-17 function return value is modified 
                      # of samples before sync block is returned instead of 0
                      when sync is detected
                      Note that: this shows the number of samples before sync
                      block start. Therefore header block start 64 samples
                      earlier.  In case of VLBA format, the start of frame
                      is the sync block, so the first header looks like without
                      AUX blocks.


        [out]
               bitsframe = 20000 for Mark-IV
                         = 20160 for VLBA format
        [return]
               -1  :  sync not found
               +VE or 0 : # of samples before header 

*/
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36], tdat[72];
       //int sync[36], tdat[108];  // tdat was extend to included 1st word data in the
                                 // data field
       //int ssync[36];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       //int i,ii,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
       int i,ii, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
       int nbuf,hmax;
       int nsync,ikai0;
       int ikaiold; //2008-01-08
       int vlba;
       int headok;
       int syncpos;
       int flag1,fcount;  // to save 64 bit data in 1st data block
       int d1st[72];
       int64_t iwork1;
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;
       //int64_t lbuf[1024];
	   double dw1,dw2,diffsec,rsampl,fact;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

       /* constants set */
	   ierr=0;
       icount=0;
       if(parity==1){  // with parity
           imax=36;
           ihead=180;
           hmax=72;
           sync_pat=sync_pat1;
           fact=8.0/9.0;
           nbuf=900;
       } else { // nonparity
           imax=32;
           hmax=64;
           ihead=160;
           sync_pat=sync_pat0;
           fact=1.0;
           nbuf=800;
       }
       if(n<nbuf) nbuf=n;

       ikaiold=-99;

       tc=-1;  // bit counter in the time field in the header block
               // set to 0 when sync pattern is detected

       vlba=0; // Mark-IV mode is temporary set
       if(*bitsframe==20160) vlba=1;  // this can be valid if   
                                      // bitsframe is given when called
                                      // and this info will be used for
                                      // first time field decoding
                                      // second time field will be decoded
                                      // using actual self mode judge
       
       *bitsframe=0;        // reset bit number in a frame
	   nloop=0;
       ikai=0;
       //kold=0;
       kold=*ko;
	   sampl=0;
       nsync=0;
       headok=0; // header OK flag
       syncpos=0;

       for(ii=0; ii<ihead; ii++) head[ii]=0;

       flag1=0; fcount=0;

       /* sync detection loop */
       while(1){
          if(bitposmax==8){            // 8 track mode
              iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
          } else if(bitposmax==16){  // 16 track mode
              iread = fread( sbuf, sizeof( short int ), nbuf, fin );
          } else if(bitposmax==32){  // 32 track mode
              iread = fread( ibuf, sizeof( int ), nbuf, fin );
          } else {                   // 64 track mode
              iread = fread( lbuf, sizeof( int64_t ), nbuf, fin );
          }
          if (iread ==0 ) { // EOF
             exit(1);
          }
          /* parse track data */
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int64_t)cbuf[i];
               else if(bitposmax==16) iwork1=(int64_t)sbuf[i];
               else if(bitposmax==32) iwork1=(int64_t)ibuf[i];
               else iwork1=lbuf[i];

			  //if(bitposmax==32 && ibit==8 && i <20 && ikai < 20)
				//  printf("Debug %d %08x\n",i,iwork1);
              /* pick up "ibit" position data */
              ipick= (int)((iwork1 >> ibit) & 0x1);

              //if(nrzm==1) know=nrzm2nrzl(kold,ipick);  // decode NRZM data
              if(nrzm==1) know=kold ^ ipick;  // decode NRZM data
               else know=ipick;  // in case of NRZL mode
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;   // 32 (36) bit step data 
              head[ihead-1]=know;  // 160 (180) bit step data
              kold=ipick;


              if(flag1==1){// save 1st data block
                  d1st[fcount]=know;
                  fcount++;
                  if(fcount==hmax){ // convert word data
                     if(parity==1){
                         data1st[0]=conv2int(d1st,1);
                         data1st[1]=conv2int(d1st+imax,1);
                     } else {
                         data1st[0]=conv2intn(d1st,1);
                         data1st[1]=conv2intn(d1st+imax,1);
                     } 
                     flag1=0;
                     fcount=0;
                  }
              }


              /* time block data analysis */
              if(tc >= 0) {
                  tdat[tc]=know;
                  tc++;
                  /* time block data is filled */
                  if(tc >= hmax){     // hmax is 64 for non-parity data
                                      //         72 for with parity data
                      tc=-1;

                      //if(parity==1) conv2time2(tdat, itimew);
                      //else conv2time2n(tdat, itimew);
                      /* convert to time according to the mode */
                      conv2time_full(tdat,itimew,parity,vlba); // 2003-10-28
                      /* convert header to int (4byte) data */
                    //  for(ii=0; ii<5; ii++){
                    //      for(k=0; k<imax; k++){   // imax is 32 for non-parity data
                    //                               //         36 for with parity data
                    //         sync[k]=head[ii*imax+k];
                    //      } 
                    //      //header[ii]=conv2int(*(head+ii*36));
                    //      //headerw[ii]=conv2int(sync,1);
                    //      if(parity==1) headerw[ii]=conv2int(sync,1);
                    //        else headerw[ii]=conv2intn(sync,1);
					//  }
                      headbit2int4(head, headerw, parity);
					  //debug
                      //printf("sampl = %d\n",sampl);
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii])); //debug
				      //printf("\n");

                      if(nloop == 0 ){
                          if(sampl >= ihead ) { // all header block is valid
 						     for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
                             headok=1;
                          } else { // sync pattern and time field only valid
                                   // AUX field is not yet filled
 						     for(ii=2; ii<5; ii++) header[ii]=headerw[ii];
                          }
                          flag1=1;  // to prepare getting 1st data block
                          *ipos=i;
                          // read data block here if hope so


						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
                          sampl=0;
					      //printf("debug1\n");
					  } 
                      if(nloop == 1 && headok ==0){ // aux field get 
						 for(ii=0; ii<2; ii++) header[ii]=headerw[ii];
                         headok=1;
                      }

                      //sampl=0;
              //        for(m=0; m<6; m++){
               //           printf(" %d",itime[m]);
               //       }
               //       printf("\n");
				//	  printf("debug\n");
                      if(nloop==3) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }

              /* sync field check */
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }


				  //if(bitposmax==32 && ihan ==0 && ibit==8 && nloop==1){
				  //  printf("debug bitposmax ibit nloop %d %d %d\n",bitposmax,ibit,nloop);
				  //  printf("debug imax, ikai %d %d\n",imax,ikai);
                  //  for(k=0; k< imax; k++) printf(" %d",sync[k]); //debug
				  //  printf("\n");
                  //}

                  /* sync detected */
                  //if( ihan == 0) {
                  if( ihan == 0 && ikai-ikaiold != 1) { // 2008-01-08
                      //printf("sync detected at %d\n",ikai);
                      //printf("%d  ",nframe);
                      //nframe++;
                      ikaiold=ikai;
                      if(nsync==0) {
                          ikai0=ikai;
                          syncpos=ikai-imax+1;
                      }
                      if(nsync==1){
                          //printf("ikai0,ikai %d %d\n",ikai0,ikai);
                          //printf("sync detected at %d\n",sampl); // debug
                          if(parity==1){
                              *bitsframe=(ikai-ikai0)*8/9;
                          } else {
                              *bitsframe=ikai-ikai0;
                          }
                          /* mode check either VLBA or Mark-IV */
                          if(*bitsframe==20160) {  // VLBA mode
                             vlba=1;
                          } else {                 // Mark-IV mode
                             vlba=0;
                          } 
                      }
                      nsync++;   // count up sync detected counter 
                      tc=0;      // reset bit number counter in the time field
                  }
              }


              /* shift sync data by 1bit */
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              /* shift header data by 1 bit */
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              //headbit2int32(head, headerw, parity);
                      //printf("sampl = %d\n",sampl);
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii])); //debug
					  //printf("\n");

              ikai++;
			  sampl++;   
          }
		  icount+=iread;   // 
		  if(icount > 100000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;

       /* AUX field check */
       /* if AUX field has sync pattern, it may be caused by wrong # of track */
       if(header[1] == 0xffffffff ) { // no sync
           ierr=-1;
           return ierr;
       }

       *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");

       /* estimate sampling frequency */
       if(itimew[0]==-1){ // VLBA mode
	      dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		       +(double)itimew[4]+((double)itimew[5])/10000.0;
	      dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		      +(double)itime[4]+((double)itime[5])/10000.0;
       } else { //Mark-IV mode
	      dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		      +(double)itimew[4]+((double)itimew[5])/1000.0;
	      dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		      +(double)itime[4]+((double)itime[5])/1000.0;
       }
       //printf("%d %d %d %d\n",itimew[2],itimew[3],itimew[4],itimew[5]); //debug
       //printf("%d %d %d %d\n",itime[2],itime[3],itime[4],itime[5]);  //debug
	   diffsec=dw2-dw1;
       //rsampl=((double)sampl)*8.0/(9.0*diffsec);
       rsampl=fact*((double)sampl)/diffsec;
	   /* check fsampl */
       *fsampl=rsampl;
       //printf("%f %f %f\n",rsampl, dw1,dw2);
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;

	   return syncpos;
}

int sync_detect_full(FILE *fin, int ibit, 
                 int *ipos, int *itime, int *header, int *ko, 
				 double *fsampl,
                 int bitposmax, int parity, int nrzm)  
/*
     Sync detection for all data mode
         Ver. 2004-09-03 

    [in]
        fin      -----  Mrak 5 file discripter
        ibit     -----  bit position to pick up sync pattern
        bitposmax  ---  # of bits (tracks) for a unit word
                              8,16,32,64
        parity    ----  =1 data has parity    =0 no parity
        nrzm      ----  =1 NRZM encoded ( necessary to decode the data)
                        =0 NRZL  ( any decode is not necceassay)
    [return]
        sync_detect_full ---- 0: sync found
                             -1: sync not found 
        ireadr 


*/
{
	   unsigned int ibuf[2048];  //2003-10-11
       unsigned short *sbuf;
       unsigned char *cbuf;
       int64_t *lbuf;  //2003-10-10
	   int inum=1024;
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36], tdat[72];
       int head[180];  // header 180bits including parity
       //int header[5];
       int ihead,itimew[6],headerw[20];
       int i,ii,ipick, kold, know, iread, tc, imax, ikai, k, ihan;
	   int icount,ierr,nloop,sampl;
       int nbuf,hmax;
       int ibitcount;
	   double dw1,dw2,diffsec,rsampl,fact;
       int64_t iwork1;

       sbuf=(unsigned short *)ibuf; 
       cbuf=(unsigned char *)ibuf;
       lbuf=(int64_t *)ibuf;

	   ierr=0;
       icount=0;
       ibitcount=0;
       if(parity==1){  // with parity
           imax=36;
           ihead=180;
           hmax=72;
           sync_pat=sync_pat1;
           fact=8.0/9.0;
           nbuf=900;
       } else { // nonparity
           imax=32;
           hmax=64;
           ihead=160;
           sync_pat=sync_pat0;
           fact=1.0;
           nbuf=800;
       }
       //if(n<nbuf) nbuf=n;

       tc=-1;

       

	   nloop=0;
       ikai=0;
       kold=0;
	   sampl=0;
       /* sync detection */
       while(1){
          if(bitposmax==8){
              iread = fread( cbuf, sizeof( unsigned char ), nbuf, fin );
          } else if(bitposmax==16){
              iread = fread( sbuf, sizeof( short int ), nbuf, fin );
          } else if(bitposmax==32){
              iread = fread( ibuf, sizeof( int ), nbuf, fin );
          } else {
              iread = fread( lbuf, sizeof( int64_t ), nbuf, fin );
          }
          if (iread ==0 ) { // EOF
             //exit(1);
             goto exit2return;
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              if(bitposmax==8) iwork1=(int64_t)cbuf[i];
               else if(bitposmax==16) iwork1=(int64_t)sbuf[i];
               else if(bitposmax==32) iwork1=(int64_t)ibuf[i];
               else iwork1=lbuf[i];

              //iwork=ibuf[i];
              //iwork= ibuf[i] >> ibit;
              ipick= (int)((iwork1 >> ibit) & 0x1);
              //if(nrzm==1) know=nrzm2nrzl(kold,ipick);
              if(nrzm==1) know=kold ^ ipick;    // NRZM -> NRZL
               else know=ipick;  // in case of NRZL mode
              //if(icount < iread) printf("%1d",know); // debug
              sync[imax-1]=know;
              head[ihead-1]=know;
              //  sync[0],sync[1],........, sync[imax-1]
              //                                |
              //                                know
              //  head[0],head[1],........, head[ihead-1]
              //                                |
              //                                know
              kold=ipick;
              ibitcount++;
              if(tc >= 0) {
                  // tc : initial value -1
                  //      set to 0  when sync pattern is detected
                  //      set to -1 again when tdat field is filled
                  tdat[tc]=know;
                  //      tdats contain bit data just after sync block
                  tc++;
                  if(tc >= hmax){
                      //  hmax = 72  for parity=1,  hmax=64 for parity=0
                      tc=-1;

                      if(parity==1) conv2time2(tdat, itimew);
                      else conv2time2n(tdat, itimew);
                      /* convert header to int data */
                      for(ii=0; ii<5; ii++){
                         for(k=0; k<imax; k++){
                             // imax=36 for parity=1,  imax=32 for parity=0
                             sync[k]=head[ii*imax+k];
                             // head[] contains all header bit data
                             // sync[] is used as a work array
						 }
                         // now synck[] contains 32 or 36 bits data
                         if(parity==1) headerw[ii]=conv2int(sync,1);
                         else headerw[ii]=conv2intn(sync,1);
                         // 32 bit data (36bit data with parity) are converted 
                         //   into a 4 byte-integer
					  }
					  //debug
                      //for (ii=0; ii<5; ii++) printf("%08x ",rvsint(headerw[ii]));
					  //printf("\n");
					  if(nloop == 0) {
                          *ipos=i;  // next pick up pointer in fread buffer 
	                      sampl=0;
                          /* save header and time label */
						  for(ii=0; ii<5; ii++) header[ii]=headerw[ii];
						  for(ii=0; ii<6; ii++) itime[ii]=itimew[ii];
					  } 
                      if(nloop==3) goto exit2return;
					  // if you want to check more header, change nloop=1 to 
					  // nloop=2 or 3 or more and also change 
					  // icount > 50000 condition below properly 
					  nloop++;
                  }
              }
              if (ikai > imax-1 ){ // sync check
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                     printf("sync detected at %d\n",ibitcount);
                     //printf("%d  ",nframe);
                     ibitcount=0;
                     tc=0;
                     //goto exitsync;
                  }
              }
              // shift sync data
              for(k=0; k< imax-1; k++){
                  sync[k]=sync[k+1];
              }
              // shift header data
              for(k=0; k< ihead-1; k++){
                  head[k]=head[k+1];
              }
              ikai++;
			  sampl++;
          }
		  icount+=iread;   // 
		  if(icount > 200000) {  // sync not found  (sync separation should be 20000) 
             ierr=-1;
			 return ierr;
		  }
	   }
exit2return:
       //*ipos=i;
   //    *ireadr=iread;
       *ko=kold;
       /* convert header to int data */
       //for(i=0; i<5; i++){
       //    for(k=0; k<36; k++){
       //        sync[k]=head[i*36+k];
       //    }
       //    //header[i]=conv2int(*(head+i*36));
       //    header[i]=conv2int(sync,1);
       //}
       //printf("debug\n");
	   dw2=3600.0*(double)itimew[2]+60.0*(double)itimew[3]
		   +(double)itimew[4]+((double)itimew[5])/1000.0;
	   dw1=3600.0*(double)itime[2]+60.0*(double)itime[3]
		   +(double)itime[4]+((double)itime[5])/1000.0;
	   diffsec=dw2-dw1;
       //rsampl=((double)sampl)*8.0/(9.0*diffsec);
       rsampl=fact*((double)sampl)/diffsec;
	   /* check fsampl */
       *fsampl=rsampl;
       //printf("%f %f %f\n",rsampl, dw1,dw2);
	   if(rsampl > 5.0e6) *fsampl=8.0e6;
	   if(rsampl > 9.0e6) *fsampl=16.0e6;
	   return ierr;
}

void mk5_atamadashi_old(FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
                    int *iposr, int *itime, int *header, int *ko)
{
    // return ipos :  pointer in ibuf at the last of headder including parity bit
       int it[6];
       int m,iread, khan, kold;
       int ipos;
       while(1){
           sync_detect(fin, ibuf, n, ibit, &iread, &ipos, it, header, &kold);
           khan=0;
           for(m=0; m<6; m++){
               // printf(" %d",it[m]);
               if(itime[m] != it[m]) khan=1;
           }
           //printf("\n");
           if(khan ==0) goto exit2return;    // atamadashi finish
       }
exit2return:
       *iposr=ipos;
       *ireadr=iread;
       *ko=kold;
}

int mk5_atamadashi(mk5_read_t *mk5, int soffset)
//                    int *itime, int *header, int *head180)
                    
//                    FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
//                    int *iposr, int *itime, int *header, int *ko)
/*  
     2004-09-17
  Do just second atamadashi
  
  [in] 
     soffset -- sec offset from current position

  Return
     itime[6]  --  Time at BOFRAME
	 header[5*max32] --  Header at BOFRAME
	 head180[180*max32] --  Header (bit data) at BOFRAME

*/
{
    // return ipos :  pointer in ibuf at the last of headder including parity bit
       int itime[6];
       //int m,iread, khan;
       int m,khan;
       //int nkai,i,kk, nbytes;
       int nkai,kk, nbytes;
       double diffsec, dbits;
       double dtsec1,dtsec2;

try_again:
       for(m=0; m<6; m++) itime[m]=mk5->itime[m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }
       printf("   Atamadashi time is %d/%03d %02d:%02d:%02d.%03d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       dtsec1=(double)itime[1]*86400.0+
           (double)(itime[2]*3600+itime[3]*60+itime[4])+(double)itime[5]/1000.0;

       // fast forward
       diffsec=1.0-(double)(mk5->itime[5])/1000.0 + (double)soffset;
       dbits=diffsec*mk5->fsampl;  
       if(mk5->parity==1) dbits*=9.0/8.0; // 9/8 is to include parity bits
       //nkai=(int)(dbits/(double)mk5->nbuf)-1;
       nkai=(int)((dbits-(double)mk5->bitsframe)/(double)mk5->nbuf)-20; 
       //printf("diffsec, dbits, nkai %f %f %d\n",diffsec,dbits,nkai);

       nbytes=(int)((dbits-(double)mk5->bitsframe)/8.0)*mk5->ntrack-20*mk5->nbuf;  // move byte #  2007-03-30

       //printf("nbytes = %d\n",nbytes); //debug
       fseek(mk5->fin,nbytes,SEEK_CUR); // move pointer by K5->bytesec
       if( ferror( mk5->fin ) ){
            kk=fclose( mk5->fin) ;
            return -1;
       }
       /*
       if(nkai > 0) {
          for (i=0; i< nkai; i++){
             if(mk5->ntrack==8){
                iread = fread( mk5->cbuf, sizeof( unsigned char ), mk5->nbuf, mk5->fin );
             } else if (mk5->ntrack==16){
                iread = fread( mk5->sbuf, sizeof( unsigned short ), mk5->nbuf, mk5->fin );
             } else if (mk5->ntrack==32){
                iread = fread( mk5->ibuf, sizeof( unsigned int ), mk5->nbuf, mk5->fin );
             } else {
                iread = fread( mk5->lbuf, sizeof( int64_t ), mk5->nbuf, mk5->fin );
             }
             if (iread ==0 ) { // EOF
                 kk=fclose( mk5->fin) ;
                 return -1;
             }
          }
       }
       */

       while(1){
           //sync_detect(fin, ibuf, n, ibit, &iread, &ipos, it, header, &kold);
           //printf("Debug\n");
           mk5_sync_detect(mk5);

           dtsec2=(double)mk5->itime[1]*86400.0+
                    (double)(mk5->itime[2]*3600+
                    mk5->itime[3]*60+mk5->itime[4])+(double)mk5->itime[5]/1000.0;

           if(dtsec2 > dtsec1){
               printf("   Found time      is %d/%03d %02d:%02d:%02d.%03d\n",
               mk5->itime[0],mk5->itime[1],mk5->itime[2],mk5->itime[3],
               mk5->itime[4],mk5->itime[5]);
               printf("    Found time is beyond the supposed time. So change Atamadashi Time.\n");
               goto try_again;
           }
           khan=0;
           for(m=0; m<6; m++){
               //printf(" %d",mk5->itime[m]);   //debug
               if(itime[m] != mk5->itime[m]) khan=1;
           }
           //printf("\n");
           //exit(0);
           if(khan ==0) goto exit2return;    // atamadashi finish
       }
exit2return:
       return 0;
}


int mk5_atamadashi_old2(mk5_read_t *mk5, int soffset)
//                    int *itime, int *header, int *head180)
                    
//                    FILE *fin, int *ibuf, int n, int ibit, int *ireadr, 
//                    int *iposr, int *itime, int *header, int *ko)
/*  
     2004-09-17
  Do just second atamadashi
  
  [in] 
     soffset -- sec offset from current position

  Return
     itime[6]  --  Time at BOFRAME
	 header[5*max32] --  Header at BOFRAME
	 head180[180*max32] --  Header (bit data) at BOFRAME

*/
{
    // return ipos :  pointer in ibuf at the last of headder including parity bit
       int itime[6];
       int m,iread, khan;
       int nkai,i,kk;
       double diffsec, dbits;


       for(m=0; m<6; m++) itime[m]=mk5->itime[m];
       itime[5]=0;
       itime[4]++;
       if(itime[4] >= 60){
            itime[4]=0;
            itime[3]++;
            if(itime[3]>=60){
                itime[3]=0;
                itime[2]++;
                if(itime[2]>=24){
                   itime[2]=0;
                   itime[1]++;
                }
			}
	   }
       // soffset is added to second
       if(soffset > 0){
          itime[4]+=soffset;
          m=itime[4]/60;
          itime[4]=itime[4] % 60;
          if(m>0) itime[3]+=m;
          m=itime[3]/60;
          itime[3]=itime[3] % 60;
          if(m>0) itime[2]+=m;
          m=itime[2]/24;
          itime[2]=itime[2] % 24;
          if(m>0) itime[1]+=m;
       }
       printf("   Atamadashi time is %d/%03d %02d:%02d:%02d.%03d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);

       // fast forward
       diffsec=1.0-(double)(mk5->itime[5])/1000.0 + (double)soffset;
       dbits=diffsec*mk5->fsampl;  
       if(mk5->parity==1) dbits*=9.0/8.0; // 9/8 is to include parity bits
       //nkai=(int)(dbits/(double)mk5->nbuf)-1;
       nkai=(int)((dbits-(double)mk5->bitsframe)/(double)mk5->nbuf)-20; 
       //printf("diffsec, dbits, nkai %d %d %d\n",diffsec,dbits,nkai);
       if(nkai > 0) {
          for (i=0; i< nkai; i++){
             if(mk5->ntrack==8){
                iread = fread( mk5->cbuf, sizeof( unsigned char ), mk5->nbuf, mk5->fin );
             } else if (mk5->ntrack==16){
                iread = fread( mk5->sbuf, sizeof( unsigned short ), mk5->nbuf, mk5->fin );
             } else if (mk5->ntrack==32){
                iread = fread( mk5->ibuf, sizeof( unsigned int ), mk5->nbuf, mk5->fin );
             } else {
                iread = fread( mk5->lbuf, sizeof( int64_t ), mk5->nbuf, mk5->fin );
             }
             if (iread ==0 ) { // EOF
                 kk=fclose( mk5->fin) ;
                 return -1;
             }
          }
       }

       while(1){
           //sync_detect(fin, ibuf, n, ibit, &iread, &ipos, it, header, &kold);
           //printf("Debug\n");
           mk5_sync_detect(mk5);

           khan=0;
           for(m=0; m<6; m++){
               printf(" %d",mk5->itime[m]);   //debug
               if(itime[m] != mk5->itime[m]) khan=1;
           }
           //printf("\n");
           if(khan ==0) goto exit2return;    // atamadashi finish
       }
exit2return:
       return 0;
}


void sync_detect4_old(FILE *fin, int *ibuf, int n, int nch, int *ibit,
                  int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head)
/*
     sync_detect for 4ch 
          ibit[4] --  bit pick up position for 4 ch 
*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       //int sync[36], tdat[72];
       //int head[180];  // header 180bits including parity
       int sync[144], tdat[72];
       //int head[720];  // header 180bits including parity X 4ch
       //int header[5];
       //int kold[4];
       int ihead;
       //int i,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
       int i,iwork, ipick, know, iread, tc, imax, ikai, k, ihan;
       int tmax;
       int ch;
       imax=36;
       ihead=180;
       tmax=72;
       tc=-1;

       ikai=0;
       //kold[0]=0; kold[1]=0; kold[2]=0; kold[3]=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];

              for (ch=0; ch< nch; ch++){
                 iwork= ibuf[i] >> ibit[ch];
                 ipick= iwork & 0x1;
                 //know=nrzm2nrzl(kold[ch],ipick);
                 know=kold[ch] ^ ipick;    // NRZM -> NRZL
                 sync[ch*imax+imax-1]=know;
                 head[ch*ihead+ihead-1]=know;
                 kold[ch]=ipick;
                 // ch# 0 is used for time hantei
                 if(ch==0){
                    if(tc >= 0) {
                       tdat[tc]=know;
                       tc++;
                       if(tc >= 72){
                          tc=-1;
                          conv2time2(tdat, itime);
              //          for(m=0; m<6; m++){
              //             printf(" %d",itime[m]);
              //          }
              //          printf("\n");
                          goto exit2return;
                       }
                    }
                 }
              }
              if (ikai > imax-1 ){ // sync check for ch# 0 data only
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              for (ch=0; ch< nch; ch++){
                 // shift sync data
                 for(k=0; k< imax-1; k++){
                     sync[ch*imax+k]=sync[ch*imax+k+1];
                 }
                 // shift header data
                 for(k=0; k< ihead-1; k++){
                     head[ch*ihead+k]=head[ch*ihead+k+1];
                 }
              }
              ikai++;
          }
       }
exit2return:
       *ipos=i;
       *ireadr=iread;
       /* convert header 1bit data to int data */
       for (ch=0; ch< nch; ch++){
          for(i=0; i<5; i++){
             for(k=0; k<36; k++){
               sync[k]=head[ch*ihead+i*36+k];
             }
             //header[i]=conv2int(*(head+i*36));
             header[ch*5+i]=conv2int(sync,1);
             //if (ch==0) printf("%08x\n",rvsint(header[ch*5+i]));  //debug
          }
       }
}

void sync_detect4(FILE *fin, unsigned int *ibuf, int n, int nch, int *ibit,
                 int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head)
/*
     sync_detect for 4ch with parity     Ver. 2004-09-07
          ibit[4] --  bit pick up position for 4 ch 
*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       //int sync[36], tdat[72];
       //int head[180];  // header 180bits including parity
       int sync[144], tdat[72];
       //int head[720];  // header 180bits including parity X 4ch
       //int header[5];
       //int kold[4];
       int ihead;
       //int i,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
       //int i,iwork, ipick, know, iread, tc, imax, ikai, k, ihan;
       int i,ipick, iread, tc, imax, ikai, k, ihan;
       int know;
       int tmax;
       int ch;
       int ntrack;   // number of tracks in unit word  8,16,32,64
       int parity; 
       //int nrzm;

       ntrack=32;
       parity=1;


       imax=36;
       ihead=180;
       tmax=72;
       tc=-1;

       ikai=0;
       //kold[0]=0; kold[1]=0; kold[2]=0; kold[3]=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];

              know= *kold ^ ibuf[i];   // NRZM -> NRZL
              *kold=ibuf[i];  // 2004-09-06
              for (ch=0; ch< nch; ch++){
                 ipick=(int)(know >> ibit[ch]) & 0x1;
                 sync[ch*imax+imax-1]=ipick;
                 head[ch*ihead+ihead-1]=ipick;
                 // ch# 0 is used for time hantei
                 if(ch==0){
                    if(tc >= 0) {
                       tdat[tc]=ipick;
                       tc++;
                       if(tc >= 72){
                          tc=-1;
                          conv2time2(tdat, itime);
              //          for(m=0; m<6; m++){
              //             printf(" %d",itime[m]);
              //          }
              //          printf("\n");
                          goto exit2return;
                       }
                    }
                 }
              }

              if (ikai > imax-1 ){ // sync check for ch# 0 data only
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              for (ch=0; ch< nch; ch++){
                 // shift sync data
                 for(k=0; k< imax-1; k++){
                     sync[ch*imax+k]=sync[ch*imax+k+1];
                 }
                 // shift header data
                 for(k=0; k< ihead-1; k++){
                     head[ch*ihead+k]=head[ch*ihead+k+1];
                 }
              }
              ikai++;
          }
       }
exit2return:
       *ipos=i;
       *ireadr=iread;
       /* convert header 1bit data to int data */
       for (ch=0; ch< nch; ch++){
          for(i=0; i<5; i++){
             for(k=0; k<36; k++){
               sync[k]=head[ch*ihead+i*36+k];
             }
             //header[i]=conv2int(*(head+i*36));
             header[ch*5+i]=conv2int(sync,1);
             //if (ch==0) printf("%08x\n",rvsint(header[ch*5+i]));  //debug
          }
       }
}

void sync_detect4s(FILE *fin, short int *ibuf, int n, int nch, int *ibit,
                  int *kold, int *ireadr, 
                 int *ipos, int *itime, int *header, int *head)
/*
     sync_detect for 4ch    non-parity
          ibit[4] --  bit pick up position for 4 ch 
*/
{
       int sync_pat[]={1,1,1,1,1,1,1,1,      //sync pattern bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       //int sync[36], tdat[72];
       //int head[180];  // header 180bits including parity
       int sync[144], tdat[72];
       //int head[720];  // header 180bits including parity X 4ch
       //int header[5];
       //int kold[4];
       int ihead;
       //int i,iwork, ipick, kold, know, iread, tc, imax, ikai, k, ihan;
       int i,iwork, ipick, know, iread, tc, imax, ikai, k, ihan;
       int tmax;
       int ch;
       //imax=36;
       imax=32;
       //ihead=180;
       ihead=160;
       //tmax=72;
       tmax=64;
       tc=-1;

       ikai=0;
       //kold[0]=0; kold[1]=0; kold[2]=0; kold[3]=0;
       /* sync detection */
       while(1){       
          iread = fread( ibuf, sizeof( short int ), n, fin );
          if (iread ==0 ) { // EOF
             exit(1);
          }
          for (i=0; i< iread; i++){
              //print_intbit(ibuf[1]);  //debug

              //iwork=ibuf[i];

              for (ch=0; ch< nch; ch++){
                 iwork= ibuf[i] >> ibit[ch];
                 ipick= iwork & 0x1;
                 //know=nrzm2nrzl(kold[ch],ipick);
                 know=ipick;
                 sync[ch*imax+imax-1]=know;
                 head[ch*ihead+ihead-1]=know;
                 kold[ch]=ipick;
                 // ch# 0 is used for time hantei
                 if(ch==0){
                    if(tc >= 0) {
                       tdat[tc]=know;
                       tc++;
                       if(tc >= tmax){
                          tc=-1;
                          conv2time2n(tdat, itime);
              //          for(m=0; m<6; m++){
              //             printf(" %d",itime[m]);
              //          }
              //          printf("\n");
                          goto exit2return;
                       }
                    }
                 }
              }
              if (ikai > imax-1 ){ // sync check for ch# 0 data only
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              for (ch=0; ch< nch; ch++){
                 // shift sync data
                 for(k=0; k< imax-1; k++){
                     sync[ch*imax+k]=sync[ch*imax+k+1];
                 }
                 // shift header data
                 for(k=0; k< ihead-1; k++){
                     head[ch*ihead+k]=head[ch*ihead+k+1];
                 }
              }
              ikai++;
          }
       }
exit2return:
       *ipos=i;
       *ireadr=iread;
       /* convert header 1bit data to int data */
       for (ch=0; ch< nch; ch++){
          for(i=0; i<5; i++){
             //for(k=0; k<36; k++){
             for(k=0; k<32; k++){
               //sync[k]=head[ch*ihead+i*36+k];
               sync[k]=head[ch*ihead+i*32+k];
             }
             //header[i]=conv2int(*(head+i*36));
             //header[ch*5+i]=conv2int(sync,1);
             header[ch*5+i]=conv2intn(sync,1);
             //if (ch==0) printf("%08x\n",rvsint(header[ch*5+i]));  //debug
          }
       }
}

void mk5_atamadashi4(FILE *fin, int *ibuf, int n, int nch, int *ibit, int *kold, int *ireadr, 
                    int *iposr, int *itime, int *header, int * head)
// atamadashi for 4ch
{
    // return ipos :  pointer in ibuf at the last of headder including parity bit
       int it[6];
       //int m,iread, khan, kold;
       int m,iread, khan;
       int ipos;
       while(1){
           sync_detect4(fin, ibuf, n, nch, ibit, kold, &iread, &ipos, 
               it, header, head);
           khan=0;
           for(m=0; m<6; m++){
               // printf(" %d",it[m]);
               if(itime[m] != it[m]) khan=1;
           }
           //printf("\n");
           if(khan ==0) goto exit2return;    // atamadashi finish
       }
exit2return:
       *iposr=ipos;
       *ireadr=iread;
}


void mk5_atamadashi4s(FILE *fin, short int *ibuf, int n, int nch, int *ibit, int *kold, int *ireadr, 
                    int *iposr, int *itime, int *header, int * head)
// atamadashi for 4ch
{
    // return ipos :  pointer in ibuf at the last of headder including parity bit
       int it[6];
       //int m,iread, khan, kold;
       int m,iread, khan;
       int ipos;
       while(1){
           sync_detect4s(fin, ibuf, n, nch, ibit, kold, &iread, &ipos, 
               it, header, head);
           khan=0;
           for(m=0; m<6; m++){
               // printf(" %d",it[m]);
               if(itime[m] != it[m]) khan=1;
           }
           //printf("\n");
           if(khan ==0) goto exit2return;    // atamadashi finish
       }
exit2return:
       *iposr=ipos;
       *ireadr=iread;
}

void mk5hdbit2int(mk5_read_t *mk5)
{
    int ch,i,k;
    int work[36];
    for (ch=0; ch< mk5->tot_track; ch++){
       for(i=0; i<5; i++){
             //for(k=0; k<36; k++){
           for(k=0; k<mk5->nbitword; k++){
               work[k]=mk5->head180[(i*mk5->nbitword+k)*mk5->tot_track+ch];
           } 
           if(mk5->parity==1){
               mk5->header[ch*5+i]=conv2int(work,1);
           } else {
               mk5->header[ch*5+i]=conv2intn(work,1);
           }
       } 
    } 
}

void mk5_sync_detect(mk5_read_t *mk5)
//                     int *itime, int *header, int *head180)
/*
     fast sync_detection for 4ch     Ver. 2007-03-30
     2008-01-08  to make robust against year2008 data 
 [in]
      fin  ---- file descripter 
      ibuf  --- read buffer
      nbuf  --- read buffer integer size 
      ntrack --  bits (track)# per word
      tot_track -- total track number to read
      bit_track[] --  bit position table to read
      nrzm   ----  0: NRZL mode 1: NRZM mode  
      parity ----  0: without parity   1: with parity
      kold  ----   data used to NRZM decode of first sample data

  [out]
      kold  ----   data used to NRZM decode when next called 
      ireadr ---   actual number of elements read agains requested # (nbuf)
      ipos   ---   position in ibuf just next to headder block
      itime[6] --  time in header for first bit position in bit_track table
      header[5*tot_track] ---   header block ( 160 bit X tot_track) (parity is removed)
      head180[180*tot_tarck] ---   header block 1bit data X 180 X tot_tarck (with parity)

*/
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                        1,1,1,1,1,1,1,1,0,
                        1,1,1,1,1,1,1,1,0,
                        1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                        1,1,1,1,1,1,1,1,
                        1,1,1,1,1,1,1,1,
                        1,1,1,1,1,1,1,1};
       int *sync_pat;
       int64_t iwork1;
       int sync[1152], tdat[72];  // 1152 = 36*32track
       int ihead;
       int i,ipick, iread, tc, imax, ikai, k, ihan;
       int ikaiold; //2008-01-08
       int know;
       int64_t know64;
       int tmax;
       int ch;
       int trn=0; // track# to use for time read
       int flag_exit;
       int nframe;
	   int bytes;
       int vflag;

       imax=mk5->nbitword;
       ihead=mk5->nbithead;
       tmax=8*mk5->nbitbyte;

       if(mk5->parity==1) {
           sync_pat=sync_pat1;
           //imax=36;   ->  already set as mk5->nbithword
           //ihead=180; ->  already set as mk5->nbithead
           //tmax=72;
       } else {
           sync_pat=sync_pat0;
           //imax=32;
           //ihead=160;
           //tmax=64;
       }
       tc=-1;

       ikai=0;
       flag_exit=0;
	   bytes=0;  
       vflag=0;
       ikaiold=-99;
       /* sync detection */
       while(1){       
           if(mk5->ntrack==8){
              iread = fread( mk5->cbuf, sizeof( unsigned char ), mk5->nbuf, mk5->fin );
           } else if (mk5->ntrack==16){
              iread = fread( mk5->sbuf, sizeof( unsigned short ), mk5->nbuf, mk5->fin );
           } else if (mk5->ntrack==32){
              iread = fread( mk5->ibuf, sizeof( unsigned int ), mk5->nbuf, mk5->fin );
           } else {
              iread = fread( mk5->lbuf, sizeof( int64_t ), mk5->nbuf, mk5->fin );
           }
		   bytes+=iread;
           nframe=(bytes*8/mk5->ntrack)/mk5->bitsframe;
           //printf("%d\n",nframe);
           if(nframe >= 50 && (nframe % 50) == 0){
               if(vflag==0)
                  printf("Sync pattern not found in %d frames. Still searching..\n",nframe);
               vflag=1;
           } else {
               vflag=0;
           }
           if (iread ==0 ) { // EOF
             exit(1);
           }
           for (i=0; i< iread; i++){

              if(mk5->ntrack==8) iwork1=(int64_t)mk5->cbuf[i];
              else if(mk5->ntrack==16) iwork1=(int64_t)mk5->sbuf[i];
              else if(mk5->ntrack==32) iwork1=(int64_t)mk5->ibuf[i];
              else iwork1=mk5->lbuf[i];

              //print_intbit((int)iwork1);  //debug

              if(mk5->ntrack == 64){
                  know64=iwork1;
                  if(mk5->nrzm==1) {
                     know64 ^= mk5->lkold;   // NRZM -> NRZL
                     mk5->lkold=iwork1;
                  } 
              } else {
                  know=(int)iwork1;  // ntrack < 64 is assuumed for speed up
                  if(mk5->nrzm==1) {
                     know ^= mk5->kold;   // NRZM -> NRZL
                     mk5->kold=(int)iwork1;
                  } 
              }
              for (ch=0; ch< mk5->tot_track; ch++){
                 if(mk5->ntrack == 64){
                     ipick=(int)(know64 >> mk5->bit_trk[ch]) & 0x1;
                 } else {
                     ipick=(int)(know >> mk5->bit_trk[ch]) & 0x1;
                 }
                 sync[ch*imax+imax-1]=ipick;
                 //mk5->head180[ch*ihead+ihead-1]=ipick;
                 mk5->head180[(ihead-1)*mk5->tot_track+ch]=ipick; 
                       // for speed up in later processing in main or another function
                 // ch# trn is used for time hantei
                 if(ch==trn){
                    if(tc >= 0) {
                       tdat[tc]=ipick;
                       tc++;
                       //if(tc >= 72){
                       if(tc >= tmax){
                          tc=-1;
                          //if(mk5->parity==1){
                          //    conv2time2(tdat, mk5->itime);
                          //} else {
                          //    conv2time2n(tdat, mk5->itime);
                          //}
                          conv2time_full(tdat,mk5->itime,mk5->parity,mk5->vlba); 
                          //for(m=0; m<6; m++){
                          //   printf(" %d",mk5->itime[m]);
                          //}
                          //printf("\n");
                          flag_exit=1;
                       }
                    }
                 }
              }
              if(flag_exit==1) goto exit2return;

              if (ikai > imax-1 ){ // sync check for ch# 0 data only
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  //if( ihan == 0) {
                  if( ihan == 0 && ikai-ikaiold != 1) { // 2008-01-08
                      ikaiold=ikai;
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              for (ch=0; ch< mk5->tot_track; ch++){
                 // shift sync data
                 for(k=0; k< imax-1; k++){
                     sync[ch*imax+k]=sync[ch*imax+k+1];
                 }
                 // shift header data
                 for(k=0; k< ihead-1; k++){
                     //mk5->head180[ch*ihead+k]=mk5->head180[ch*ihead+k+1];
                     mk5->head180[k*mk5->tot_track+ch]=mk5->head180[(k+1)*mk5->tot_track+ch];
                 }
              }
              ikai++;
          }
       }
exit2return:
       mk5->ipos=i;
       mk5->iread=iread;
       /* convert header 1bit data to int data */
       mk5hdbit2int(mk5);
}

void mk5_sync_detect_old(mk5_read_t *mk5)
//                     int *itime, int *header, int *head180)
/*
     fast sync_detection for 4ch     Ver. 2004-09-13
 [in]
      fin  ---- file descripter 
      ibuf  --- read buffer
      nbuf  --- read buffer integer size 
      ntrack --  bits (track)# per word
      tot_track -- total track number to read
      bit_track[] --  bit position table to read
      nrzm   ----  0: NRZL mode 1: NRZM mode  
      parity ----  0: without parity   1: with parity
      kold  ----   data used to NRZM decode of first sample data

  [out]
      kold  ----   data used to NRZM decode when next called 
      ireadr ---   actual number of elements read agains requested # (nbuf)
      ipos   ---   position in ibuf just next to headder block
      itime[6] --  time in header for first bit position in bit_track table
      header[5*tot_track] ---   header block ( 160 bit X tot_track) (parity is removed)
      head180[180*tot_tarck] ---   header block 1bit data X 180 X tot_tarck (with parity)

*/
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                        1,1,1,1,1,1,1,1,0,
                        1,1,1,1,1,1,1,1,0,
                        1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                        1,1,1,1,1,1,1,1,
                        1,1,1,1,1,1,1,1,
                        1,1,1,1,1,1,1,1};
       int *sync_pat;
       int64_t iwork1;
       int sync[1152], tdat[72];  // 1152 = 36*32track
       int ihead;
       int i,ipick, iread, tc, imax, ikai, k, ihan;
       int know;
       int64_t know64;
       int tmax;
       int ch;
       int trn=0; // track# to use for time read
       int flag_exit;
	   //int bytes;

       imax=mk5->nbitword;
       ihead=mk5->nbithead;
       tmax=8*mk5->nbitbyte;

       if(mk5->parity==1) {
           sync_pat=sync_pat1;
           //imax=36;   ->  already set as mk5->nbithword
           //ihead=180; ->  already set as mk5->nbithead
           //tmax=72;
       } else {
           sync_pat=sync_pat0;
           //imax=32;
           //ihead=160;
           //tmax=64;
       }
       tc=-1;

       ikai=0;
       flag_exit=0;
	   //bytes=0;  //debug
       /* sync detection */
       while(1){       
           if(mk5->ntrack==8){
              iread = fread( mk5->cbuf, sizeof( unsigned char ), mk5->nbuf, mk5->fin );
           } else if (mk5->ntrack==16){
              iread = fread( mk5->sbuf, sizeof( unsigned short ), mk5->nbuf, mk5->fin );
           } else if (mk5->ntrack==32){
              iread = fread( mk5->ibuf, sizeof( unsigned int ), mk5->nbuf, mk5->fin );
           } else {
              iread = fread( mk5->lbuf, sizeof( int64_t ), mk5->nbuf, mk5->fin );
           }
		   //bytes+=iread;
		   //if((bytes % 100000)==0) printf("%d \n",bytes); // debug
           if (iread ==0 ) { // EOF
             exit(1);
           }
           for (i=0; i< iread; i++){

              if(mk5->ntrack==8) iwork1=(int64_t)mk5->cbuf[i];
              else if(mk5->ntrack==16) iwork1=(int64_t)mk5->sbuf[i];
              else if(mk5->ntrack==32) iwork1=(int64_t)mk5->ibuf[i];
              else iwork1=mk5->lbuf[i];

              //print_intbit((int)iwork1);  //debug

              if(mk5->ntrack == 64){
                  know64=iwork1;
                  if(mk5->nrzm==1) {
                     know64 ^= mk5->lkold;   // NRZM -> NRZL
                     mk5->lkold=iwork1;
                  } 
              } else {
                  know=(int)iwork1;  // ntrack < 64 is assuumed for speed up
                  if(mk5->nrzm==1) {
                     know ^= mk5->kold;   // NRZM -> NRZL
                     mk5->kold=(int)iwork1;
                  } 
              }
              for (ch=0; ch< mk5->tot_track; ch++){
                 if(mk5->ntrack == 64){
                     ipick=(int)(know64 >> mk5->bit_trk[ch]) & 0x1;
                 } else {
                     ipick=(int)(know >> mk5->bit_trk[ch]) & 0x1;
                 }
                 sync[ch*imax+imax-1]=ipick;
                 //mk5->head180[ch*ihead+ihead-1]=ipick;
                 mk5->head180[(ihead-1)*mk5->tot_track+ch]=ipick; 
                       // for speed up in later processing in main or another function
                 // ch# trn is used for time hantei
                 if(ch==trn){
                    if(tc >= 0) {
                       tdat[tc]=ipick;
                       tc++;
                       //if(tc >= 72){
                       if(tc >= tmax){
                          tc=-1;
                          //if(mk5->parity==1){
                          //    conv2time2(tdat, mk5->itime);
                          //} else {
                          //    conv2time2n(tdat, mk5->itime);
                          //}
                          conv2time_full(tdat,mk5->itime,mk5->parity,mk5->vlba); 
                          //for(m=0; m<6; m++){
                          //   printf(" %d",mk5->itime[m]);
                          //}
                          //printf("\n");
                          flag_exit=1;
                       }
                    }
                 }
              }
              if(flag_exit==1) goto exit2return;

              if (ikai > imax-1 ){ // sync check for ch# 0 data only
                  ihan=0;
                  for(k=0; k< imax; k++){
                      if(sync[k]!=sync_pat[k]) ihan=1;
                  }
                  if( ihan == 0) {
                   //  printf("sync detected at %d\n",ikai);
                    // printf("%d  ",nframe);
                   //  nframe++;
                     tc=0;
                     //goto exitsync;
                  }
              }
              for (ch=0; ch< mk5->tot_track; ch++){
                 // shift sync data
                 for(k=0; k< imax-1; k++){
                     sync[ch*imax+k]=sync[ch*imax+k+1];
                 }
                 // shift header data
                 for(k=0; k< ihead-1; k++){
                     //mk5->head180[ch*ihead+k]=mk5->head180[ch*ihead+k+1];
                     mk5->head180[k*mk5->tot_track+ch]=mk5->head180[(k+1)*mk5->tot_track+ch];
                 }
              }
              ikai++;
          }
       }
exit2return:
       mk5->ipos=i;
       mk5->iread=iread;
       /* convert header 1bit data to int data */
       mk5hdbit2int(mk5);
}

//void mk5_dat_anal(char *mk5name, int *validtrk, int *trktable, 
//                  int *header, double *fsampl, int *istart)
void mk5_dat_anal_old(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart)
// mk5_dat_anal was renamed to mk5_dat_anal_old on 2004-06-29
{
/*
     Mark5 data file structure analysis 
	 2003-03-11
    [in]
	  mk5nmae  ---  Mark5 data file name
	[out]
	  validtrk[32] -- valid track table by bit position
      trktable[32] -- track number table by bit position
	  header[5*32] -- headder block by bit position
	  fsampl -------- supposed sampling frequency
	  istart[6] ----- 1st sync block time  y,ddd,hh,mm,ss,msec

*/
       int i;
       int ibuf[1024], head[5];  // header  5 X 4ch
       int nbuf=900;   //ǂ݂Ƃv@X̔{ɂƋ؂肪悭֗
                       //    ȂȂ΁@f[^͂WrbgɃpeBPrbg
       int iread; 
       int start[6];
	   //int validtrk[32];
	   int bitpos, bitposmax,trk, adid;
       int ipos, ko;
       double fsamp;
       //int k;  //debug
       //double telapsed;           // to measure processing time
       //char mk5name[80];
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1805";
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1730";
       //char* dflt_fname="D:/IPVLBI/data/jiveexp/Wb_test";
       FILE *fin;
       //mk5_aux_t aux;

       for(i=0; i<32; i++) {
           trktable[i]=0;
           validtrk[i]=0;
       }
       bitposmax=32;
       bitposmax=16;
       //bitposmax=8;

       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Mark 5 Data File : %s\n",mk5name);

	   printf("BIT#  HEADER     AUX     SYNC     TIME FIELD  CRC  Y/DDD HH:MM:SS.SSS FMHz\n");
	   for(bitpos=0; bitpos<bitposmax; bitpos++){

           //for (k=0; k<5; k++){
          if(sync_detect2(fin, ibuf, nbuf, bitpos, &iread, &ipos, 
          //if(sync_detect2s(fin, ibuf, nbuf, bitpos, &iread, &ipos, 
			  start, head, &ko, &fsamp) < 0) {  // 
			  //printf("sync is not detected for bit pos %d\n",bitpos);
			  printf("  %02d  --- sync not detected ---\n",bitpos);
			  validtrk[bitpos]=0;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=0;

		  } else {

			  validtrk[bitpos]=1;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=head[i];
			  for(i=0; i<6; i++)  istart[i]=start[i];
			  *fsampl=fsamp;

			  printf("  %02d ",bitpos);
              for (i=0; i<5; i++){   //check           
                 printf("%08x ",rvsint(head[i]));
			  }
              printf(" %d/%03d %02d:%02d:%02d.%03d",
                 istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

              printf(" %4.1f\n",fsamp/1.0e6);

              get_trk_from_header(head, &trk, &adid);
              trktable[bitpos]=trk;
              //printf("%d %d\n",trk,adid);

              //parseMK4aux(head,&aux);
              //printf("%d %d %d %d %d\n",aux.h_pos1,aux.h_pos2,aux.h_stack,
              //    aux.trk,aux.adid);
		  }
           //}
		  rewind(fin);
	   }
       i=fclose(fin) ;
}

void mk5_dat_anal_np(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart, int *ntrack)
{
/*
     Mark5 data file structure analysis for nonparity mode
	 2003-07-12
     2003-10-10  support 64 track mode
    [in]
	  mk5nmae  ---  Mark5 data file name
	[out]
	  //validtrk[32] -- valid track table by bit position
      //trktable[32] -- track number table by bit position
	  validtrk[64] -- valid track table by bit position
      trktable[64] -- track number table by bit position
	  //header[5*32] -- headder block by bit position
	  header[5*64] -- headder block by bit position
	  fsampl -------- supposed sampling frequency
	  istart[6] ----- 1st sync block time  y,ddd,hh,mm,ss,msec
      ntrack -------- # of tracks (8,16,32) or 0 (for no meaning data)
      

*/
       int i;
       int head[5];  // header  5 X 4ch
       //int lbuf[1024];  
       unsigned int lbuf[2048];  //2003-10-10  for 64 track mode
       //short int ibuf[1024];  
       int nbuf=800;   //ǂ݂Ƃv@
       int iread; 
       int start[6];
	   int bitpos, bitposmax,trk, adid;
       int ipos, ko, iok;
       int parity,nrzm;
       double fsamp;
       //double telapsed;           // to measure processing time
       //char mk5name[80];
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1805";
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1730";
       //char* dflt_fname="D:/IPVLBI/data/jiveexp/Wb_test";
       FILE *fin;
       //mk5_aux_t aux;

       //for(i=0; i<32; i++) {
       for(i=0; i<64; i++) {
           trktable[i]=0;
           validtrk[i]=0;
       }
       //bitposmax=32;
       //bitposmax=16;
       //bitposmax=8;
       *ntrack=0;
       parity=0;     // non parity
       nrzm=0;       // NRZL mode
       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Mark 5 Data File : %s\n",mk5name);

       /**** added on 2003-07-12 ****/
       printf("Now analizing the data .....\n");  
       /* check bitposmax 8 or 16 or 32 or 64*/
       //for(bitposmax=8; bitposmax <=32; bitposmax*=2){
       for(bitposmax=8; bitposmax <=64; bitposmax*=2){
       //for(bitposmax=16; bitposmax <=64; bitposmax*=2){
          iok=0;
          printf("   Checking %d track mode ..... ",bitposmax);
	      for(bitpos=0; bitpos<bitposmax; bitpos++){
             //if(sync_detect2new(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
             if(sync_detect2new64(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
			  start, head, &ko, &fsamp,bitposmax,parity,nrzm) == 0) iok++;
		      rewind(fin);
          }
          if(iok > 0) { //  this bitposmax is valid
              printf(" OK this mode\n");
              *ntrack=bitposmax;
              break;
          }
          printf(" not this mode\n");
       }
       printf("\n");
       if(*ntrack == 0) goto exitreturn;
       /****** add end ****/
       
       /* get header info */

	   printf("BIT#  HEADER     AUX     SYNC     TIME FIELD  CRC  Y/DDD HH:MM:SS.SSS FMHz\n");
	   for(bitpos=0; bitpos<bitposmax; bitpos++){

          //if(sync_detect2new(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
          if(sync_detect2new64(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
			  start, head, &ko, &fsamp,bitposmax,parity,nrzm) < 0) {  // 
			  //printf("sync is not detected for bit pos %d\n",bitpos);
			  printf("  %02d  --- sync not detected ---\n",bitpos);
			  validtrk[bitpos]=0;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=0;

		  } else {
			  validtrk[bitpos]=1;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=head[i];
			  for(i=0; i<6; i++)  istart[i]=start[i];
			  *fsampl=fsamp;

			  printf("  %02d ",bitpos);
              for (i=0; i<5; i++){   //check           
                 printf("%08x ",rvsint(head[i]));
			  }
              printf(" %d/%03d %02d:%02d:%02d.%03d",
                 istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

              printf(" %4.1f\n",fsamp/1.0e6);

              get_trk_from_header(head, &trk, &adid);
              trktable[bitpos]=trk;

              //parseMK4aux(head,&aux);
              //printf("%d %d %d %d %d\n",aux.h_pos1,aux.h_pos2,aux.h_stack,
              //    aux.trk,aux.adid);
		  }
		  rewind(fin);
	   }
exitreturn:
       i=fclose(fin) ;
}


int mk5_dat_anal_without_sync(char *mk5name, int *validtrk, int *trktable, int *hstack_tbl,
                  int *header, double *fsampl, int *istart, int ntrack, 
                  int *bitsframe, int parity, int nrzm, int mode, 
                  int vlba, int mode_in_vlba, int skpbits,
                  int *vcntable)
{
/*
     Mark5 data file structure analysis without sync detection
     modified from mk5_dat_anal
     2009-02-16

    [in]
	  mk5name  ---  Mark5 data file name
      parity   ----  1: assume with parity   0: without parity
      nrzm     ----  1: assume nrzm          0: nrzl
      mode     ----  Display style  0: old style   1: new including 1st data block
      vlba     ----  1: assume VLBA format   0: Mark-IV format
      mode_in_vlba  ---- define frame start position in VLBA format 
                          1: frame start in middle of headder]
                          0: frame start at the begining of headder
      ntrack ------  assumed # of tracks (8,16,32,64)

	[out]
	  validtrk[64] -- valid track table by bit position
      trktable[64] -- track number table by bit position
      hstack_tbl[64] -- head stack number table by bit position 1,2,3,4
	  header[5*64] -- headder block by bit position
	  fsampl -------- supposed sampling frequency
	  istart[6] ----- 1st sync block time  
                         y,ddd,hh,mm,ss,msec  for Mark-IV format
                         -1,mjd(3digits),hh,mm,ss,msec*10  for VLBA format
      bitsframe  ---- # of bits/frame   20000: Mark-IV format
                                        20160: VLBA format
      data1st[2*64] -- First 2 word data just after headder by bit position
      vcntable[64] -- Video converter number table by bit position

    [return]
       0  ---  succeed in data read
      -1  ---  failure in data read

*/
       int n,i;
       int head[5];  // header  
       int itime[6];
       int data1st[2]; // First 2 word data just after headder at given bit position
	   int bitpos, bitposmax;
       //int mode_in_vlba;
       //int parity,nrzm;
       int vlba_mode;   // =1  VLBA format   =0 Mark-IV format  =-1 no mode
       int64_t iwo;
       unsigned int dat1frame[630*64];
       unsigned int ibuf[500];
       //int skpbits;
       FILE *fin;
       mk5_aux_t aux;

       //for(i=0; i<32; i++) {
       for(i=0; i<64; i++) {
           trktable[i]=0;
           validtrk[i]=0;
           hstack_tbl[i]=0;
       }
       iwo=0;
       bitposmax=ntrack;
       vlba_mode=vlba;
       //skpbits=0;
       //if(vlba==1) skpbits=1; // this can be another values but this can use
                               // for checking "evlbi6\11-K178-1321" VLBA file
       //mode_in_vlba=1;
       //parity=0;     // non parity
       //nrzm=0;       // NRZL mode
       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL ){
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Mark 5 Data File : %s\n",mk5name);

       printf("\n");
       /* get header info */
       if(mode==0){//old style 
         if(vlba_mode ==0){ // Mark-IV format
	       printf("                                  TIME FIELD  CRC  \n");
 	       //printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  Y/DDD HH:MM:SS.SSS FMHz\n");
 	       printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  Y/DDD HH:MM:SS.SSS\n");
         } else {  // VLBA format
           printf("                                  TIME CODE   CRC  \n");
           //printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  MJD HH:MM:SS.SSSS FMHz\n");
           printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  MJD HH:MM:SS.SSSS\n");
         } 
       } else {// new style
         if(vlba_mode ==0){ // Mark-IV format
	       printf("                                  TIME FIELD  CRC  \n");
 	       //printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  DATA#1   DATA#2   FMHz\n");
 	       printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  DATA#1   DATA#2   \n");
         } else {  // VLBA format
           printf("                                  TIME CODE  CRC-  \n");
           //printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  DATA#1   DATA#2   FMHz\n");
           printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  DATA#1   DATA#2   \n");
         } 
       }

       /* 1 frame data read without checking sync */
       n=read_m5_1frame_new(fin, ibuf, vlba, bitposmax, parity, nrzm, 
                      &iwo, skpbits, dat1frame);

       *bitsframe=n;

       for(bitpos=0; bitpos<bitposmax; bitpos++){
           pickup_m5_header(dat1frame, bitpos, vlba, mode_in_vlba, head, data1st);
           parse_m5_header(head, vlba, itime, &aux);
		   validtrk[bitpos]=1;
		   for(i=0; i<5; i++)  header[5*bitpos+i]=head[i];
		   for(i=0; i<6; i++)  istart[i]=itime[i];
		   //*fsampl=fsamp;
		   printf("  %02d ",bitpos);
           for (i=0; i<5; i++){   //check           
               printf("%08X ",rvsint(head[i]));
           }
           if(mode==0) { // old style
              if(istart[0] == -1 ){ // VLBA format
                  printf(" %03d %02d:%02d:%02d.%04d",
                    istart[1],istart[2],istart[3],istart[4],istart[5]);
              } else { //  Mark-IV format
                  printf(" %d/%03d %02d:%02d:%02d.%03d",
                    istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);
              }
           } else { // new style
              for (i=0; i<2; i++){   //check           
                  printf("%08X ",data1st[i]);
              } 
           }
           //printf(" %4.1f\n",fsamp/1.0e6);
           printf("\n");

           trktable[bitpos]=aux.trk;
           hstack_tbl[bitpos]=aux.h_stack;
           vcntable[bitpos]=aux.vcn;
	   }
       i=fclose(fin) ;
       return vlba_mode;
}


int mk5_dat_anal(char *mk5name, int *validtrk, int *trktable, int *hstack_tbl,
                  int *header, double *fsampl, int *istart, int *ntrack,
                  int *bitsframe, int parity, int nrzm, int mode,
                  int *vcntable)
// mk5_dat_anal_np_r was renamed to mk5_dat_anal on 2004-06-29
//  hstack_tbl was added in parameter
{
/*
     Mark5 data file structure analysis
     modified from mk5_dat_anal_np
     2003-10-15
     2003-10-19
     2003-10-28
     2004-06-29 renamed to mk5_dat_anal
     2006-09-26 add video converter # table

    [in]
	  mk5name  ---  Mark5 data file name
      parity   ----  1: assume with parity   0: without parity
      nrzm     ----  1: assume nrzm          0: nrzl
      mode     ----  Display style  0: old style   1: new including 1st data block

	[out]
	  validtrk[64] -- valid track table by bit position
      trktable[64] -- track number table by bit position
      hstack_tbl[64] -- head stack number table by bit position 1,2,3,4
	  header[5*64] -- headder block by bit position
	  fsampl -------- supposed sampling frequency
	  istart[6] ----- 1st sync block time  
                         y,ddd,hh,mm,ss,msec  for Mark-IV format
                         -1,mjd(3digits),hh,mm,ss,msec*10  for VLBA format
      ntrack -------- # of tracks (8,16,32,64) or 0 (for no meaning data)
      bitsframe  ---- # of bits/frame   20000: Mark-IV format
                                        20160: VLBA format
      data1st[2*64] -- First 2 word data just after headder by bit position
      vcntable[64] -- Video converter number table by bit position

    [return]
       0  ---  succeed in data format analysis (Mark-IV format)
       1  ---  succeed in data format analysis (VLBA format)
      -1  ---  failure in data format analysis

*/
       int i;
       int head[5];  // header  5 X 4ch
       unsigned int lbuf[2048];  //2003-10-10  for 64 track mode
       int nbuf=800;   //ǂ݂Ƃv@
       int iread; 
       int start[6];
       int data1st[2]; // First 2 word data just after headder at given bit position
	   int bitpos, bitposmax;
       int ipos, ko, iok;
       int syncpos; //2009-02-17
       //int parity,nrzm;
       int bits;
       int vlba_mode;   // =1  VLBA format   =0 Mark-IV format  =-1 no mode
       double fsamp;
       //double telapsed;           // to measure processing time
       //char mk5name[80];
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1805";
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1730";
       //char* dflt_fname="D:/IPVLBI/data/jiveexp/Wb_test";
       FILE *fin;
       mk5_aux_t aux;

       //for(i=0; i<32; i++) {
       for(i=0; i<64; i++) {
           trktable[i]=0;
           validtrk[i]=0;
           hstack_tbl[i]=0;
       }
       //bitposmax=32;
       //bitposmax=16;
       //bitposmax=8;
       vlba_mode=-1;
       *ntrack=0;
       //parity=0;     // non parity
       //nrzm=0;       // NRZL mode
       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Mark 5 Data File : %s\n",mk5name);


       /**** added on 2003-07-12 ****/
       printf("Now analizing the data .....\n");  
       /* check bitposmax 8 or 16 or 32 or 64*/
       //for(bitposmax=8; bitposmax <=32; bitposmax*=2){
       for(bitposmax=8; bitposmax <=64; bitposmax*=2){
       //for(bitposmax=16; bitposmax <=64; bitposmax*=2){
          iok=0;
          printf("   Checking %2d track mode ..... ",bitposmax);
	      for(bitpos=0; bitpos<bitposmax; bitpos++){
             ko=0;
             if(sync_detect2new64r(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
                 start, head, &ko, &fsamp, &bits, bitposmax,parity,nrzm, data1st) >= 0) {
                   *bitsframe=bits;
                   iok++;
				   //if(bitposmax==32) printf("bitpos %d %d\n",bitpos,iok);
             }
		      rewind(fin);
          }
          if(iok > 0) { //  this bitposmax is valid
              if(*bitsframe >= 20000){ // for double check added on 2004-06-29
                  printf(" OK this mode\n");
                  *ntrack=bitposmax;
                  break;
              }
          }
          printf(" not this mode\n");
       }
       printf("\n");
       if(*ntrack == 0) goto exitreturn;
       /****** add end ****/

       vlba_mode=0;
       if(*bitsframe==20160) {
           printf(" FORMAT is VLBA\n");
           vlba_mode=1;
       }
       
       /* get header info */
       if(mode==0){//old style 
         if(vlba_mode ==0){ // Mark-IV format
	       printf("                                  TIME FIELD  CRC  \n");
 	       printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  Y/DDD HH:MM:SS.SSS FMHz  OFSTS\n");
         } else {  // VLBA format
           printf("                                  TIME CODE   CRC  \n");
           printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  MJD HH:MM:SS.SSSS FMHz  OFST\n");
         } 
       } else {// new style
         if(vlba_mode ==0){ // Mark-IV format
	       printf("                                  TIME FIELD  CRC  \n");
 	       printf("BIT#  HEADER     AUX     SYNC   YDDDHHMM SSsss 12  DATA#1   DATA#2   FMHz  OFST\n");
         } else {  // VLBA format
           printf("                                  TIME CODE  CRC-  \n");
           printf("BIT#  HEADER     AUX     SYNC   MJDsssss.ssss  16  DATA#1   DATA#2   FMHz  OFST\n");
         } 
       }

       for(bitpos=0; bitpos<bitposmax; bitpos++){
         ko=0;
          //if(sync_detect2new(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
         if((syncpos=sync_detect2new64r(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
			  start, head, &ko, &fsamp,&bits,bitposmax,parity,nrzm,data1st)) < 0) {  // 
			  //printf("sync is not detected for bit pos %d\n",bitpos);
			  printf("  %02d  --- sync not detected ---\n",bitpos);
			  validtrk[bitpos]=0;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=0;

		  } else {
			  validtrk[bitpos]=1;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=head[i];
			  for(i=0; i<6; i++)  istart[i]=start[i];
			  *fsampl=fsamp;

              if(syncpos >= 64) syncpos-=64; // convert to the position of headder start

              //printf("Debug syncpos %d\n", syncpos); // debug

			  printf("  %02d ",bitpos);
              for (i=0; i<5; i++){   //check           
                 //printf("%08x ",rvsint(head[i]));
                 printf("%08X ",rvsint(head[i]));
			  }
              if(mode==0) { // old style
                 if(istart[0] == -1 ){ // VLBA format
                   printf(" %03d %02d:%02d:%02d.%04d",
                    istart[1],istart[2],istart[3],istart[4],istart[5]);
                 } else { //  Mark-IV format
                   printf(" %d/%03d %02d:%02d:%02d.%03d",
                    istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);
                 }
              } else { // new style

                 // printf("bitpos %d\n",bitpos);
         //         readM5_8byte(fin, lbuf,nbuf, bitpos,&iread, &ipos,  // no more use
		//	         &ko,bitposmax,parity,nrzm, data1st);
                 for (i=0; i<2; i++){   //check           
                    //printf("%08x ",data1st[i]);
                    printf("%08X ",data1st[i]);
                 } 
              }
              printf(" %4.1f",fsamp/1.0e6);
              printf(" %5d",syncpos);
              printf("\n");

              //get_trk_from_header_a(head, &trk, &adid, vlba_mode);
              parseMK5aux(head, vlba_mode, &aux); //2004-09-13
              trktable[bitpos]=aux.trk;
              hstack_tbl[bitpos]=aux.h_stack;  //2004-09-13
              vcntable[bitpos]=aux.vcn;   // 2006-09-26

              //parseMK4aux(head,&aux);
              //printf("%d %d %d %d %d\n",aux.h_pos1,aux.h_pos2,aux.h_stack,
              //    aux.trk,aux.adid);


		  }
		  rewind(fin);
	   }
exitreturn:
       i=fclose(fin) ;
       return vlba_mode;
}

void mk5_dat_anal_np_full(char *mk5name, int *validtrk, int *trktable, 
                  int *header, double *fsampl, int *istart)  //just for debug purpose
{
/*
     Mark5 data file structure analysis for nonparity and 16bit-word data mode
	 2003-06-27
    [in]
	  mk5nmae  ---  Mark5 data file name
	[out]
	  validtrk[32] -- valid track table by bit position
      trktable[32] -- track number table by bit position
	  header[5*32] -- headder block by bit position
	  fsampl -------- supposed sampling frequency
	  istart[6] ----- 1st sync block time  y,ddd,hh,mm,ss,msec

*/
       int i;
       int head[5];  // header  5 X 4ch
       //int lbuf[1024];  
       //short int ibuf[1024];  
       int nbuf=800;   //ǂ݂Ƃv@
       int start[6];
	   //int validtrk[32];
	   int bitpos, bitposmax,trk, adid;
       int ipos, ko;
       int parity,nrzm;
       double fsamp;
       //double telapsed;           // to measure processing time
       //char mk5name[80];
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1805";
       //char* dflt_fname="D:/IPVLBI/data/evlbi3/288-1730";
       //char* dflt_fname="D:/IPVLBI/data/jiveexp/Wb_test";
       FILE *fin;
       //mk5_aux_t aux;

       for(i=0; i<32; i++) {
           trktable[i]=0;
           validtrk[i]=0;
       }
       //bitposmax=32;
       bitposmax=16;
       parity=0;     // non parity
       nrzm=0;       // NRZL mode
       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Mark 5 Data File : %s\n",mk5name);

	   printf("BIT#  HEADER     AUX     SYNC     TIME FIELD  CRC  Y/DDD HH:MM:SS.SSS FMHz\n");
	   //for(bitpos=0; bitpos<bitposmax; bitpos++){
	   for(bitpos=10; bitpos<11; bitpos++){

          //if(sync_detect2s(fin, ibuf, nbuf, bitpos, &iread, &ipos, 
			//  start, head, &ko, &fsamp) < 0) {  // 
          //if(sync_detect2new(fin, lbuf, nbuf, bitpos, &iread, &ipos, 
          if(sync_detect_full(fin, bitpos, &ipos, 
			  start, head, &ko, &fsamp,bitposmax,parity,nrzm) < 0) {  // 
                 //int bitposmax, int parity, int nrzm)  
			  //printf("sync is not detected for bit pos %d\n",bitpos);
			  printf("  %02d  --- sync not detected ---\n",bitpos);
			  validtrk[bitpos]=0;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=0;

		  } else {

			  validtrk[bitpos]=1;
			  for(i=0; i<5; i++)  header[5*bitpos+i]=head[i];
			  for(i=0; i<6; i++)  istart[i]=start[i];
			  *fsampl=fsamp;

			  printf("  %02d ",bitpos);
              for (i=0; i<5; i++){   //check           
                 printf("%08x ",rvsint(head[i]));
			  }
              printf(" %d/%03d %02d:%02d:%02d.%03d",
                 istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

              printf(" %4.1f\n",fsamp/1.0e6);

              get_trk_from_header(head, &trk, &adid);
              trktable[bitpos]=trk;
              //printf("%d %d\n",trk,adid);

              //parseMK4aux(head,&aux);
              //printf("%d %d %d %d %d\n",aux.h_pos1,aux.h_pos2,aux.h_stack,
              //    aux.trk,aux.adid);
		  }
		  rewind(fin);
	   }
       i=fclose(fin) ;
}




// no more use because Mark5 2bit sampling is confirmed to be as same as K5's
void decodepatn(int *iwork)
{
	int n,w1,w2,w3,iw;
	int convtab[]={1,0,2,3}; // 2bit sampling conversion table Mk5->K5
	                         //   Mk5?   K5
	                         //   00    01
	                         //   01    00
	                         //   10    10
	                         //   11    11
	w1=*iwork;
	iw=0;
	for(n=0; n<16; n++){
		iw=iw>>2;
		w2=w1 & 0x3;
		w3=convtab[w2];
	    w3=w3 << 30;
        iw= w3 | (iw & 0x3FFFFFFF);
		w1=w1>>2;
	}
	*iwork=iw;
}

void parse_hstr(char *hstr, int *hs, int *tr)
// parese "hs-tr" to "hs" integer and "tr" integer  2004-09-13
{
    int n,n1, len;
    char cbuf[20];

    len=strlen(hstr);

    *hs=1;  // head stack # default
    *tr=0;
    for(n=0; n<len; n++){
        cbuf[n]=hstr[n];
        if( hstr[n]=='-' ) {
            n1=n;
            goto exitok;
        }
    }
    *tr=atoi(hstr);
    goto exitreturn;
exitok:
    cbuf[n1]='\0';
    *hs=atoi(cbuf);
    *tr=atoi(hstr+n1+1);
exitreturn:;
}

void mk5tok5_info_setup(char *info_file, mk5tok5_info_t *mk5info)
// modified on 2003-10-28
//  serious bug was fixed on 2004-09-02
//      track information was wrong when adbit=1 and fanout =2 or more
{
     int k,n,m;
     char line[500];
     char cbuf[20];
     int hs,tr;
     FILE *fin;
     vparam_t vp;

     if( (fin  = fopen(info_file, "r" )) == NULL ){
        printf("mk5track_setup: File (%s) not found or Open Error! \n",info_file);
        exit(1);
     } 

     /* initialize */
     mk5info->m5tok5mode=1; // this means this parameters for Mark5 to K5 conv
     mk5info->nch=0;
     mk5info->ngroup=0;
     mk5info->ntrack=0;  //2003-07-12
     mk5info->parity=0;  //2003-10-28
     mk5info->nrzm=0;  //2003-10-28
     mk5info->vlba=0;  //2003-10-28

     for(n=0; n<16; n++){
         mk5info->track_channel[n]=0;
         mk5info->ch_ch[n]=0;  //2004-12-28
         for(m=0; m<4; m++){
             mk5info->signtrack[n][m]=0;
             mk5info->hs_sign[n][m]=0;  //2004-09-13
             mk5info->magtrack[n][m]=0;
             mk5info->hs_mag[n][m]=0;  //2004-09-13
         }
     }
     //for(n=0; n<32; n++){
     for(n=0; n<64; n++){
         mk5info->bit_track[n]=0;
         mk5info->bit_hs[n]=0;
         mk5info->bit_valid[n]=0;
     }
     for(n=0; n<4; n++){
         for(k=0; k<4; k++) mk5info->group_channel[n][k]=0;
     }


mainloop:
     if( fgets( line, 500, fin ) == NULL) goto exitend;
     //printf("%s",line);
     k=vex_parse(line, &vp);
     //if(line[0]=='$') printf("%s\n",line);
     //block analysis
     if(vp.n <= 0) goto mainloop;
mloop1:

     /* $CHANNEL analysis */
     if(strncmp(vp.param[0],"$CHANNEL",8)==0){  // 
cloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
cloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto cloop;
        if(strncmp(vp.param[0],"adbit",5)==0){  // 
            mk5info->adbit=atoi(vp.param[2]);
        }
        if(strncmp(vp.param[0],"sample",6)==0){  // 
          mk5info->samplrate=atof(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"fanout",6)==0){  // 
          mk5info->fanout=atoi(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"ch",2)==0){  //
           mk5info->track_channel[mk5info->nch]=atoi(vp.param[2]);
           mk5info->ch_ch[mk5info->nch]=atoi(vp.param[2]);
           m=4;
           for(n=0; n<mk5info->fanout; n++){
               //mk5info->signtrack[mk5info->nch][n]=atoi(vp.param[m]);
               strcpy(cbuf,vp.param[m]);  //2004-09-13
               parse_hstr(cbuf,&hs,&tr);  //2004-09-13
                  // if hs is not defined,  hs should be 1 
                  // this is set in parse_hstr
               mk5info->signtrack[mk5info->nch][n]=tr; //2004-09-13
               mk5info->hs_sign[mk5info->nch][n]=hs;   //2004-09-13

               //printf("debug ch fanout signtrack %d %d %d\n",n,
               //  mk5info->track_channel[mk5info->nch],
               //  mk5info->signtrack[mk5info->nch][n]);

               //if(mk5info->adbit==2){
               //   mk5info->magtrack[mk5info->nch][n]=atoi(vp.param[m+2]);
               //} 
               //m+=4;   <== This is not good in case of adbit =1
               /* 2004-09-02 */
               if(mk5info->adbit==2){
                  //mk5info->magtrack[mk5info->nch][n]=atoi(vp.param[m+2]);
                  strcpy(cbuf,vp.param[m+2]);  //2004-09-13
                  parse_hstr(cbuf,&hs,&tr);  //2004-09-13
                  mk5info->magtrack[mk5info->nch][n]=tr; //2004-09-13
                  mk5info->hs_mag[mk5info->nch][n]=hs;   //2004-09-13

                  m+=2;
               } 
               m+=2;
               /*************/
           } 
           mk5info->nch++;
        }
        if( k>0) goto cloop1;
        goto cloop;
     }

     /* $DATAMODE analysis */  // added on 2003-10-28
     if(strncmp(vp.param[0],"$DATAMODE",9)==0){  // 
dloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
dloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto dloop;
        if(strncmp(vp.param[0],"parity",6)==0){  // 
            mk5info->parity=atoi(vp.param[2]);
        }
        if(strncmp(vp.param[0],"nrzm",4)==0){  // 
          mk5info->nrzm=atoi(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"format",6)==0){  // 
          if(strncmp(vp.param[2],"VLBA",4)==0){
              mk5info->vlba=1;
          }
        } 
        //if(strncmp(vp.param[0],"ntrack",6)==0){  // 
        //  mk5info->ntrack=atoi(vp.param[2]);
        //} 
        if( k>0) goto dloop1;
        goto dloop;
     }

     /* $BITPOS analysis */
     if(strncmp(vp.param[0],"$BITPOS",7)==0){  
bloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
bloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto bloop;
        if(strncmp(vp.param[0],"bitpos",6)==0){  //
            mk5info->ntrack++;  //2003-07-12
            m=atoi(vp.param[2]);
            if(strncmp(vp.param[4],"--",2) != 0){
               //mk5info->bit_track[m]=atoi(vp.param[4]);
               strcpy(cbuf,vp.param[4]);  //2004-09-13
               parse_hstr(cbuf,&hs,&tr);  //2004-09-13
               mk5info->bit_track[m]=tr;  //2004-09-13
               mk5info->bit_hs[m]=hs;  //2004-09-13
               mk5info->bit_valid[m]=1; //2004-09-13

            }
        }
        if( k>0) goto bloop1;
        goto bloop;
     }

     /* $GROUP analysis */
     if(strncmp(vp.param[0],"$GROUP",6)==0){  
gloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
gloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto gloop;
        if(strncmp(vp.param[0],"group",5)==0){  //
            //printf("group found\n");
            m=atoi(vp.param[2]);
            n=0;
            while(n*2+4 < vp.n){
               //printf("m-1,n, vp.n %d %d %d\n",m-1,n,vp.n);
               mk5info->group_channel[m-1][n]=atoi(vp.param[n*2+4]);
               n++;
            }
            mk5info->ngroup++;
        }
        if( k>0) goto gloop1;
        goto gloop;
     }
     goto mainloop;
exitend:;
     k=fclose(fin);

     /* added on 2004-09-01 */
     if(mk5info->vlba==1) {
         mk5info->bitsframe=20160;
     } else {
         mk5info->bitsframe=20000;
     }

}

void k5tom5_info_setup(char *info_file, mk5tok5_info_t *mk5info)
// information table setup for K5 to Mark5 conversion
//    2008-09-26
{
     int k,n,m;
     char line[500];
     char cbuf[20];
     int hs,tr;
     FILE *fin;
     vparam_t vp;

     if( (fin  = fopen(info_file, "r" )) == NULL ){
        printf("mk5track_setup: File (%s) not found or Open Error! \n",info_file);
        exit(1);
     } 

     /* initialize */
     mk5info->m5tok5mode=0; // this means this parameters for K5 to Mark5 conv
     mk5info->nch=0;
     mk5info->ngroup=0;
     mk5info->ntrack=0; 
     mk5info->parity=0; 
     mk5info->nrzm=0;  
     mk5info->vlba=0; 
     mk5info->modulation=0; // 2008-07-04

     for(n=0; n<16; n++){
         mk5info->track_channel[n]=0;
         mk5info->ch_ch[n]=0;
         mk5info->ch_bbc[n]=0;
         mk5info->ch_lsb[n]=0;
         mk5info->k5ch[n]=n+1;   // 2008-09-26
         for(m=0; m<4; m++){
             mk5info->signtrack[n][m]=0;
             mk5info->hs_sign[n][m]=0;  //2004-09-13
             mk5info->magtrack[n][m]=0;
             mk5info->hs_mag[n][m]=0;  //2004-09-13
         }
     }
     //for(n=0; n<32; n++){
     for(n=0; n<64; n++){
         mk5info->bit_track[n]=0;
         mk5info->bit_hs[n]=0;
         mk5info->bit_valid[n]=0;
     }
     for(n=0; n<4; n++){
         for(k=0; k<4; k++) mk5info->group_channel[n][k]=0;
     }


mainloop:
     if( fgets( line, 500, fin ) == NULL) goto exitend;
     //printf("%s",line);
     k=vex_parse(line, &vp);
     //if(line[0]=='$') printf("%s\n",line);
     //block analysis
     if(vp.n <= 0) goto mainloop;
mloop1:

     /* $CHANNEL analysis */
     if(strncmp(vp.param[0],"$CHANNEL",8)==0){  // 
cloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
cloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto cloop;
        if(strncmp(vp.param[0],"adbit",5)==0){  // 
            mk5info->adbit=atoi(vp.param[2]);
        }
        if(strncmp(vp.param[0],"sample",6)==0){  // 
          mk5info->samplrate=atof(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"fanout",6)==0){  // 
          mk5info->fanout=atoi(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"ch",2)==0){  //
           mk5info->track_channel[mk5info->nch]=atoi(vp.param[2]);
           mk5info->ch_ch[mk5info->nch]=atoi(vp.param[2]);
           m=4;
           for(n=0; n<mk5info->fanout; n++){
               //mk5info->signtrack[mk5info->nch][n]=atoi(vp.param[m]);
               strcpy(cbuf,vp.param[m]);  //2004-09-13
               parse_hstr(cbuf,&hs,&tr);  //2004-09-13
                  // if hs is not defined,  hs should be 1 
                  // this is set in parse_hstr
               mk5info->signtrack[mk5info->nch][n]=tr; //2004-09-13
               mk5info->hs_sign[mk5info->nch][n]=hs;   //2004-09-13

               //printf("debug ch fanout signtrack %d %d %d\n",n,
               //  mk5info->track_channel[mk5info->nch],
               //  mk5info->signtrack[mk5info->nch][n]);

               //if(mk5info->adbit==2){
               //   mk5info->magtrack[mk5info->nch][n]=atoi(vp.param[m+2]);
               //} 
               //m+=4;   <== This is not good in case of adbit =1
               /* 2004-09-02 */
               if(mk5info->adbit==2){
                  //mk5info->magtrack[mk5info->nch][n]=atoi(vp.param[m+2]);
                  strcpy(cbuf,vp.param[m+2]);  //2004-09-13
                  parse_hstr(cbuf,&hs,&tr);  //2004-09-13
                  mk5info->magtrack[mk5info->nch][n]=tr; //2004-09-13
                  mk5info->hs_mag[mk5info->nch][n]=hs;   //2004-09-13

                  m+=2;
               } 
               m+=2;
               /*************/
           } 
           /*2004-12-25*/
           // BBC# get
           mk5info->ch_bbc[mk5info->nch]=atoi(vp.param[m]);
           m+=2;
           strcpy(cbuf,vp.param[m]); 
           if(cbuf[0]=='L') mk5info->ch_lsb[mk5info->nch]=1;

           /*2008-09-26*/
           // K5ch# get if existed
           if(vp.n > m+2){
               mk5info->k5ch[mk5info->nch]=atoi(vp.param[m+2]);
           }



           mk5info->nch++;
        }
        if( k>0) goto cloop1;
        goto cloop;
     }

     /* $DATAMODE analysis */  // added on 2003-10-28
     if(strncmp(vp.param[0],"$DATAMODE",9)==0){  // 
dloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
dloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto dloop;
        if(strncmp(vp.param[0],"parity",6)==0){  // 
            mk5info->parity=atoi(vp.param[2]);
        }
        if(strncmp(vp.param[0],"nrzm",4)==0){  // 
          mk5info->nrzm=atoi(vp.param[2]);
        } 
        if(strncmp(vp.param[0],"format",6)==0){  // 
          if(strncmp(vp.param[2],"VLBA",4)==0){
              mk5info->vlba=1;
          }
        } 
        if(strncmp(vp.param[0],"modulation",10)==0){  // 2008-07-04
          if(strncmp(vp.param[2],"ON",2)==0){
              mk5info->modulation=1;
          }
        } 
        //if(strncmp(vp.param[0],"ntrack",6)==0){  // 
        //  mk5info->ntrack=atoi(vp.param[2]);
        //} 
        if( k>0) goto dloop1;
        goto dloop;
     }

     /* $BITPOS analysis */
     if(strncmp(vp.param[0],"$BITPOS",7)==0){  
bloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
bloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto bloop;
        if(strncmp(vp.param[0],"bitpos",6)==0){  //
            mk5info->ntrack++;  //2003-07-12
            m=atoi(vp.param[2]);
            if(strncmp(vp.param[4],"--",2) != 0){
               //mk5info->bit_track[m]=atoi(vp.param[4]);
               strcpy(cbuf,vp.param[4]);  //2004-09-13
               parse_hstr(cbuf,&hs,&tr);  //2004-09-13
               mk5info->bit_track[m]=tr;  //2004-09-13
               mk5info->bit_hs[m]=hs;  //2004-09-13
               mk5info->bit_valid[m]=1; //2004-09-13

            }
        }
        if( k>0) goto bloop1;
        goto bloop;
     }

     /* $GROUP analysis */
     if(strncmp(vp.param[0],"$GROUP",6)==0){  
gloop:
        if(fgets(line, 500, fin ) == NULL) goto exitend;  // 1 line read
gloop1:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) goto mloop1;
        if(vp.n <= 0) goto gloop;
        if(strncmp(vp.param[0],"group",5)==0){  //
            //printf("group found\n");
            m=atoi(vp.param[2]);
            n=0;
            while(n*2+4 < vp.n){
               //printf("m-1,n, vp.n %d %d %d\n",m-1,n,vp.n);
               mk5info->group_channel[m-1][n]=atoi(vp.param[n*2+4]);
               n++;
            }
            mk5info->ngroup++;
        }
        if( k>0) goto gloop1;
        goto gloop;
     }
     goto mainloop;
exitend:;
     k=fclose(fin);

     /* added on 2004-09-01 */
     if(mk5info->vlba==1) {
         mk5info->bitsframe=20160;
     } else {
         mk5info->bitsframe=20000;
     }

}


//void make_mk5tok5_info_file(char *info_file, char* mk5file, char* vex_file) // interactive operation
void make_mk5tok5_info_file_old(char *info_file, char* mk5file, char* vex_file) // interactive operation
// renamed from make_mk5tok5_info_file to make_mk5tok5_info_file_old
//    on 2004-09-06
{
    /*
      create mk5tok5_info file from VEX file
      Ver 1.00 2003-03-25
      debugged on 2003-10-10 
    */
    char sid[10]; // station ID (usually 2char)
    char mode[30];
    char cbuf[30];
    char chanid[20];
    //char mk5file[80];
    double fsampl;
	//int validtrk[32],trktable[32], header[160], istart[6];
	int validtrk[64],trktable[64], header[160], istart[6];  //2003-10-10
    int i,n,nstation,nmax, npos;
    int chtr[8];
    FILE *fo;
    vex_t vex;
    site_t site;
    sidtable_t sidtable[20];
    track_freq_t track_freq;

 //   printf("Enter VEX file name -> ");
 //   //read1line(vex.vex_file_name);      // 1 line read from stdin
 //   scanf("%s",&vex.vex_file_name);
    strcpy(vex.vex_file_name, vex_file);

    vex_read(&vex);

 //   printf("Enter Mark5 file name -> ");
 //   //read1line(vex.vex_file_name);      // 1 line read from stdin
 //   scanf("%s",&mk5file);


    if( (fo  = fopen( info_file, "w" )) == NULL ){
       printf("File (%s) Open Error! \n",info_file);
       exit(1);
    }

    // station selction
    create_siteid_table(&vex,&nstation,sidtable);
    printf("SITES (Station ID) defined are\n");
    for(i=0;i<nstation;i++){
        printf("  %d -- %s\n",i+1,sidtable[i].sid);
    }
    printf("Select Station by number   ----> ");
    scanf("%s",&cbuf);
    i=atoi(cbuf);
    strcpy(sid,sidtable[i-1].sid);


    // search and get site info for sid
    if(search_site(&vex,sid,&site)==0){
       printf("\nDetailed site information\n");
       printf("   site definition :  %s\n",site.def);
       printf("   site name       :  %s\n",site.name);
       printf("   site ID         :  %s\n",site.id);
       printf("   site position   : %f %f %f\n",site.position[0],
                        site.position[1],site.position[2]);
       printf("   site clock\n");
       printf("      validity epoch : %d %d %d %d %d\n",
                   site.sclock.validity_epoch[0],
                   site.sclock.validity_epoch[1],
                   site.sclock.validity_epoch[2],
                   site.sclock.validity_epoch[3],
                   site.sclock.validity_epoch[4]);
       printf("      clock epoch    : %d %d %d %d %d\n",
                   site.sclock.clock_epoch[0],
                   site.sclock.clock_epoch[1],
                   site.sclock.clock_epoch[2],
                   site.sclock.clock_epoch[3],
                   site.sclock.clock_epoch[4]);
       printf("      clock offset   : %e\n",site.sclock.clock_offset);
       printf("      clock rate     : %e\n",site.sclock.clock_rate);

    }

    // mode selection from scan No.1 info
    strcpy(mode,vex.scan[0].mode);
    printf("mode is %s\n",mode);


    // search and get track and freq info  for sid
    printf("\nTRACK and FREQUENCY information for %s\n",site.name);
    if(search_track_freq(&vex,mode,sid,&track_freq)==0){
          printf("   BARREL ROLL : %s\n",track_freq.roll);
          printf("   FREQDEF = %s   TRACKDEF = %s\n",track_freq.freqdef,track_freq.trackdef);
          printf("   adbit= %d sample_rate= %f\n",track_freq.adbit, track_freq.samplrate);
          printf("\n");
          printf("   bb Tr   AD fo  chan     RF(Hz)        S     VBW(Hz)\n");
          for(i=0; i<track_freq.ntrack; i++){
              printf("   %2d %2d %5s %d %s %f %s %f\n",i+1, track_freq.trk[i].track,
                  track_freq.trk[i].bitstream,track_freq.trk[i].fan_out_pos,
                  track_freq.trk[i].chan_id,track_freq.trk[i].rf,
                  track_freq.trk[i].sideband,track_freq.trk[i].vbw);
          }
    }

    // create info file

    fprintf(fo,"*** mk5tok5 information file created by make_mk5tok5_info_file 1.00\n");
    fprintf(fo,"*** station : %s %s    mode : %s\n",site.name, site.id,mode);
    fprintf(fo,"*** \n");

    // $CHANNEL
    fprintf(fo,"$CHANNEL;  * channel-track info block \n");
    fprintf(fo,"adbit  = %d ;  *   A/D resolution\n",track_freq.adbit);
    fprintf(fo,"sample = %f ;  *   Sampling frequency\n",track_freq.samplrate);
    fprintf(fo,"fanout = %d ;  *   Fanout\n",track_freq.fanout);
    fprintf(fo,"** ch = ch# : sign_tr# : mag_tr# :  sign_tr# : mag_tr# : .....\n");
    // 
    nmax=track_freq.adbit*track_freq.fanout;
    // check chan id (number)  &Ch01 or &CH01 type
    for(n=0; n<track_freq.nfreq; n++){
        sprintf(cbuf,"%02d\0",n+1);
        for(i=0; i<8; i++) chtr[i]=0;

        for(i=0; i<track_freq.ntrack; i++){
            strcpy(chanid,track_freq.trk[i].chan_id);
            if(strstr(chanid,cbuf) != NULL){
                //npos=track_freq.fanout*(track_freq.trk[i].fan_out_pos-1);
                npos=track_freq.adbit*(track_freq.trk[i].fan_out_pos-1); //2003-10-10
                if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                    chtr[npos]=track_freq.trk[i].track;
                } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                    chtr[npos+1]=track_freq.trk[i].track;
                }
            }
        }
        fprintf(fo,"   ch = %02d : %02d ",n+1,chtr[0]);
        if(nmax >=2) {
            for(i=1; i<nmax; i++){
                fprintf(fo,": %02d ",chtr[i]);
            }
        }
        fprintf(fo,";\n");
    }

    // $BITPOS
    mk5_dat_anal_old(mk5file, validtrk, trktable, header, &fsampl, istart);
    fprintf(fo,"$BITPOS;   * bit position versus track information\n");
    fprintf(fo,"** bitpos = nit_pos# : track #\n");
    for(n=0; n<32; n++){
        if(validtrk[n]==0){
            fprintf(fo,"   bitpos = %02d : -- ;\n",n);
        } else {
            fprintf(fo,"   bitpos = %02d : %02d ;\n",n,trktable[n]);
        }
    }

    // $GROUP    default group set
    fprintf(fo,"$GROUP;   * group # versus channel # table\n");
    fprintf(fo,"** group = group# : ch 1st : ch 2nd : ch 3rd : ch 4th;\n");
    if(track_freq.nfreq == 14){  // Mark4 Westford
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :   9 :  10 ;\n");
       fprintf(fo,"   group = 4 :  11 :  12 :  13 :  14 ;\n");
    } else {
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :  11 :  12 ;\n");
       fprintf(fo,"   group = 4 :  13 :  14 :  15 :  16 ;\n");
    }
    i=fclose(fo) ; //2003-10-10
}

void make_mk5tok5_info_file_np(char *info_file, char* mk5file, char* vex_file, 
                               char *sid, int scan, char* program_version,
                               char* dflt_pass) 
// interactive operation if sid is not selected
// 2003-10-13 Ver 1.10
// 2004-06-29 Ver 1.10  default tape pass was added
{
    /*
      create mk5tok5_info file from VEX file
      Ver 1.01 2003-07-12
      modified on 2003-08-01 
      debugged on 2003-10-10 
      Ver 1.10 2004-06-29  default tape pass was added
    */
    //char sid[10]; // station ID (usually 2char)
    char mode[30];
    char cbuf[30];
    char chanid[20];
    //char mk5file[80];
    double fsampl;
	//int validtrk[32],trktable[32], header[160], istart[6];
	int validtrk[64],trktable[64], header[320], istart[6];  //2003-10-10
    int i,n,nstation,nmax, npos;
    int scan_n;   // scan # for getting MODE
    int chtr[8];
    int ntrack;
    FILE *fo;
    time_t tnow;
    vex_t vex;
    site_t site;
    sidtable_t sidtable[20];
    track_freq_t track_freq;


 //   printf("Enter VEX file name -> ");
 //   //read1line(vex.vex_file_name);      // 1 line read from stdin
 //   scanf("%s",&vex.vex_file_name);
    strcpy(vex.vex_file_name, vex_file);

    /* VEX file read and do analysis */

    vex_read(&vex);

 //   printf("Enter Mark5 file name -> ");
 //   //read1line(vex.vex_file_name);      // 1 line read from stdin
 //   scanf("%s",&mk5file);
    //FREQ_mon(&vex);  //debug

    if( (fo  = fopen( info_file, "w" )) == NULL ){
       printf("File (%s) Open Error! \n",info_file);
       exit(1);
    }

    // station selction
    create_siteid_table(&vex,&nstation,sidtable);
    printf("SITES (Station ID) defined are\n");
    for(i=0;i<nstation;i++){
        printf("  %d -- %s\n",i+1,sidtable[i].sid);
    }
    if(sid[0]=='\0' || sid[0]=='-'){  // interactive operation
        printf("Select Station by number   ----> ");
        scanf("%s",&cbuf);
        i=atoi(cbuf);
        strcpy(sid,sidtable[i-1].sid);

    } else { // non interactive operation
        printf("Selected Station ID ----> ");
        printf("%s\n",sid);
    }

    // search and get site info for sid
    if(search_site(&vex,sid,&site)==0){
       printf("\nDetailed site information\n");
       printf("   site definition :  %s\n",site.def);
       printf("   site name       :  %s\n",site.name);
       printf("   site ID         :  %s\n",site.id);
       printf("   site position   : %f %f %f\n",site.position[0],
                        site.position[1],site.position[2]);
       printf("   site clock\n");
       printf("      validity epoch : %d %d %d %d %d\n",
                   site.sclock.validity_epoch[0],
                   site.sclock.validity_epoch[1],
                   site.sclock.validity_epoch[2],
                   site.sclock.validity_epoch[3],
                   site.sclock.validity_epoch[4]);
       printf("      clock epoch    : %d %d %d %d %d\n",
                   site.sclock.clock_epoch[0],
                   site.sclock.clock_epoch[1],
                   site.sclock.clock_epoch[2],
                   site.sclock.clock_epoch[3],
                   site.sclock.clock_epoch[4]);
       printf("      clock offset   : %e\n",site.sclock.clock_offset);
       printf("      clock rate     : %e\n",site.sclock.clock_rate);

    } else {
        printf("Selected Station ID (%s) is not included in the VEX file\n",sid);
        exit(0);  // stop
    }

    // mode selection from given scan #
    scan_n=scan;   // default scan#
    //printf("numobs scan_n %d %d\n",vex.numobs,scan_n);
    if(scan_n > vex.numobs) {
        printf("  Given scan # (%d) for MODE get eceeds maximum scan # (%d)\n",
            scan,vex.numobs);
        printf("  so scan # for mode get was changed to 1\n");
        scan_n=1;
    }
    strcpy(mode,vex.scan[scan_n-1].mode);
    printf("mode is %s\n",mode);
    printf("Mode was taken from Scan #%d as %s\n",scan_n,mode);


    // search and get track and freq info  for sid
    printf("\nTRACK and FREQUENCY information for %s\n",site.name);
    if(search_track_freq(&vex,mode,sid,&track_freq)==0){
          printf("   BARREL ROLL : %s\n",track_freq.roll);
          printf("   FREQDEF = %s   TRACKDEF = %s\n",track_freq.freqdef,track_freq.trackdef);
          printf("   adbit= %d sample_rate= %f\n",track_freq.adbit, track_freq.samplrate);
          printf("\n");
          printf("   bb Pass Tr   AD fo  chan     RF(Hz)       S     VBW(Hz)\n");
          for(i=0; i<track_freq.ntrack; i++){
              printf("   %2d    %s %2d %5s %d %s %f %s %f\n",i+1, 
                  track_freq.trk[i].sub_pass,
                  track_freq.trk[i].track,
                  track_freq.trk[i].bitstream,track_freq.trk[i].fan_out_pos,
                  track_freq.trk[i].chan_id,track_freq.trk[i].rf,
                  track_freq.trk[i].sideband,track_freq.trk[i].vbw);
          }
    }
    //FREQ_mon(&vex);  //debug

    // create info file

    tnow=time(NULL);
    //fprintf(fo,"*** mk5tok5 information file created by make_mk5tok5_info_file 1.10\n");
    fprintf(fo,"*** mk5tok5 information file created by %s\n",program_version);
    fprintf(fo,"***       on %s",ctime(&tnow));
    fprintf(fo,"***   analyzed VEX file    : %s\n",vex_file);
    fprintf(fo,"***   analyzed Mark-5 file : %s\n",mk5file);
    fprintf(fo,"***   station : %s    (%s)\n",site.name, site.id);
    fprintf(fo,"***   mode (for scan # %d)  : %s\n",scan_n,mode);
    fprintf(fo,"*** \n");

    // $CHANNEL
    fprintf(fo,"$CHANNEL;  * channel-track info block \n");
    fprintf(fo,"adbit  = %d ;  *   A/D resolution\n",track_freq.adbit);
    fprintf(fo,"sample = %f ;  *   Sampling frequency\n",track_freq.samplrate);
    fprintf(fo,"fanout = %d ;  *   Fanout\n",track_freq.fanout);
    fprintf(fo,"** default pass = %s\n",dflt_pass);
    //fprintf(fo,"** ch = ch# : sign_tr# : mag_tr# :  sign_tr# : mag_tr# : .....\n");
    if(track_freq.adbit==1){
        strcpy(cbuf," : sign_tr#");
    } else {
        strcpy(cbuf," : sign_tr# : mag_tr#");
    }
    fprintf(fo,"** ch = ch#");
    for(n=0; n<track_freq.fanout; n++) fprintf(fo,"%s",cbuf);
    fprintf(fo,"\n");

    // 
    nmax=track_freq.adbit*track_freq.fanout;
    // check chan id (number)  &Ch01 or &CH01 type
    //printf("Debug #1 %d %d\n",track_freq.ntrack, nmax);
    for(n=0; n<track_freq.nfreq; n++){
        sprintf(cbuf,"%02d\0",n+1);
        for(i=0; i<8; i++) chtr[i]=0;

        for(i=0; i<track_freq.ntrack; i++){
            strcpy(chanid,track_freq.trk[i].chan_id);
            if(strstr(chanid,cbuf) != NULL){
          //      npos=track_freq.fanout*(track_freq.trk[i].fan_out_pos-1);
                npos=track_freq.adbit*(track_freq.trk[i].fan_out_pos-1); //2003-10-10
  //  printf("Debug n = %d, i= %d npos=%d  %d %s\n",n,i,npos,
  //      track_freq.trk[i].fan_out_pos,track_freq.trk[i].bitstream);
                if(track_freq.trk[i].sub_pass[0]==dflt_pass[0]){ // 2004-06-29
                   if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                       chtr[npos]=track_freq.trk[i].track;
                   } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                       chtr[npos+1]=track_freq.trk[i].track;
                   } 
                }
            }
        }
        fprintf(fo,"   ch = %02d : %02d ",n+1,chtr[0]);
        if(nmax >=2) {
            for(i=1; i<nmax; i++){
                fprintf(fo,": %02d ",chtr[i]);
            }
        }
        fprintf(fo,";\n");
    }

    // $BITPOS
    mk5_dat_anal_np(mk5file, validtrk, trktable, header, &fsampl, istart, &ntrack);
    fprintf(fo,"$BITPOS;   * bit position versus track information\n");
    //printf("$BITPOS;   * bit position versus track information\n");
    fprintf(fo,"** bitpos = bit_pos# : track #\n");
    //printf("** bitpos = bit_pos# : track #\n");
    //for(n=0; n<32; n++){
    for(n=0; n<ntrack; n++){  // 2003-07-12
        if(validtrk[n]==0){
            fprintf(fo,"   bitpos = %02d : -- ;\n",n);
            //printf("   bitpos = %02d : -- ;\n",n);
        } else {
            fprintf(fo,"   bitpos = %02d : %02d ;\n",n,trktable[n]);
            //printf("   bitpos = %02d : %02d ;\n",n,trktable[n]);
        }
    }

    // $GROUP    default group set
    fprintf(fo,"$GROUP;   * group # versus channel # table\n");
    fprintf(fo,"**     Please edit this table as you like\n");
    fprintf(fo,"** group = group# : ch 1st : ch 2nd : ch 3rd : ch 4th;\n");
    if(track_freq.nfreq == 14){  // Mark4 Westford
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :   9 :  10 ;\n");
       fprintf(fo,"   group = 4 :  11 :  12 :  13 :  14 ;\n");
    } else {
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :  11 :  12 ;\n");
       fprintf(fo,"   group = 4 :  13 :  14 :  15 :  16 ;\n");
    }
    i=fclose(fo) ; //2003-10-10

}

void make_mk5tok5_info_file(char *info_file, char* mk5file, char* vex_file, 
                            char *sid, int scan, char* program_version,
                            char* dflt_pass) 
// renamed from make_mk5tok5_info_file_r to make_mk5tok5_info_file
//    on 2004-09-06
/*
   2004-09-13   head stack info was added in a information file
   2005-01-13   miner change in sid selection

 ** ch = ch# : sign_tr# : mag_tr# : sign_tr# : mag_tr#
    ch = 01 : 02 : 10 : 04 : 12 ;
    ........
                      |
                      v

 ** ch = ch# : sign_tr# : mag_tr# : sign_tr# : mag_tr#
    ch = 01 : 1-02 : 1-10 : 1-04 : 1-12 ;

      where  "1-02" :  1 means head stack#



      create an file containing information necessary
      for conversion from Mark-V format to K5 format

      both with parity and non-parity mode are supported

      If sid is not given or "--", station selection will be 
      made interactively.
       
      Ver 1.00 2003-10-28
      Ver 1.10 2004-06-29  default tape pass was added
      Ver 1.11 2004-09-13  head stack info was added in track#

   [in] 

*/
{
    //char sid[10]; // station ID (usually 2char)
    char mode[30];
    char cbuf[30];
    char chanid[20];
    //char mk5file[80];
    double fsampl;
	//int validtrk[32],trktable[32], header[160], istart[6];
	int validtrk[64],trktable[64], header[320], istart[6];  //2003-10-10
    int vcntable[64]; //2006-09-26
    int hstack_tbl[64];
    int i,n,nstation,nmax, npos;
    int iok;
    int scan_n;   // scan # for getting MODE
    int chtr[8];
    int hstr[8]; //2004-09-13  
    int ntrack;
      //ntrack -------- # of tracks (8,16,32,64) or 0 (for no meaning data)
    int bitsframe, parity, nrzm;
      //parity   ----  1: assume with parity   0: without parity
      //nrzm     ----  1: assume nrzm          0: nrzl
      //bitsframe  ---- # of bits/frame   20000: Mark-IV format
      //                                  20160: VLBA format
    int dispmode=0;  // data analysis display mode 0: old style 1: new style
    FILE *fo;
    time_t tnow;
    vex_t vex;
    site_t site;
    sidtable_t sidtable[20];
    track_freq_t track_freq;

    strcpy(vex.vex_file_name, vex_file);

    /* analyze VEX file */

    vex_read(&vex);

    /* information file open to write */

    if( (fo  = fopen( info_file, "w" )) == NULL ){
       printf("File (%s) Open Error! \n",info_file);
       exit(1);
    }

    /* station selction */

    create_siteid_table(&vex,&nstation,sidtable);
    printf("SITES (Station ID) defined are\n");
    printf("SITES (Station ID) defined are\n");
    printf("\n");
    printf("  ID    SITE NAME\n");
    printf(" -----------------\n");
    for(i=0;i<nstation;i++){
        printf("  %2s    %s\n",sidtable[i].sid,sidtable[i].site);
    }
    printf(" -----------------\n");
    if(sid[0]=='\0' || sid[0]=='-'){  // interactive operation
tryagain:
        printf("Select Station by ID   ----> ");
        scanf("%s",&cbuf);
        //i=atoi(cbuf);
        //strcpy(sid,sidtable[i-1].sid);
        strcpy(sid,cbuf);

        //check code is OK or not
        iok=0;
        for(i=0;i<nstation;i++){
            if(strncmp(sidtable[i].sid,sid,strlen(sid))==0){
                iok=1;
                break;
            }
        }
        if(iok==0){
            printf(" ID(%s) is not included in the table! Input again!\n",sid);
            goto tryagain;
        }

    } else { // non interactive operation
        printf("Selected Station ID = ");
        printf("%s\n",sid);
    }

    /* search and get site information for sid */

    if(search_site(&vex,sid,&site)==0){
       printf("\nDetailed site information\n");
       printf("   site definition :  %s\n",site.def);
       printf("   site name       :  %s\n",site.name);
       printf("   site ID         :  %s\n",site.id);
       printf("   site position   : %f %f %f\n",site.position[0],
                        site.position[1],site.position[2]);
       printf("   site clock\n");
       printf("      validity epoch : %d %d %d %d %d\n",
                   site.sclock.validity_epoch[0],
                   site.sclock.validity_epoch[1],
                   site.sclock.validity_epoch[2],
                   site.sclock.validity_epoch[3],
                   site.sclock.validity_epoch[4]);
       printf("      clock epoch    : %d %d %d %d %d\n",
                   site.sclock.clock_epoch[0],
                   site.sclock.clock_epoch[1],
                   site.sclock.clock_epoch[2],
                   site.sclock.clock_epoch[3],
                   site.sclock.clock_epoch[4]);
       printf("      clock offset   : %e\n",site.sclock.clock_offset);
       printf("      clock rate     : %e\n",site.sclock.clock_rate);

    } else {
        printf("Selected Station ID (%s) is not included in the VEX file\n",sid);
        exit(0);  // stop
    }

    /* mode selection from given scan # */

    scan_n=scan;   // default scan#
    //printf("numobs scan_n %d %d\n",vex.numobs,scan_n); //debug
    if(scan_n > vex.numobs) {
        printf("  Given scan # (%d) for MODE get exceeds maximum scan # (%d)\n",
            scan,vex.numobs);
        printf("  so scan # for mode get was changed to 1\n");
        scan_n=1;
    }
    if(scan_n < 0) {
        printf("  Given scan # (%d) for MODE get is illegal,\n",scan);
        printf("  so scan # for mode get was changed to 1\n");
        scan_n=1;
    }
    strcpy(mode,vex.scan[scan_n-1].mode);
    printf("mode is %s\n",mode);
    printf("Mode was taken from Scan #%d as %s\n",scan_n,mode);
    // check this scan includes given station or not  added on 2005-01-13
    iok=0;
    for(n=scan_n-1; n< vex.numobs; n++){
        for(i=0; i< vex.scan[n].numstat; i++){
            if(strncmp(vex.scan[n].station[i].id,sid,strlen(sid))==0){
                if(vex.scan[n].station[i].data_stop > 0){  //  OK
                    iok=1;
                    break;
                }
            }
        }
        if(iok==1) break;
    }
    if(iok==1){
        scan_n=n+1;
        printf("  Scan # for mode get is %d\n",scan_n);
    } else {
        printf("  Valid scan for given station ID (%s) was not found!\n",sid);
        printf("  Anyway we use scan # of %d for getting mode\n",scan_n);
    }


    /* search and get track and freq information for sid */

    printf("\nTRACK and FREQUENCY information for %s\n",site.name);
    if(search_track_freq(&vex,mode,sid,&track_freq)==0){
          printf("   BARREL ROLL : %s\n",track_freq.roll);
          printf("   FREQDEF = %s   TRACKDEF = %s\n",track_freq.freqdef,track_freq.trackdef);
          printf("   adbit= %d sample_rate= %f\n",track_freq.adbit, track_freq.samplrate);
          printf("\n");
          printf("   bb Pass HS Tr   AD fo  chan     RF(Hz)    S   VBW(Hz)\n");
          //                 HS : Head Stack #
          //                            fo : fan out
          for(i=0; i<track_freq.ntrack; i++){
              printf("   %2d    %s %2d %2d %5s %d %s %13.1f %s %11.1f\n",i+1, 
                  track_freq.trk[i].sub_pass,
                  track_freq.trk[i].headstack,  //2004-09-13
                  track_freq.trk[i].track,
                  track_freq.trk[i].bitstream,track_freq.trk[i].fan_out_pos,
                  track_freq.trk[i].chan_id,track_freq.trk[i].rf,
                  track_freq.trk[i].sideband,track_freq.trk[i].vbw);
          }
    }

    /* write information on info_file */

    tnow=time(NULL);
    fprintf(fo,"*** mk5tok5 information file created by %s\n",program_version);
    fprintf(fo,"***       on %s",ctime(&tnow));
    fprintf(fo,"***   (head stack number included in track info)\n");
    fprintf(fo,"***   analyzed VEX file    : %s\n",vex_file);
    fprintf(fo,"***   analyzed Mark-5 file : %s\n",mk5file);
    fprintf(fo,"***   station : %s    (%s)\n",site.name, site.id);
    fprintf(fo,"***   mode (for scan # %d)  : %s\n",scan_n,mode);
    fprintf(fo,"*** \n");

    // $CHANNEL
    fprintf(fo,"$CHANNEL;  * channel-track info block \n");
    fprintf(fo,"   adbit  = %d ;  *   A/D resolution\n",track_freq.adbit);
    fprintf(fo,"   sample = %f ;  *   Sampling frequency\n",track_freq.samplrate);
    fprintf(fo,"   fanout = %d ;  *   Fanout\n",track_freq.fanout);
    fprintf(fo,"** default pass = %s\n",dflt_pass);
    fprintf(fo,"**\n");
    fprintf(fo,"**     nn   => channel #\n");
    fprintf(fo,"**     h-ss => h: head stack #, ss: sign bit track #\n");
    fprintf(fo,"**     h-mm => h: head stack #, mm: magnitude bit track #\n");
    //fprintf(fo,"** ch = ch# : sign_tr# : mag_tr# :  sign_tr# : mag_tr# : .....\n");
    if(track_freq.adbit==1){
        strcpy(cbuf," : h-ss");
    } else {
        strcpy(cbuf," : h-ss : h-mm");
    }
    fprintf(fo,"** ch = nn");
    for(n=0; n<track_freq.fanout; n++) fprintf(fo,"%s",cbuf);
    fprintf(fo,"\n");
    // 
    nmax=track_freq.adbit*track_freq.fanout;
    // check chan id (number)  &Ch01 or &CH01 type
    //printf("Debug #1 %d %d\n",track_freq.ntrack, nmax);
    for(n=0; n<track_freq.nfreq; n++){
        sprintf(cbuf,"%02d\0",n+1);
        for(i=0; i<8; i++) chtr[i]=0;
        for(i=0; i<8; i++) hstr[i]=0;  //2004-09-13

        for(i=0; i<track_freq.ntrack; i++){
            strcpy(chanid,track_freq.trk[i].chan_id);
            if(strstr(chanid,cbuf) != NULL){
          //      npos=track_freq.fanout*(track_freq.trk[i].fan_out_pos-1);
                npos=track_freq.adbit*(track_freq.trk[i].fan_out_pos-1); //2003-10-10
  //  printf("Debug n = %d, i= %d npos=%d  %d %s\n",n,i,npos,
  //      track_freq.trk[i].fan_out_pos,track_freq.trk[i].bitstream);

                // check sub pass is defined in a vex file
                if(track_freq.trk[i].sub_pass[0]== '\0'){ // 2004-09-20
                   if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                       chtr[npos]=track_freq.trk[i].track;
                       hstr[npos]=track_freq.trk[i].headstack; //2004-09-13
                   } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                       chtr[npos+1]=track_freq.trk[i].track;
                       hstr[npos+1]=track_freq.trk[i].headstack; //2004-09-13
                   } 
                } else {
                   if(track_freq.trk[i].sub_pass[0]==dflt_pass[0]){ // 2004-06-29
                      if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                          chtr[npos]=track_freq.trk[i].track;
                          hstr[npos]=track_freq.trk[i].headstack; //2004-09-13
                      } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                          chtr[npos+1]=track_freq.trk[i].track;
                          hstr[npos+1]=track_freq.trk[i].headstack; //2004-09-13
                      } 
                   } 
                }
            }
        }
        //fprintf(fo,"   ch = %02d : %02d ",n+1,chtr[0]);
        fprintf(fo,"   ch = %02d : %d-%02d ",n+1,hstr[0],chtr[0]);
        if(nmax >=2) {
            for(i=1; i<nmax; i++){
                fprintf(fo,": %d-%02d ",hstr[i],chtr[i]);
            }
        }
        fprintf(fo,";\n");
    }

    /* analyze Mark-V data file */

    // first non-parity mode check then parity mode check
    parity=0;
    nrzm=0;
    printf("Non-parity mode is first assumed in data format analysis..\n");
    for(i=0; i<2; i++){
        if(i==1){
            parity=1;
            nrzm=1;
            printf("Parity mode is assumed in data format analysis..\n");
        }
        if(mk5_dat_anal(mk5file, validtrk, trktable, hstack_tbl, 
            header, &fsampl, istart, &ntrack, &bitsframe, parity, nrzm, dispmode,
            vcntable) < 0){
            if(i==0){
                printf("Data format is not a non-parity mode.\n");
            }
            if(i==1){
                printf("Data format is not a parity mode.\n");
            }
        } else {
            break; //  meaning data found
        }
    }
    // $DATAMODE
    fprintf(fo,"$DATAMODE;   * Mark-V data format\n");
    if(ntrack==0){
        printf("Mark-V data could not be analyzed!\n"); 
        fprintf(fo,"** Mark-V data could not be analyzed!\n"); 
    } else {
        printf("Analyzed Mark-V data format is as follows\n");
        if(parity==1){
           printf("  data encode -- NRZM  with parity\n");
           fprintf(fo,"   parity = 1 ;  * with parity\n");
           fprintf(fo,"   nrzm   = 1 ;  * NRZM encoding\n");
        } else {
           printf("  data encode -- NRZL  without parity\n");
           fprintf(fo,"   parity = 0 ;  * non-parity\n");
           fprintf(fo,"   nrzm   = 0 ;  * NRZL encoding\n");
        }
        if(bitsframe==20160){  // VLBA format
           printf("  data format -- VLBA\n");
           fprintf(fo,"   format = VLBA ; * VLBA format\n");
        } else {
           printf("  data format -- Mark-III/IV\n");
           fprintf(fo,"   format = Mark-IV ; * Mark-III or IV format\n");
        }
        printf("  #_of_track  = %d\n",ntrack);
        fprintf(fo,"   ntrack = %d ; * # of tracks (bits/word)\n",ntrack);
    }
    // $BITPOS
    fprintf(fo,"$BITPOS;   * bit position versus track information\n");
    if(ntrack==0){
        printf("Mark-V data could not be analyzed!\n"); 
        fprintf(fo,"** Mark-V data could not be analyzed!\n"); 
    } else {
        //printf("$BITPOS;   * bit position versus track information\n");
        fprintf(fo,"**\n");
        fprintf(fo,"**     bb   => bit position #\n");
        fprintf(fo,"**     h-tt => h: head stack #, tt: track #\n");
        fprintf(fo,"** bitpos = bb : h-tt\n");
        //printf("** bitpos = bit_pos# : track #\n");
        //for(n=0; n<32; n++){
        for(n=0; n<ntrack; n++){  // 2003-07-12
           if(validtrk[n]==0){
               //fprintf(fo,"   bitpos = %02d : -- ;\n",n);
               fprintf(fo,"   bitpos = %02d : ---- ;\n",n);
               //printf("   bitpos = %02d : -- ;\n",n);
           } else {
               //fprintf(fo,"   bitpos = %02d : %02d ;\n",n,trktable[n]);
               fprintf(fo,"   bitpos = %02d : %d-%02d ;\n",n,
                   hstack_tbl[n],trktable[n]);
              //printf("   bitpos = %02d : %02d ;\n",n,trktable[n]);
           } 
        } 
    }

    // $GROUP    default group set
    fprintf(fo,"$GROUP;   * group # versus channel # table\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**     Please edit this table as you like    **\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**\n");
    fprintf(fo,"**     g   => group #\n");
    fprintf(fo,"**     ch1 => 1st channel # in this group\n");
    fprintf(fo,"**     ch2 => 2nd channel # in this group\n");
    fprintf(fo,"**     ch3 => 3rd channel # in this group\n");
    fprintf(fo,"**     ch4 => 4th channel # in this group\n");
    fprintf(fo,"** group = g : ch1 : ch2 : ch3 : ch4 ;\n");
    if(track_freq.nfreq == 14){  // Mark4 Westford
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :   9 :  10 ;\n");
       fprintf(fo,"   group = 4 :  11 :  12 :  13 :  14 ;\n");
    } else {
       fprintf(fo,"   group = 1 :   1 :   2 :   3 :   4 ;\n");
       fprintf(fo,"   group = 2 :   5 :   6 :   7 :   8 ;\n");
       fprintf(fo,"   group = 3 :   9 :  10 :  11 :  12 ;\n");
       fprintf(fo,"   group = 4 :  13 :  14 :  15 :  16 ;\n");
    }
    i=fclose(fo) ; //2003-10-10

}

void make_k5tom5_info_file(k5m5runp_t *runp) // 2008-09-26
/* 
void make_k5tom5_info_file(char *info_file, char* vex_file, 
                           char *sid, int scan, char* program_version,
                           char* dflt_pass)

 ** ch = ch# : sign_tr# : mag_tr# : sign_tr# : mag_tr#
    ch = 01 : 1-02 : 1-10 : 1-04 : 1-12 ;

      where  "1-02" :  1 means head stack#



      create an file containing information necessary
      for conversion from K5 to Mark-V format


      If sid is not given or "--", station selection will be 
      made interactively.
       
      Ver 1.10 2005-01-12
               2008-07-04
               2008-09-26  add k5 channel information in $CHANNEL block

   [in] 
      runp.info_file    ---  information file name
      runp.vex_file     ---  VEX file name
      runp.sid          ---  station ID
      runp.scan         ---  scan # to get MODE
      runp.program_version -- program version
      runp.dflt_pass    ---  default pass


*/
{
    //char sid[10]; // station ID (usually 2char)
    char mode[30];
    char cbuf[30];
    char chanid[20];
    //char mk5file[80];
	//int validtrk[32],trktable[32], header[160], istart[6];
	//int validtrk[64],trktable[64], header[320], istart[6];  //2003-10-10
    //int hstack_tbl[64];
    char sideband[5];
    int hstack,trk;
    int i,n,nstation,nmax, npos;
    int iok;
    int bbc;
    int scan_n;   // scan # for getting MODE
    int chtr[8];
    int hstr[8]; //2004-09-13  
    int ntrack;
      //ntrack -------- # of tracks (8,16,32,64) or 0 (for no meaning data)
    int bitsframe, parity, nrzm;
      //parity   ----  1: assume with parity   0: without parity
      //nrzm     ----  1: assume nrzm          0: nrzl
      //bitsframe  ---- # of bits/frame   20000: Mark-IV format
      //                                  20160: VLBA format
    int vlba;     // 1: VLBA  0: Mark-IV
    int modulation;  // 1: modulation on    0: modulation off
    int dispmode=0;  // data analysis display mode 0: old style 1: new style
    int k5ch; // k5 channel #  2008-09-26
    FILE *fo;
    time_t tnow;
    vex_t vex;
    site_t site;
    sidtable_t sidtable[20];
    track_freq_t track_freq;

    strcpy(vex.vex_file_name, runp->vex_file);

    /* analyze VEX file */

    vex_read(&vex);

    /* information file open to write */

    if( (fo  = fopen( runp->info_file, "w" )) == NULL ){
       printf("File (%s) Open Error! \n",runp->info_file);
       exit(1);
    }

    /* station selction */

    create_siteid_table(&vex,&nstation,sidtable);
    printf("SITES (Station ID) defined are\n");
    printf("\n");
    printf("  ID    SITE NAME\n");
    printf(" -----------------\n");
    for(i=0;i<nstation;i++){
        printf("  %2s    %s\n",sidtable[i].sid,sidtable[i].site);
    }
    printf(" -----------------\n");
    if(runp->sid[0]=='\0' || runp->sid[0]=='-'){  // interactive operation
tryagain:
        printf("Select Station by ID   ----> ");
        scanf("%s",&cbuf);
        //i=atoi(cbuf);
        //strcpy(sid,sidtable[i-1].sid);
        strcpy(runp->sid,cbuf);

        //check code is OK or not
        iok=0;
        for(i=0;i<nstation;i++){
            if(strncmp(sidtable[i].sid,runp->sid,strlen(runp->sid))==0){
                iok=1;
                break;
            }
        }
        if(iok==0){
            printf(" ID(%s) is not included in the table! Input again!\n",runp->sid);
            goto tryagain;
        }
    } else { // non interactive operation
        printf("Selected Station ID = ");
        printf("%s\n",runp->sid);
    }

    /* search and get site information for sid */

    if(search_site(&vex,runp->sid,&site)==0){
       printf("\nDetailed site information\n");
       printf("   site definition :  %s\n",site.def);
       printf("   site name       :  %s\n",site.name);
       printf("   site ID         :  %s\n",site.id);
       printf("   site position   : %f %f %f\n",site.position[0],
                        site.position[1],site.position[2]);
       printf("   site clock\n");
       printf("      validity epoch : %d %d %d %d %d\n",
                   site.sclock.validity_epoch[0],
                   site.sclock.validity_epoch[1],
                   site.sclock.validity_epoch[2],
                   site.sclock.validity_epoch[3],
                   site.sclock.validity_epoch[4]);
       printf("      clock epoch    : %d %d %d %d %d\n",
                   site.sclock.clock_epoch[0],
                   site.sclock.clock_epoch[1],
                   site.sclock.clock_epoch[2],
                   site.sclock.clock_epoch[3],
                   site.sclock.clock_epoch[4]);
       printf("      clock offset   : %e\n",site.sclock.clock_offset);
       printf("      clock rate     : %e\n",site.sclock.clock_rate);

    } else {
        printf("Selected Station ID (%s) is not included in the VEX file\n",runp->sid);
        exit(0);  // stop
    }

    /* mode selection from given scan # */

    scan_n=runp->scan;   // default scan#
    //printf("numobs scan_n %d %d\n",vex.numobs,scan_n); //debug
    if(scan_n > vex.numobs) {
        printf("  Given scan # (%d) for MODE get exceeds maximum scan # (%d)\n",
            runp->scan,vex.numobs);
        printf("  so scan # for mode get was changed to 1\n");
        scan_n=1;
    }
    if(scan_n < 0) {
        printf("  Given scan # (%d) for MODE get is illegal,\n",runp->scan);
        printf("  so scan # for mode get was changed to 1\n");
        scan_n=1;
    }
    // check this scan includes given station or not  added on 2005-01-13
    iok=0;
    for(n=scan_n-1; n< vex.numobs; n++){
        for(i=0; i< vex.scan[n].numstat; i++){
            if(strncmp(vex.scan[n].station[i].id,runp->sid,strlen(runp->sid))==0){
                if(vex.scan[n].station[i].data_stop > 0){  //  OK
                    iok=1;
                    break;
                }
            }
        }
        if(iok==1) break;
    }
    if(iok==1){
        scan_n=n+1;
        printf("  Scan # for mode get is %d\n",scan_n);
    } else {
        printf("  Valid scan for given station ID (%s) was not found!\n",runp->sid);
        printf("  Anyway we use scan # of %d for getting mode\n",scan_n);
    }

    strcpy(mode,vex.scan[scan_n-1].mode);
    printf("mode is %s\n",mode);
    printf("Mode was taken from Scan #%d as %s\n",scan_n,mode);


    /* search and get track and freq information for sid */

    printf("\nTRACK and FREQUENCY information for %s\n",site.name);
    if(search_track_freq(&vex,mode,runp->sid,&track_freq)==0){
          printf("   BARREL ROLL : %s\n",track_freq.roll);
          printf("   FREQDEF = %s   TRACKDEF = %s\n",track_freq.freqdef,track_freq.trackdef);
          printf("   adbit= %d sample_rate= %f\n",track_freq.adbit, track_freq.samplrate);
          printf("\n");
          printf("   bb HS Tr   AD fo   chan  bbcid BBC#     RF(Hz)    S   VBW(Hz)    PASS\n");
          //                 HS : Head Stack #
          //                            fo : fan out
          for(i=0; i<track_freq.ntrack; i++){
              printf("   %2d %2d %2d %4s %2d %6s %6s   %2d %13.1f %s %11.1f %s\n",i+1, 
                  track_freq.trk[i].headstack,  //2004-09-13
                  track_freq.trk[i].track,
                  track_freq.trk[i].bitstream,track_freq.trk[i].fan_out_pos,
                  track_freq.trk[i].chan_id,
                  track_freq.trk[i].bbcid,track_freq.trk[i].bbc,
                  track_freq.trk[i].rf,
                  track_freq.trk[i].sideband,track_freq.trk[i].vbw,
                  track_freq.trk[i].sub_pass);
          }
    }

    /* write information on info_file */

    tnow=time(NULL);
    fprintf(fo,"*** k5tom5 information file created by %s\n",runp->program_version);
    fprintf(fo,"***       on %s",ctime(&tnow));
    fprintf(fo,"***   (head stack number included in track info)\n");
    fprintf(fo,"***   analyzed VEX file    : %s\n",runp->vex_file);
    fprintf(fo,"***   station : %s    (%s)\n",site.name, site.id);
    fprintf(fo,"***   mode (for scan # %d)  : %s\n",scan_n,mode);
    fprintf(fo,"*** \n");

    // $CHANNEL
    fprintf(fo,"$CHANNEL;  * channel-track info block \n");
    fprintf(fo,"   adbit  = %d ;  *   A/D resolution\n",track_freq.adbit);
    fprintf(fo,"   sample = %f ;  *   Sampling frequency\n",track_freq.samplrate);
    fprintf(fo,"   fanout = %d ;  *   Fanout\n",track_freq.fanout);
    fprintf(fo,"** default pass = %s\n",runp->dflt_pass);
    fprintf(fo,"**\n");
    fprintf(fo,"**     nn   => channel # (Mark5)\n");
    fprintf(fo,"**     h-ss => h: head stack #, ss: sign bit track #\n");
    fprintf(fo,"**     h-mm => h: head stack #, mm: magnitude bit track #\n");
    fprintf(fo,"**     bbc# => BBC#\n");
    fprintf(fo,"**     sb   => sideband  L(SB) or U(SB)\n");
    fprintf(fo,"**     k5ch => K5 channel # \n");
    fprintf(fo,"******************************************************\n");
    fprintf(fo,"**     Please edit k5ch in the table as you like    **\n");
    fprintf(fo,"******************************************************\n");
    //fprintf(fo,"** ch = ch# : sign_tr# : mag_tr# :  sign_tr# : mag_tr# : .....\n");
    if(track_freq.adbit==1){
        strcpy(cbuf," : h-ss");
    } else {
        strcpy(cbuf," : h-ss : h-mm");
    }
    fprintf(fo,"** ch = nn");
    for(n=0; n<track_freq.fanout; n++) fprintf(fo,"%s",cbuf);
    //fprintf(fo," : bbc# : sb\n");
    fprintf(fo," : bbc# : sb : k5ch\n");  // 2008-09-26
    // 
    nmax=track_freq.adbit*track_freq.fanout;
    // check chan id (number)  &Ch01 or &CH01 type
    //printf("Debug #1 %d %d\n",track_freq.ntrack, nmax);
    for(n=0; n<track_freq.nfreq; n++){
        sprintf(cbuf,"%02d\0",n+1);
        for(i=0; i<8; i++) chtr[i]=0;
        for(i=0; i<8; i++) hstr[i]=0;  //2004-09-13

        for(i=0; i<track_freq.ntrack; i++){
            strcpy(chanid,track_freq.trk[i].chan_id);
            if(strstr(chanid,cbuf) != NULL){
          //      npos=track_freq.fanout*(track_freq.trk[i].fan_out_pos-1);
                npos=track_freq.adbit*(track_freq.trk[i].fan_out_pos-1); //2003-10-10
  //  printf("Debug n = %d, i= %d npos=%d  %d %s\n",n,i,npos,
  //      track_freq.trk[i].fan_out_pos,track_freq.trk[i].bitstream);

                // check sub pass is defined in a vex file
                if(track_freq.trk[i].sub_pass[0]== '\0' || 
                    track_freq.trk[i].sub_pass[0]== ' '){ 
                   if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                       chtr[npos]=track_freq.trk[i].track;
                       hstr[npos]=track_freq.trk[i].headstack; //2004-09-13
                   } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                       chtr[npos+1]=track_freq.trk[i].track;
                       hstr[npos+1]=track_freq.trk[i].headstack; //2004-09-13
                   } 
                } else {
                   if(track_freq.trk[i].sub_pass[0]==runp->dflt_pass[0]){ // 2004-06-29
                      if(strncmp(track_freq.trk[i].bitstream,"sign",4)==0){
                          chtr[npos]=track_freq.trk[i].track;
                          hstr[npos]=track_freq.trk[i].headstack; //2004-09-13
                      } else if(strncmp(track_freq.trk[i].bitstream,"mag",3)==0){
                          chtr[npos+1]=track_freq.trk[i].track;
                          hstr[npos+1]=track_freq.trk[i].headstack; //2004-09-13
                      } 
                   } 
                }
                bbc=track_freq.trk[i].bbc; //2004-12-25
                strcpy(sideband,track_freq.trk[i].sideband); //2004-12-25
                k5ch=n+1; // 2008-09-26
            }
        }
        //fprintf(fo,"   ch = %02d : %02d ",n+1,chtr[0]);
        fprintf(fo,"   ch = %02d : %d-%02d ",n+1,hstr[0],chtr[0]);
        if(nmax >=2) {
            for(i=1; i<nmax; i++){
                fprintf(fo,": %d-%02d ",hstr[i],chtr[i]);
            }
        }
        //fprintf(fo,":   %2d : %s ",bbc, sideband); // 2004-12-25
        fprintf(fo,":   %2d :  %s : %2d ",bbc, sideband, k5ch); // 2009-9-26

        fprintf(fo,";\n");
    }

    /* set data file format */

    parity=runp->parity;
    nrzm=runp->nrzm;
    modulation=runp->modulation;
    vlba=runp->vlba;

    if(vlba==1){   // VLBA mode   (2008-07-04)
       parity=1;  // non-parity
       nrzm=1;    // NRZL coding
       bitsframe=20160; // VLBA frame format
       //modulation=1;  // modulation off
    } else {
       parity=0;  // non-parity
       nrzm=0;    // NRZL coding
       bitsframe=20000; // MarkIV frame format
       //bitsframe=20160; // VLBA frame format
       //modulation=0;  // modulation off
    }

    ntrack=track_freq.nfreq*nmax;
    
    if(ntrack <= 16){
        ntrack=16;
    } else if(ntrack <= 32){
        ntrack=32;
    } else {
        ntrack=64;
    }
    // $DATAMODE
    fprintf(fo,"**\n");
    fprintf(fo,"$DATAMODE;   * Mark-V data format\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**     Please edit this table as you like    **\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**\n");
    printf("default Mark-V data format is as follows\n");
    if(parity==1){
       printf("  data encode -- NRZM  with parity\n");
       fprintf(fo,"   parity = 1 ;       * 1: with parity, 0: non-parity \n");
       fprintf(fo,"   nrzm   = 1 ;       * 1: NRZM encode, 0: NRZL encode\n");
    } else {
       printf("  data encode -- NRZL  without parity\n");
       fprintf(fo,"   parity = 0 ;       * 1: with parity, 0: non-parity \n");
       fprintf(fo,"   nrzm   = 0 ;       * 1: NRZM encode, 0: NRZL encode\n");
    }
    if(bitsframe==20160){  // VLBA format
       printf("  data format -- VLBA\n");
       fprintf(fo,"   format = VLBA ;    * Mark-IV | VLBA\n");
    } else {
       printf("  data format -- Mark-IV\n");
       fprintf(fo,"   format = Mark-IV ; * Mark-IV | VLBA\n");
    }
    printf("  #_of_track  = %d\n",ntrack);
    fprintf(fo,"   ntrack = %d ;      * # of tracks (bits/word)\n",ntrack);
    if(modulation == 1){
        printf("  data modulation -- ON\n");
        fprintf(fo,"   modulation = ON ;  * modulation ON | OFF\n");
    } else {
        printf("  data modulation -- OFF\n");
        fprintf(fo,"   modulation = OFF ; * modulation ON | OFF\n");
    }
    // $BITPOS
    fprintf(fo,"**\n");
    fprintf(fo,"$BITPOS;   * bit position versus track information\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**     Please edit this table as you like    **\n");
    fprintf(fo,"***********************************************\n");
    fprintf(fo,"**\n");
    //printf("$BITPOS;   * bit position versus track information\n");
    fprintf(fo,"**     bb   => bit position #\n");
    fprintf(fo,"**     h-tt => h: head stack #, tt: track #\n");
    fprintf(fo,"** bitpos = bb : h-tt\n");
    //printf("** bitpos = bit_pos# : track #\n");
    //for(n=0; n<32; n++){
    hstack=1;
    trk=2;
    for(n=0; n<ntrack; n++){  // 2003-07-12
        if(n==32) {
            hstack=2;
            trk=2;
        }
        fprintf(fo,"   bitpos = %02d : %d-%02d ;\n",n,
               hstack,trk);
        //printf("   bitpos = %02d : %02d ;\n",n,trktable[n]);
        trk++;
    } 
    fprintf(fo,"**\n");
    i=fclose(fo) ; 

}

void get_bit_track(mk5tok5_info_t *mk5info, int numch, int *ch_table, 
//                   int *trktable, int *h_stack, int *bit_trk)
                   int *bit_trk)
{
/*
   Get bit position table corresponding to track
   2003-07-16
   2003-10-10   allow 2-bit 4 fan out and 64 track mode
   2004-09-13   add use of head stack info
   2004-11-14   debug on bit position order in case of 2bit sampling

     [in]
        mk5info  -- Mark 5 track info;
        numch     -- converting number of channel (1 or 4)
        ch_table[16] -- channel number converting to K5 format
        //trktable[32]-- Mark5 track# table for bit position 0-31
        trktable[64]-- Mark5 track# table for bit position 0-63
        h_stack[64]-- Mark5 head strack# table for bit position 0-63
     [return]
        //bit_trk[16] --- bit position (0-31) in Mark5 corresponding 
        bit_trk[64] --- bit position (0-63) in Mark5 corresponding 
                      to the channel given by ch_table[4]
                      (suport 2bit fanout 4)

*/
    int n, k, k1, kk, iw, m;
    int num_tracks_per_ch, tot_track;
    //int work[16];
    int hs[64];
    int trk[64];  
    // number of tracks/ch  is  adbit*fanout
    num_tracks_per_ch = mk5info->adbit*mk5info->fanout;
    // printf("number of tracks per channel = %d\n",num_tracks_per_ch);
    tot_track=numch*num_tracks_per_ch;

    // setup bit position table corresponding to track

    for(n=0; n<64; n++) hs[n]=0;

    m=0;
/*     obsoleted 2003-07-16
    for(n=0; n<numch; n++){
       iw=ch_table[n];
       for(kk=0; kk<mk5info->nch; kk++){
              if(iw == mk5info->track_channel[kk]){
                  for(k1=0; k1<mk5info->fanout; k1++){
                      bit_trk[m]=mk5info->signtrack[kk][k1];
                      m++;
                      if(mk5info->adbit==2){
                          bit_trk[m]=mk5info->magtrack[kk][k1];
                          m++;
                      }
                  }  
              }  
       } 
    }
*/
    for(k1=0; k1<mk5info->fanout; k1++){
       for(n=0; n<numch; n++){
           iw=ch_table[n];
           for(kk=0; kk<mk5info->nch; kk++){
               if(iw == mk5info->track_channel[kk]){
                  
                   // obsoleted on 2004-11-14
                   //trk[m]=mk5info->signtrack[kk][k1];
                   //hs[m]=mk5info->hs_sign[kk][k1];  //2004-09-13
                   //m++;
                   //if(mk5info->adbit==2){
                   //    trk[m]=mk5info->magtrack[kk][k1];
                   //    hs[m]=mk5info->hs_mag[kk][k1];  //2004-09-13
                   //    m++;
                   //}
                   /* above is not correct (found on 2004-11-14) */
                   /* corrected as follows  (if adbit=2, magnitude bit 
                      must be LSB side    */
                   if(mk5info->adbit==2){
                       trk[m]=mk5info->magtrack[kk][k1];
                       hs[m]=mk5info->hs_mag[kk][k1];  
                       m++;
                   }
                   trk[m]=mk5info->signtrack[kk][k1];
                   hs[m]=mk5info->hs_sign[kk][k1];  
                   m++;
               }   
           }   
       }  
    }
       // now bit_trk[N] is Track# for ch N 
       // convert bit_trk[N] to bit position (0-31) table 

    for(n=0; n<tot_track; n++) {
        //work[n]=bit_trk[n];
        bit_trk[n]=-1;
    }


      //printf("m,tot_track %d %d\n",m,tot_track);
      //printf("Before: bit_trk[n]\n");
      //for(n=0; n<tot_track; n++) printf(" %d",bit_trk[n]);
      //printf("\n");
      for(n=0; n<tot_track; n++){
           //for(k=0; k<32; k++){
          for(k=0; k<64; k++){  //2003-10-10
               //if(trk[n]==trktable[k]){   //trktable shows bitpos vs Track#
              //if(trk[n]==trktable[k] && hs[n]==h_stack[k]){  //2004-09-13
              if(trk[n]==mk5info->bit_track[k] && 
                  hs[n]==mk5info->bit_hs[k]){  //2004-09-13
                    bit_trk[n]=k;
                    break;
               }
           }
       }
  //    printf("After: bit_trk[n]\n");
  //    for(n=0; n<tot_track; n++) printf(" %d",bit_trk[n]);
  //    printf("\n");
}

void parse_dir_and_name(char *fpath, char *dir, char *fname) //2003-03-23
{
    int len,i,k;
    char cbuf[256];  // 2007-07-17  128->256
    len=strlen(fpath);
    // converting '\' to '/' if so
    for(i=0; i<len; i++){
       if(fpath[i]=='\\'){
            cbuf[i]='/';
       } else {
            cbuf[i]=fpath[i];
       }
    }
    cbuf[len]='\0';
    k=-1;
    for (i=len-1; i>=0; i--){
        if(cbuf[i] == '/') {
            k=i;
            break;
        }
    }
    //printf("debug %d %s\n",k,cbuf);
    if(k<0){
        dir[0]='\0';
        strcpy(fname,cbuf);
    } else if (k == len-1){
        strcpy(dir,cbuf);
        fname[0]='\0';
    } else {
        strncpy(dir,cbuf,k+1);
        dir[k+1]='\0';
        strcpy(fname,cbuf+k+1);
    }
    //printf("debug %d %s\n",k,dir);
}



void k5name_make(char *mk5name, char *outdir, int channel, int group, 
                 char *k5name)
{
/*
  ftHg̍쐬jTt@C̃[
           (MK5͌Mark5t@C)
       S[h
            MK5.k5a   ---- ch1 - ch4
            MK5.k5b   ---- ch5 - ch8
            MK5.k5c   ---- ch9 - ch12
            MK5.k5d   ---- ch13 - ch16
       P[h
            MK5.k5NN --- NN͂Qchԍ 01-16
*/
    int i,len;
    //char k5dir[80],m5dir[80], m5name[20],fname[20];
    //char k5dir[80],m5dir[80], m5name[80],fname[80];  // 2006-09-27
    char k5dir[256],m5dir[256], m5name[256],fname[256];  // 2007-07-18
    char cbuf[10];
    char ag[4][2]={"a","b","c","d"};

    parse_dir_and_name(mk5name, m5dir, m5name);


    // out directory prepare
    if(outdir[0] == '\0'){
        k5dir[0]='\0';
    } else {
        // converting '\' to '/' if so
        len=strlen(outdir);
        //printf("C %d %s\n",len,outdir);
        for(i=0; i<len; i++){
            if(outdir[i]=='\\'){
                k5dir[i]='/';
            } else {
                k5dir[i]=outdir[i];
            }
        }
        k5dir[len]='\0';
        // check apri dir name accompanied by "/" or not 
        if(k5dir[len-1] != '/') {
           k5dir[len]='/';
           k5dir[len+1]='\0';
        }
    }
   // create k5name
    strcpy(fname,m5name);
    strcat(fname,".k5");
    if(channel > 0) {
        sprintf(cbuf,"-%02d\0",channel);
        strcat(fname,cbuf);
    } else {
        if(group > 0 && group <5) {
           strcat(fname,ag[group-1]);
        }
    }
    strcpy(k5name,k5dir);
    strcat(k5name,fname);
}

void monit_mk5info(mk5tok5_info_t *mk5info)
/*
      monitor mk5info
*/
{
    int n,k,k1,k2;
    printf("******** mk5info monitor *********\n");
    printf("m5tok5mode: %d\n",mk5info->m5tok5mode);
    printf("nch       : %d\n",mk5info->nch);
    printf("adbit     : %d\n",mk5info->adbit);
    printf("fanout    : %d\n",mk5info->fanout);
    printf("samplrate : %f\n",mk5info->samplrate);
    printf("ngroup    : %d\n",mk5info->ngroup);
    printf("ntrack    : %d\n",mk5info->ntrack);
    printf("parity    : %d\n",mk5info->parity);
    printf("nrzm      : %d\n",mk5info->nrzm);
    printf("vlba      : %d\n",mk5info->vlba);
    printf("bitsframe : %d\n",mk5info->bitsframe);
    printf("modulation: %d\n",mk5info->modulation);

    printf("track_channel, signtrack, magtrack :\n");
    printf("    n   CH");
    for(k=0; k<mk5info->fanout; k++){
       printf("   sign");
       if(mk5info->adbit >=2){
          printf(" magbit");
       }
    }
    printf("\n");
    for(n=0; n<mk5info->nch; n++){
       printf("   %2d   %2d",n+1, mk5info->track_channel[n]);
       for(k=0; k<mk5info->fanout; k++){
           //printf("%9d", mk5info->signtrack[n][k]);
           printf("%4d-%2d", mk5info->hs_sign[n][k],   // 2004-09-13
               mk5info->signtrack[n][k]);
           if(mk5info->adbit >=2){
              //printf("%9d", mk5info->magtrack[n][k]);
              printf("%4d-%2d",mk5info->hs_mag[n][k],  //2004-09-13
                  mk5info->magtrack[n][k]);
           }
       }
       printf("\n");
    }
       
    printf("bit_track :\n");
    k1=0; k2=16;
    while(1){
       if(k2 <= mk5info->ntrack) {
          printf("   bitpos# ");
          for(n=k1; n<k2; n++) printf("%3d",n); printf("\n");
          printf("    track# ");
          for(n=k1; n<k2; n++) printf("%3d",mk5info->bit_track[n]);
          printf("\n");
          printf(" hd_stack# ");
          for(n=k1; n<k2; n++) printf("%3d",mk5info->bit_hs[n]);
          printf("\n");
          k1=k2;
          k2+=16;
       } else {
           break;
       }
    }
    
    if(mk5info->m5tok5mode==1){ // Mark5->K5 mode
        printf("group_channel :\n");
        for(n=0; n<4; n++){
            printf("   group=%2d  ch# ",n+1);
            for(k=0; k<4; k++) printf("%02d ", mk5info->group_channel[n][k]);
            printf("\n");
        }
    } else {

        printf("for K5 to M5 :\n");
        printf("  K5ch#  Mark5ch#  bbc# sideband\n");
        for(n=0; n<mk5info->nch; n++){
            //printf("  %5d  %8d  %4d %4d\n", n+1,
            printf("  %5d  %8d  %4d %4d\n", mk5info->k5ch[n],  // 2008-09-26
                mk5info->ch_ch[n],mk5info->ch_bbc[n],mk5info->ch_lsb[n]);
                //mk5info->track_channel[n],mk5info->ch_bbc[n],mk5info->ch_lsb[n]);
           
        }
    }
    printf("******** mk5info monitor end ******\n\n");
}


int Grinfo_fm_Mk5info(mk5tok5_info_t *mk5info, gr_info_t *gr)
{
/*
      Set up table necessary to pick up Mark5 data
     
    [in]
      mk5info
      gr->channel  ---  channel # to pickup
      gr->group    ---  group # to pick up
                        channel # parameter is stronger than group #
                        Only in the case of channel=0, group parameter works
      gr->reverse  ---  reversal flag
    [return]
      0  ---   normal end
      -1 ---   nothing to do (parameter range is wrong)
      gr->num_tracks_per_ch --  number of tracks per channel
      gr->tot_track --  total number of tracks 
      gr->numch     --  number of channels  (1 or 4)
      gr->bit_trk[] ---  bit position table for conversion
      gr->validtrk[]  ---  valid track table 
      gr->ch_table[]  ---  channel number table
*/
       int i,k,n,m;
       int work[64];

       if(gr->group >4) return -1;
       if(gr->group > mk5info->ngroup) return -1;
       // setup processing channel number table
       gr->numch=4;

       if(gr->channel !=0 ) {
           gr->numch=1;
           gr->ch_table[0]=gr->channel;
       } else {
           for(n=0; n<4; n++){
               gr->ch_table[n]=mk5info->group_channel[gr->group-1][n];
               //if(ch_table[n]==0) ch_table[n]=mk5info.nch;
           }
       }
       if(gr->numch!=1 && gr->channel==0 ){
           // if all channel ==0 next group
           if(gr->ch_table[0]==0 || gr->ch_table[1]==0 
               || gr->ch_table[2]==0 || gr->ch_table[3]==0) {
               return -2;
           }
       }
       // number of tracks/ch  is  adbit*fanout
       gr->num_tracks_per_ch = mk5info->adbit*mk5info->fanout;
       //printf("number of tracks per channel = %d\n",num_tracks_per_ch);
       gr->tot_track=gr->numch*gr->num_tracks_per_ch;

       // setup bit position table corresponding to channel track
       get_bit_track(mk5info, gr->numch, gr->ch_table, gr->bit_trk);
      
       if(gr->reverse==1){ // track order reverse
          if(mk5info->fanout>1){
             for(n=0;n<gr->numch;n++){
                 for(k=0;k<gr->num_tracks_per_ch;k++) {
                   work[k]=gr->bit_trk[n*gr->num_tracks_per_ch+k];
                 }
                 if(mk5info->adbit==1){
                    for(k=0;k<gr->num_tracks_per_ch;k++) {
                        gr->bit_trk[n*gr->num_tracks_per_ch+k]=work[gr->num_tracks_per_ch-k-1];
                    }
                 } else {  // adbit==2
                    for(k=0;k<gr->num_tracks_per_ch/2;k++) {
                        gr->bit_trk[n*gr->num_tracks_per_ch+2*k]=work[gr->num_tracks_per_ch-2*k-2];
                        gr->bit_trk[n*gr->num_tracks_per_ch+2*k+1]=work[gr->num_tracks_per_ch-2*k-1];
                    }
                 }
             }
          }
       }


       // bit track validity check to avoid error just in case
       
       //for(i=0; i<32; i++){
       for(i=0; i<64; i++){
          if(mk5info->bit_valid[i] == 1) {
             m =i;
             break;
          }
       }
       // now m is 1st valid track bit position
       for(n=0; n<gr->tot_track; n++){
           //if(bit_trk[n]<0 || bit_trk[n] >=32){
           if(gr->bit_trk[n]<0 || gr->bit_trk[n] >=64){
               // substitute 1st valid track data
               gr->bit_trk[n] =m;
           } else {
               if(mk5info->bit_valid[gr->bit_trk[n]] ==0) gr->bit_trk[n]=m;
           }
       }

       //printf("bit_trk information \n");
       //for(n=0; n<gr->tot_track; n++){
       //    printf("%d - %d\n",n,gr->bit_trk[n]);
       //}
 	   return 0;
}


void monit_bitpos_info(mk5tok5_info_t *mk5info, gr_info_t *gr)
/*   monitor bit position information for conversion  */
{
    int i,j,k,m,k5ch,m5ch,numch;

    // argument order of bit_trk array
    // bit_trk[adbit,numch,fanout] 
    // bit_trk[i,j,k] = bit_trk[i+j*adbit+k*adbit*numch]

    numch=1;
    if(gr->channel ==0) numch=4; // Group conversion (4ch) mode
    printf("\n");
    printf("   bit position information for conversion\n");
    printf("  K5CH#  Mark5 CH#     Mark5 bit position#\n");
    printf("   ----     ----      ----------------------\n");

    for(j=0; j<numch; j++){
        if(gr->channel ==0){  // Group conversion (4ch) mode
           k5ch=numch*(gr->group-1)+j+1;   // K5 CH#
           m5ch=mk5info->group_channel[gr->group-1][j];
        } else {
           k5ch=gr->channel;                 // K5 CH#
           m5ch=k5ch;
        }
        printf("   %4d     %4d      ",k5ch,m5ch);
        for(k=0; k<mk5info->fanout; k++){
            for(i=0; i<mk5info->adbit; i++){ 
                m=i+j*mk5info->adbit+k*mk5info->adbit*numch;
                printf("%4d",gr->bit_trk[m]);
            }
            printf(" : ");
        }
        printf("\n");
    }
    printf("   ----     ----      ----------------------\n\n");
}


int head180_to_time(mk5_read_t *mk5, int tr, int *itime)
/*   pick up time from header array with sync block check */
{
    int n;
    int offset1,offset2; 
    int tdat[72];
    int tmax;
    int iret;
    int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                     1,1,1,1,1,1,1,1,0,
                     1,1,1,1,1,1,1,1,0,
                     1,1,1,1,1,1,1,1,0};
    int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                     1,1,1,1,1,1,1,1,
                     1,1,1,1,1,1,1,1,
                     1,1,1,1,1,1,1,1};
    int *sync_pat;

    iret=0;
    if(mk5->parity==1) {
        sync_pat=sync_pat1;
    } else {
        sync_pat=sync_pat0;
    }

    /* check block data with sync pattern */
    offset1=8*mk5->nbitbyte;
    for(n=0; n<mk5->nbitword; n++){
        if(sync_pat[n] != mk5->head180[(offset1+n)*mk5->tot_track+tr]) iret=-1;
    }

    /* pick up time data */

    offset2=12*mk5->nbitbyte;  // 108 : with parity   96 : without parity
    tmax=8*mk5->nbitbyte;    // 72 : with parity   64 : without parity

    for(n=0; n<tmax; n++){
        tdat[n]=mk5->head180[(offset2+n)*mk5->tot_track+tr];
    }
    //if(mk5->parity==1){
    //   conv2time2(tdat, itime);
    //} else {
    //   conv2time2n(tdat, itime);
    //}
    conv2time_full(tdat,itime,mk5->parity,mk5->vlba); 
    return iret;
}


int conv2k5(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp)
/* general purpose conversion  2004-09-20 */
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;
    int ih;
    int know;
    int i1st;
    int ipick;
    int i,n,k,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    int64_t iwork64,know64;
    IPBOARD_T ipb;
    FILE *fo;
    mk5_read_t mk5;

    printf("  conv2k5 : \n");
    ierr=0;

    /* initialize mk5_read_t type variable */
	mk5.parity=mk5info->parity;
    if(mk5.parity==1){  // with parity
        mk5.nbuf=900; //ǂ݂Ƃv@
	                 //peB̏ꍇX̔{ɂƋ؂肪悭֗
                     //    ȂȂ΁@f[^͂WrbgɃpeBPrbg
        mk5.nbitbyte=9; // # of bits in a byte
        mk5.nbitword=36; // # of bits in a header word
        mk5.nbithead=180; // # of bits in header
    } else {             // without parity
	    mk5.nbuf=800;
        mk5.nbitbyte=8; 
        mk5.nbitword=32; 
        mk5.nbithead=160; 
    }
	mk5.nrzm=mk5info->nrzm;
	mk5.ntrack=mk5info->ntrack;
	//mk5.ibufsize=1800; // ibuf size in 32bit-word (int) 
	mk5.ibufsize=mk5.nbuf*mk5.ntrack/32; // ibuf size in 32bit-word (int) 
	mk5.ibuf=(unsigned int *)malloc(mk5.ibufsize*sizeof(int));
    mk5.sbuf=(unsigned short *)mk5.ibuf; 
    mk5.cbuf=(unsigned char *)mk5.ibuf;
    mk5.lbuf=(int64_t *)mk5.ibuf;
 	mk5.fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate
    mk5.vlba=mk5info->vlba;
    mk5.bitsframe=mk5info->bitsframe;  // bits per frame without parity
    if(mk5.parity==1) mk5.bitsframe=mk5.bitsframe*9/8;


    wordsec=(int)mk5info->samplrate/32;   // number of 32bit-words in 1 sec

    //printf("wordsec %d\n",wordsec);
    //if(mk5info->vlba==1){
    //    offset=5;
    //} else {
    //    offset=0;
    //}

    mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* setup bit position table corresponding to channel track */
    memcpy(mk5.bit_trk,gr->bit_trk,mk5.tot_track*sizeof(int));

    /* Mark 5 data file open */
    if( (mk5.fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5 Data File : %s\n",runp->mk5name);
 
    printf("  %dch conversion mode pick up bit position :\n   ",gr->numch);
	for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1
       if(runp->k5name_given[0]=='\0'){
          k5name_make(runp->mk5name, runp->outdir, gr->channel, 
              gr->group, runp->k5name);
          } else {
             strcpy(runp->k5name,runp->k5name_given);
          }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  
    //printf("  K5 Data File : %s\n",runp->k5name);

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */
    kk=0;
    mk5_sync_detect(&mk5);
    memcpy(istart,mk5.itime,6*sizeof(int));

    printf("  Mark 5 data : Sync block detected\n");
    //printf("ipos = %d\n", mk5->ipos);
    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

    /* header ch# 0 monitor*/
    //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5.header[i]));
    //printf("\n");

    /* atamadashi for just seconds start */

    if(mk5_atamadashi(&mk5, runp->soffset)< 0){
        i=fclose(mk5.fin) ;
        i=fclose(fo) ;
        ierr=-3;
        goto exit2return;
    }

    //printf("\n    Atamadashi finished   ipos= %d\n",mk5.ipos);
    printf("   Atamadashi finished.\n");
    printf("\n");

    /* K5 header data creation */

    ipb.adbit=mk5info->adbit;
    ipb.numch=gr->numch;
    ipb.sfreq=(int)(mk5info->samplrate/1000);
    ipb.seconds=mk5.itime[2]*3600+mk5.itime[3]*60+mk5.itime[4];
    ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

    opos=0;   // out buffer writing position
    obuf[opos]=conv4byte2int(ipb.hd);
    opos++;
    obuf[opos]=conv4byte2int(ipb.hd+4);
    opos++;
 
    nbit=0;  // bit# counter

    /* copy 1st Mark5 header block to K5 */

    if(mk5info->vlba==0){  // in case of only mark-IV mode.
                             // in case of VLBA mode, header block
                             // is not necessary to be copied.
       // head180
       //
       //  wbuf   tr#0,tr#1,..,tr#tot_track-1,tr#0,tr#1,....
       //                 k=0                          k=1     

       for (i=0; i<5; i++){  // word position in header block
          for (kk=0; kk<4; kk++){  // byte position in a header word
             for (k=0; k< mk5.nbitbyte; k++){  // bit position in a byte 
                                                 //  0-7 for parity=0
                                                 //  0-8 for parity=1
                if(k==8) continue; //this is parity bit so nothing to do
                for(ch=0; ch<mk5.tot_track; ch++){  // track#
                   wbuf[nbit]
                     =mk5.head180[(i*mk5.nbitword+kk*mk5.nbitbyte+k)*mk5.tot_track+ch];
                   //printf(" ch, k, add %2d %2d %2d\n",ch,k,k*gr.tot_track+ch);
                   nbit++;
                   if(nbit==32){
                       // wbuf contains data with parity bit already removed
                       obuf[opos]=conv2intn(wbuf,ipb.numch);
                       nbit=0;
                       icount++;
                       opos++;
                   }
                }
             } 
          } 
       } 
    } else {
       //icount=offset;
    }
    //printf("tot_track,nbit,opos  %d %d  %d\n",mk5.tot_track,nbit,opos);
    //printf("icount1st = %d\n",icount);
    //printf("bitsframe = %d\n",mk5.bitsframe);
    bpfram=mk5.nbithead-1;  // current bit position in a frame  (starting from 0)
    //printf("bpfram = %d\n",bpfram);

    ih=mk5.nbithead-1;  // bit# in a second counter

    /* 4ch header monitor */
    printf("  *****************************************************\n");
    printf("               Mark5 Header Block Data\n");
    printf("   bit#     Word#1   Word#2   Word#3   Word#4   Word#5\n");
    printf("  -----------------------------------------------------\n");
    for(ch=0; ch<gr->tot_track; ch++){
       printf("  %5d   ",gr->bit_trk[ch]);
       for (i=0; i<5; i++){
          printf("%08x ",rvsint(mk5.header[ch*5+i]));
       }
       printf("\n");
    }
    printf("  *****************************************************\n");


    printf("\n");
    printf("     sec   Mark 5 Header Time\n"); 
    printf("   in day    (1sec before)\n"); 
    printf("   ------ ----------------------\n"); 


       
    i1st=1;
    ikai=0;   // wbuf wtiting pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
    while(1){

       is=0;
       if(i1st==1){
           // data still remain in buffer
           i1st=0;
           if(mk5.ipos < mk5.iread-1){
               //is=ipos+1;
               is=mk5.ipos+1;
               iread=mk5.iread;
           } 
       } 
       if(is ==0){
           if(mk5.ntrack==8){
              iread=fread(mk5.cbuf,sizeof(unsigned char),mk5.nbuf,mk5.fin);
           } else if (mk5.ntrack==16){
              iread=fread(mk5.sbuf,sizeof(unsigned short),mk5.nbuf,mk5.fin);
           } else if (mk5.ntrack==32){
              iread=fread(mk5.ibuf,sizeof(unsigned int),mk5.nbuf,mk5.fin);
           } else {
              iread=fread(mk5.lbuf,sizeof(int64_t),mk5.nbuf,mk5.fin);
           } 
           if (iread ==0 ) { // EOF
              goto exithere;
           }
       } 
       //printf("nbitbyte= %d\n",mk5->nbitbyte);
       for (i=is; i< iread; i++){
           if(mk5.ntrack==8) iwork64=(int64_t)mk5.cbuf[i];
           else if(mk5.ntrack==16) iwork64=(int64_t)mk5.sbuf[i];
           else if(mk5.ntrack==32) iwork64=(int64_t)mk5.ibuf[i];
           else iwork64=mk5.lbuf[i];
           if(mk5.ntrack == 64){
               know64=iwork64;
               if(mk5.nrzm==1) {
                  know64 ^= mk5.lkold;   // NRZM -> NRZL
                  mk5.lkold=iwork64;
               } 
           } else {
               know=(int)iwork64;  // ntrack < 64 is assuumed for speed up
               if(mk5.nrzm==1) {
                  know ^= mk5.kold;   // NRZM -> NRZL
                  mk5.kold=(int)iwork64;
               } 
           }
           if(bpfram==mk5.bitsframe-1) bpfram=-1;
           bpfram++;  // current bit position in a frame starting from 0
           ih++;     // bit position in a second
           kcount++;  // bit position + 1 in a byte
         // debug
/*           if(bpfram==mk5.nbithead){// debug monitor
               printf("nbit head= %d\n",mk5.nbithead);
               mk5hdbit2int(&mk5);
               for(kk=0; kk<gr->tot_track; kk++){
                  printf("%5d   ",gr->bit_trk[kk]);
                  for (k=0; k<5; k++){
                     printf("%08x ",rvsint(mk5.header[kk*5+k]));
                  }  
                  printf("\n");
               }
           }
*/
           if(kcount==mk5.nbitbyte){
               kcount=0;
               if(mk5.parity==1) continue;  // this is parity bit so nothing todo
                                               // go to next ibuf 
           }


           //printf("ih kcount %d %d\n",ih,kcount);
           for(kk=0; kk<gr->tot_track; kk++){
               if(mk5.ntrack == 64){
                   ipick= (int)(know64 >> gr->bit_trk[kk]) & 0x1;
               } else { 
                   ipick= (know >> gr->bit_trk[kk]) & 0x1;
               }
               if(mk5info->vlba==1){
                   // convert only data portion 
                   if(bpfram >= mk5.nbithead) {
                       wbuf[nbit]= ipick;
                       nbit++;
                   }
               } else {
                   wbuf[nbit]= ipick;
                   nbit++;
               }


               if (ih < mk5.nbithead) {
                   mk5.head180[ih*gr->tot_track+kk]=ipick;  // header data save
               } 
               //if (bpfram < mk5.nbithead) {
               //    mk5.head180[bpfram*gr->tot_track+kk]=ipick;  // header data save
               //} 


           } 
           //if (bpfram < mk5.nbithead) printf("bpfram2 %d \n",bpfram);
 

          if(nbit==32){
              //if(nbit==mk5.nbitword){
                 // if(mk5.parity==1){
                 //     obuf[opos]=conv2int(wbuf,ipb.numch);
                 // } else {
                 //     obuf[opos]=conv2intn(wbuf,ipb.numch);
                 // }
                  //printf("nbit, kcount, ih %d %d %d\n",nbit,kcount,ih);
              obuf[opos]=conv2intn(wbuf,ipb.numch);
              opos++;
              if(opos == nwrite){
                  iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                  opos=0;
              }
              nbit=0;
              icount++;
              if( icount== wordsec*mk5info->adbit*ipb.numch){   // 
                           // wordsec=fsampl/32bit (= 125000 for 4MHz sampling) 
                  //printf("icount = %d\n",icount);

                  pcount++;
                  if(runp->period !=0 && pcount > runp->period) goto exithere;
                     ipb.seconds++;
                     ip_hd_make(&ipb);

                     //for (k=0; k<30; k++) printf("%2d",mk5.head180[k*mk5.tot_track+0]);
                     //printf("\n");
                     if(mk5.parity==1){
                        ih=-2;  // reset counter for header save
                     } else {
                        ih=-1;  // reset counter for header save
                     }
                     printf("   %6d ", ipb.seconds); // just debug
                     k=head180_to_time(&mk5, 0, itime); // pick up track0 time data 
                                             // This time is taken from 1 sec before data
                     printf(" %d/%03d %02d:%02d:%02d.%03d",
                        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);
                     if(k<0){
                         printf(" missing sync!! ");
                     } 
                     printf("\n");

                     // put header data into obuf 
                     obuf[opos]=conv4byte2int(ipb.hd);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }
                     obuf[opos]=conv4byte2int(ipb.hd+4);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }

                     icount=0;
                 } 
                 ikai=-1;

              }
              ikai++;
              //printf("i,nbit, kcount, ikai %d %d %d %d\n",i,nbit,kcount,ikai);

          }
       }
exithere:
       i=fclose( mk5.fin) ;
       // rest of data write
       if(opos != 0){
          iwrite=fwrite(obuf,sizeof(int),opos,fo);
       }
       i=fclose( fo ) ;
exit2return:
       //printf("  K5 file (%s) has been created.\n\n",runp->k5name);
       free(mk5.header);
       free(mk5.head180);
	   free(mk5.ibuf);
       free(obuf);

       return 0;
}

int conv2k5_32_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp)
/* conversion for 32track non-parity and MarkIV  2004-09-21 */
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;
    int ih;
    int know;
    int i1st;
    int ipick;
    int i,n,k,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    IPBOARD_T ipb;
    FILE *fo;
    mk5_read_t mk5;

    printf("  conv2k5_32_l : \n");
    ierr=0;

    /* initialize mk5_read_t type variable */
	mk5.parity=mk5info->parity;

	mk5.nbuf=800;
    mk5.nbitbyte=8; 
    mk5.nbitword=32; 
    mk5.nbithead=160; 

	mk5.nrzm=mk5info->nrzm;
	mk5.ntrack=mk5info->ntrack;
	mk5.ibufsize=mk5.nbuf*mk5.ntrack/32; // ibuf size in 32bit-word (int) 
	mk5.ibuf=(unsigned int *)malloc(mk5.ibufsize*sizeof(int));
    mk5.sbuf=(unsigned short *)mk5.ibuf; 
    mk5.cbuf=(unsigned char *)mk5.ibuf;
    mk5.lbuf=(int64_t *)mk5.ibuf;
 	mk5.fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate

    mk5.vlba=mk5info->vlba;
    mk5.bitsframe=mk5info->bitsframe;

    wordsec=(int)mk5info->samplrate/32;   // number of 32bit-words in 1 sec


    mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* setup bit position table corresponding to channel track */
    memcpy(mk5.bit_trk,gr->bit_trk,mk5.tot_track*sizeof(int));

    /* Mark 5 data file open */
    if( (mk5.fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5 Data File : %s\n",runp->mk5name);
 
    printf("  %dch conversion mode pick up bit position :\n   ",gr->numch);
	for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1
       if(runp->k5name_given[0]=='\0'){
          k5name_make(runp->mk5name, runp->outdir, gr->channel, 
              gr->group, runp->k5name);
          } else {
             strcpy(runp->k5name,runp->k5name_given);
          }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  
    //printf("  K5 Data File : %s\n",runp->k5name);

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */
    kk=0;
    mk5_sync_detect(&mk5);
    memcpy(istart,mk5.itime,6*sizeof(int));

    printf("  Mark 5 data : Sync block detected\n");
    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);


    /* atamadashi for just seconds start */

    if(mk5_atamadashi(&mk5, runp->soffset)< 0){
        i=fclose(mk5.fin) ;
        i=fclose(fo) ;
        ierr=-3;
        goto exit2return;
    }

    printf("   Atamadashi finished.\n");
    printf("\n");

    /* K5 header data creation */

    ipb.adbit=mk5info->adbit;
    ipb.numch=gr->numch;
    ipb.sfreq=(int)(mk5info->samplrate/1000);
    ipb.seconds=mk5.itime[2]*3600+mk5.itime[3]*60+mk5.itime[4];
    ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

    opos=0;   // out buffer writing position
    obuf[opos]=conv4byte2int(ipb.hd);
    opos++;
    obuf[opos]=conv4byte2int(ipb.hd+4);
    opos++;
 
    nbit=0;  // bit# counter

    /* copy 1st Mark5 header block to K5 */

    if(mk5info->vlba==0){  // in case of only mark-IV mode.
                             // in case of VLBA mode, header block
                             // is not necessary to be copied.
       // head180
       //
       //  wbuf   tr#0,tr#1,..,tr#tot_track-1,tr#0,tr#1,....
       //                 k=0                          k=1     

       for (i=0; i<5; i++){  // word position in header block
          for (kk=0; kk<4; kk++){  // byte position in a header word
             for (k=0; k< mk5.nbitbyte; k++){  // bit position in a byte 
                                                 //  0-7 for parity=0
                for(ch=0; ch<mk5.tot_track; ch++){  // track#
                   wbuf[nbit]
                     =mk5.head180[(i*mk5.nbitword+kk*mk5.nbitbyte+k)*mk5.tot_track+ch];
                   nbit++;
                   if(nbit==32){
                       // wbuf contains data with parity bit already removed
                       obuf[opos]=conv2intn(wbuf,ipb.numch);
                       nbit=0;
                       icount++;
                       opos++;
                   }
                }
             } 
          } 
       } 
    } 

    ih=mk5.nbithead-1;  // bit# in a second counter

    /* 4ch header monitor */
    printf("  *****************************************************\n");
    printf("               Mark5 Header Block Data\n");
    printf("   bit#     Word#1   Word#2   Word#3   Word#4   Word#5\n");
    printf("  -----------------------------------------------------\n");
    for(ch=0; ch<gr->tot_track; ch++){
       printf("  %5d   ",gr->bit_trk[ch]);
       for (i=0; i<5; i++){
          printf("%08x ",rvsint(mk5.header[ch*5+i]));
       }
       printf("\n");
    }
    printf("  *****************************************************\n");


    printf("\n");
    printf("     sec   Mark 5 Header Time\n"); 
    printf("   in day    (1sec before)\n"); 
    printf("   ------ ----------------------\n"); 


       
    i1st=1;
    ikai=0;   // wbuf writing pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
    while(1){

       is=0;
       if(i1st==1){
           // data still remain in buffer
           i1st=0;
           if(mk5.ipos < mk5.iread-1){
               is=mk5.ipos+1;
               iread=mk5.iread;
           } 
       } 
       if(is ==0){
           iread=fread(mk5.ibuf,sizeof(unsigned int),mk5.nbuf,mk5.fin);
           if (iread ==0 ) { // EOF
              goto exithere;
           }
       } 

       for (i=is; i< iread; i++){
           know=mk5.ibuf[i];
           ih++;     // bit position in a second
           for(kk=0; kk<gr->tot_track; kk++){
               ipick= (know >> gr->bit_trk[kk]) & 0x1;
               wbuf[nbit]= ipick;
               nbit++;


               if (ih < mk5.nbithead) {
                   mk5.head180[ih*gr->tot_track+kk]=ipick;  // header data save
               } 


           } 

          if(nbit==32){
              obuf[opos]=conv2intn(wbuf,ipb.numch);
              opos++;
              if(opos == nwrite){
                  iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                  opos=0;
              }
              nbit=0;
              icount++;
              if( icount== wordsec*mk5info->adbit*ipb.numch){   

                  pcount++;
                  if(runp->period !=0 && pcount > runp->period) goto exithere;
                     ipb.seconds++;
                     ip_hd_make(&ipb);

                     ih=-1;  // reset counter for header save  non parity mode
                     printf("   %6d ", ipb.seconds); // just debug
                     k=head180_to_time(&mk5, 0, itime); // pick up track0 time data 
                                             // This time is taken from 1 sec before data
                     printf(" %d/%03d %02d:%02d:%02d.%03d",
                        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);
                     if(k<0){
                         printf(" missing sync!! ");
                     } 
                     printf("\n");

                     // put header data into obuf 
                     obuf[opos]=conv4byte2int(ipb.hd);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }
                     obuf[opos]=conv4byte2int(ipb.hd+4);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }

                     icount=0;
                 } 
                 ikai=-1;

              }
              ikai++;

          }
       }
exithere:
       i=fclose( mk5.fin) ;
       // rest of data write
       if(opos != 0){
          iwrite=fwrite(obuf,sizeof(int),opos,fo);
       }
       i=fclose( fo ) ;
exit2return:
       free(mk5.header);
       free(mk5.head180);
	   free(mk5.ibuf);
       free(obuf);

       return 0;
}

int conv2k5_32_m(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp)
/* conversion for 32track with parity and MarkIV  2004-09-21 */
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;
    int ih;
    int know;
    int i1st;
    int ipick;
    int i,n,k,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    IPBOARD_T ipb;
    FILE *fo;
    mk5_read_t mk5;

    printf("  conv2k5_32_m : \n");
    ierr=0;

    /* initialize mk5_read_t type variable */
	mk5.parity=mk5info->parity;

    mk5.nbuf=900; //ǂ݂Ƃv@
	                 //peB̏ꍇX̔{ɂƋ؂肪悭֗
                     //    ȂȂ΁@f[^͂WrbgɃpeBPrbg
    mk5.nbitbyte=9; // # of bits in a byte
    mk5.nbitword=36; // # of bits in a header word
    mk5.nbithead=180; // # of bits in header

    mk5.nrzm=mk5info->nrzm;
	mk5.ntrack=mk5info->ntrack;
	mk5.ibufsize=mk5.nbuf*mk5.ntrack/32; // ibuf size in 32bit-word (int) 
	mk5.ibuf=(unsigned int *)malloc(mk5.ibufsize*sizeof(int));
    mk5.sbuf=(unsigned short *)mk5.ibuf; 
    mk5.cbuf=(unsigned char *)mk5.ibuf;
    mk5.lbuf=(int64_t *)mk5.ibuf;
 	mk5.fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate

    mk5.vlba=mk5info->vlba;
    mk5.bitsframe=mk5info->bitsframe*9/8;  // bit per frame with parity


    wordsec=(int)mk5info->samplrate/32;   // number of 32bit-words in 1 sec


    mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* setup bit position table corresponding to channel track */
    memcpy(mk5.bit_trk,gr->bit_trk,mk5.tot_track*sizeof(int));

    /* Mark 5 data file open */
    if( (mk5.fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5 Data File : %s\n",runp->mk5name);
 
    printf("  %dch conversion mode pick up bit position :\n   ",gr->numch);
	for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1 run parameters
       if(runp->k5name_given[0]=='\0'){
          k5name_make(runp->mk5name, runp->outdir, gr->channel, 
              gr->group, runp->k5name);
          } else {
             strcpy(runp->k5name,runp->k5name_given);
          }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  
    //printf("  K5 Data File : %s\n",runp->k5name);

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */
    kk=0;
    mk5_sync_detect(&mk5);
    memcpy(istart,mk5.itime,6*sizeof(int));

    printf("  Mark 5 data : Sync block detected\n");
    //printf("ipos = %d\n", mk5->ipos);
    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

    /* atamadashi for just seconds start */

    if(mk5_atamadashi(&mk5, runp->soffset)< 0){
        i=fclose(mk5.fin) ;
        i=fclose(fo) ;
        ierr=-3;
        goto exit2return;
    }

    //printf("\n    Atamadashi finished   ipos= %d\n",mk5.ipos);
    printf("   Atamadashi finished.\n");
    printf("\n");

    /* K5 header data creation */

    ipb.adbit=mk5info->adbit;
    ipb.numch=gr->numch;
    ipb.sfreq=(int)(mk5info->samplrate/1000);
    ipb.seconds=mk5.itime[2]*3600+mk5.itime[3]*60+mk5.itime[4];
    ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

    opos=0;   // out buffer writing position
    obuf[opos]=conv4byte2int(ipb.hd);
    opos++;
    obuf[opos]=conv4byte2int(ipb.hd+4);
    opos++;
 
    nbit=0;  // bit# counter

    /* copy 1st Mark5 header block to K5 */

    if(mk5info->vlba==0){  // in case of only mark-IV mode.
                             // in case of VLBA mode, header block
                             // is not necessary to be copied.
       // head180
       //
       //  wbuf   tr#0,tr#1,..,tr#tot_track-1,tr#0,tr#1,....
       //                 k=0                          k=1     

       for (i=0; i<5; i++){  // word position in header block
          for (kk=0; kk<4; kk++){  // byte position in a header word
             for (k=0; k< mk5.nbitbyte; k++){  // bit position in a byte 
                                                 //  0-7 for parity=0
                                                 //  0-8 for parity=1
                if(k==8) continue; //this is parity bit so nothing to do
                for(ch=0; ch<mk5.tot_track; ch++){  // track#
                   wbuf[nbit]
                     =mk5.head180[(i*mk5.nbitword+kk*mk5.nbitbyte+k)*mk5.tot_track+ch];
                   //printf(" ch, k, add %2d %2d %2d\n",ch,k,k*gr.tot_track+ch);
                   nbit++;
                   if(nbit==32){
                       // wbuf contains data with parity bit already removed
                       obuf[opos]=conv2intn(wbuf,ipb.numch);
                       nbit=0;
                       icount++;
                       opos++;
                   }
                }
             } 
          } 
       } 
    } 

    ih=mk5.nbithead-1;  // bit# in a second counter

    /* 4ch header monitor */
    printf("  *****************************************************\n");
    printf("               Mark5 Header Block Data\n");
    printf("   bit#     Word#1   Word#2   Word#3   Word#4   Word#5\n");
    printf("  -----------------------------------------------------\n");
    for(ch=0; ch<gr->tot_track; ch++){
       printf("  %5d   ",gr->bit_trk[ch]);
       for (i=0; i<5; i++){
          printf("%08x ",rvsint(mk5.header[ch*5+i]));
       }
       printf("\n");
    }
    printf("  *****************************************************\n");


    printf("\n");
    printf("     sec   Mark 5 Header Time\n"); 
    printf("   in day    (1sec before)\n"); 
    printf("   ------ ----------------------\n"); 


       
    i1st=1;
    ikai=0;   // wbuf wtiting pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
    while(1){

       is=0;
       if(i1st==1){
           // data still remain in buffer
           i1st=0;
           if(mk5.ipos < mk5.iread-1){
               is=mk5.ipos+1;
               iread=mk5.iread;
           } 
       } 
       if(is ==0){
           iread=fread(mk5.ibuf,sizeof(unsigned int),mk5.nbuf,mk5.fin);
           if (iread ==0 ) { // EOF
              goto exithere;
           }
       } 
       //printf("nbitbyte= %d\n",mk5->nbitbyte);
       for (i=is; i< iread; i++){
           know=mk5.ibuf[i];
           know ^= mk5.kold;   // NRZM -> NRZL
           mk5.kold=mk5.ibuf[i];

           ih++;     // bit position in a second

           kcount++;  // bit position + 1 in a byte
           if(kcount==mk5.nbitbyte){
               kcount=0;
               //if(mk5.parity==1) continue;  // this is parity bit so nothing todo
               continue;  // this is parity bit so nothing todo
                                               // go to next ibuf 
           }

           //printf("ih kcount %d %d\n",ih,kcount);
           for(kk=0; kk<gr->tot_track; kk++){
               ipick= (know >> gr->bit_trk[kk]) & 0x1;
               wbuf[nbit]= ipick;
               nbit++;


               if (ih < mk5.nbithead) {
                   mk5.head180[ih*gr->tot_track+kk]=ipick;  // header data save
               } 


           } 

          if(nbit==32){
              obuf[opos]=conv2intn(wbuf,ipb.numch);
              opos++;
              if(opos == nwrite){
                  iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                  opos=0;
              }
              nbit=0;
              icount++;
              if( icount== wordsec*mk5info->adbit*ipb.numch){   // 

                  pcount++;
                  if(runp->period !=0 && pcount > runp->period) goto exithere;
                     ipb.seconds++;
                     ip_hd_make(&ipb);

                     ih=-2;  // reset counter for header save  with parity mode
                     printf("   %6d ", ipb.seconds); // just debug
                     k=head180_to_time(&mk5, 0, itime); // pick up track0 time data 
                                             // This time is taken from 1 sec before data
                     printf(" %d/%03d %02d:%02d:%02d.%03d",
                        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);
                     if(k<0){
                         printf(" missing sync!! ");
                     } 
                     printf("\n");

                     // put header data into obuf 
                     obuf[opos]=conv4byte2int(ipb.hd);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }
                     obuf[opos]=conv4byte2int(ipb.hd+4);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }

                     icount=0;
                 } 
                 ikai=-1;

              }
              ikai++;

          }
       }
exithere:
       i=fclose( mk5.fin) ;
       // rest of data write
       if(opos != 0){
          iwrite=fwrite(obuf,sizeof(int),opos,fo);
       }
       i=fclose( fo ) ;
exit2return:
       free(mk5.header);
       free(mk5.head180);
	   free(mk5.ibuf);
       free(obuf);

       return 0;
}

int conv2k5_16_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp)
/* conversion for 16track non-parity and MarkIV  2004-09-21 */
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;
    int ih;
    int know;
    int i1st;
    int ipick;
    int i,n,k,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    IPBOARD_T ipb;
    FILE *fo;
    mk5_read_t mk5;

    printf("  conv2k5_16_l : \n");
    ierr=0;

    /* initialize mk5_read_t type variable */
	mk5.parity=mk5info->parity;

    mk5.nbuf=800;
    mk5.nbitbyte=8; 
    mk5.nbitword=32; 
    mk5.nbithead=160; 

    mk5.nrzm=mk5info->nrzm;
	mk5.ntrack=mk5info->ntrack;
	mk5.ibufsize=mk5.nbuf*mk5.ntrack/32; // ibuf size in 32bit-word (int) 
	mk5.ibuf=(unsigned int *)malloc(mk5.ibufsize*sizeof(int));
    mk5.sbuf=(unsigned short *)mk5.ibuf; 
    mk5.cbuf=(unsigned char *)mk5.ibuf;
    mk5.lbuf=(int64_t *)mk5.ibuf;
 	mk5.fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate
    mk5.vlba=mk5info->vlba;
    mk5.bitsframe=mk5info->bitsframe;


    wordsec=(int)mk5info->samplrate/32;   // number of 32bit-words in 1 sec

    mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* setup bit position table corresponding to channel track */
    memcpy(mk5.bit_trk,gr->bit_trk,mk5.tot_track*sizeof(int));

    /* Mark 5 data file open */
    if( (mk5.fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5 Data File : %s\n",runp->mk5name);
 
    printf("  %dch conversion mode pick up bit position :\n   ",gr->numch);
	for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1 run parameters
       if(runp->k5name_given[0]=='\0'){
          k5name_make(runp->mk5name, runp->outdir, gr->channel, 
              gr->group, runp->k5name);
          } else {
             strcpy(runp->k5name,runp->k5name_given);
          }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */
    kk=0;
    mk5_sync_detect(&mk5);
    memcpy(istart,mk5.itime,6*sizeof(int));

    printf("  Mark 5 data : Sync block detected\n");
    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

    /* atamadashi for just seconds start */

    if(mk5_atamadashi(&mk5, runp->soffset)< 0){
        i=fclose(mk5.fin) ;
        i=fclose(fo) ;
        ierr=-3;
        goto exit2return;
    }

    printf("   Atamadashi finished.\n");
    printf("\n");

    /* K5 header data creation */

    ipb.adbit=mk5info->adbit;
    ipb.numch=gr->numch;
    ipb.sfreq=(int)(mk5info->samplrate/1000);
    ipb.seconds=mk5.itime[2]*3600+mk5.itime[3]*60+mk5.itime[4];
    ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

    opos=0;   // out buffer writing position
    obuf[opos]=conv4byte2int(ipb.hd);
    opos++;
    obuf[opos]=conv4byte2int(ipb.hd+4);
    opos++;
 
    nbit=0;  // bit# counter

    /* copy 1st Mark5 header block to K5 */

    if(mk5info->vlba==0){  // in case of only mark-IV mode.
                             // in case of VLBA mode, header block
                             // is not necessary to be copied.
       // head180
       //
       //  wbuf   tr#0,tr#1,..,tr#tot_track-1,tr#0,tr#1,....
       //                 k=0                          k=1     

       for (i=0; i<5; i++){  // word position in header block
          for (kk=0; kk<4; kk++){  // byte position in a header word
             for (k=0; k< mk5.nbitbyte; k++){  // bit position in a byte 
                                                 //  0-7 for parity=0
                for(ch=0; ch<mk5.tot_track; ch++){  // track#
                   wbuf[nbit]
                     =mk5.head180[(i*mk5.nbitword+kk*mk5.nbitbyte+k)*mk5.tot_track+ch];
                   nbit++;
                   if(nbit==32){
                       // wbuf contains data with parity bit already removed
                       obuf[opos]=conv2intn(wbuf,ipb.numch);
                       nbit=0;
                       icount++;
                       opos++;
                   }
                }
             } 
          } 
       } 
    } 

    ih=mk5.nbithead-1;  // bit# in a second counter

    /* 4ch header monitor */
    printf("  *****************************************************\n");
    printf("               Mark5 Header Block Data\n");
    printf("   bit#     Word#1   Word#2   Word#3   Word#4   Word#5\n");
    printf("  -----------------------------------------------------\n");
    for(ch=0; ch<gr->tot_track; ch++){
       printf("  %5d   ",gr->bit_trk[ch]);
       for (i=0; i<5; i++){
          printf("%08x ",rvsint(mk5.header[ch*5+i]));
       }
       printf("\n");
    }
    printf("  *****************************************************\n");


    printf("\n");
    printf("     sec   Mark 5 Header Time\n"); 
    printf("   in day    (1sec before)\n"); 
    printf("   ------ ----------------------\n"); 


       
    i1st=1;
    ikai=0;   // wbuf wtiting pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
    while(1){

       is=0;
       if(i1st==1){
           // data still remain in buffer
           i1st=0;
           if(mk5.ipos < mk5.iread-1){
               //is=ipos+1;
               is=mk5.ipos+1;
               iread=mk5.iread;
           } 
       } 
       if(is ==0){
           iread=fread(mk5.sbuf,sizeof(unsigned short),mk5.nbuf,mk5.fin);
           if (iread ==0 ) { // EOF
              goto exithere;
           }
       } 
       //printf("nbitbyte= %d\n",mk5->nbitbyte);
       for (i=is; i< iread; i++){
           know=(int)mk5.sbuf[i];
           ih++;     // bit position in a second

           for(kk=0; kk<gr->tot_track; kk++){
               ipick= (know >> gr->bit_trk[kk]) & 0x1;
               wbuf[nbit]= ipick;
               nbit++;


               if (ih < mk5.nbithead) {
                   mk5.head180[ih*gr->tot_track+kk]=ipick;  // header data save
               } 


           } 

          if(nbit==32){
              obuf[opos]=conv2intn(wbuf,ipb.numch);
              opos++;
              if(opos == nwrite){
                  iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                  opos=0;
              }
              nbit=0;
              icount++;
              if( icount== wordsec*mk5info->adbit*ipb.numch){   

                  pcount++;
                  if(runp->period !=0 && pcount > runp->period) goto exithere;
                     ipb.seconds++;
                     ip_hd_make(&ipb);

                    ih=-1;  // reset counter for header save non parity mode
                     printf("   %6d ", ipb.seconds); // just debug
                     k=head180_to_time(&mk5, 0, itime); // pick up track0 time data 
                                             // This time is taken from 1 sec before data
                     printf(" %d/%03d %02d:%02d:%02d.%03d",
                        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);
                     if(k<0){
                         printf(" missing sync!! ");
                     } 
                     printf("\n");

                     // put header data into obuf 
                     obuf[opos]=conv4byte2int(ipb.hd);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }
                     obuf[opos]=conv4byte2int(ipb.hd+4);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }

                     icount=0;
                 } 
                 ikai=-1;

              }
              ikai++;

          }
       }
exithere:
       i=fclose( mk5.fin) ;
       // rest of data write
       if(opos != 0){
          iwrite=fwrite(obuf,sizeof(int),opos,fo);
       }
       i=fclose( fo ) ;
exit2return:
       free(mk5.header);
       free(mk5.head180);
	   free(mk5.ibuf);
       free(obuf);

       return 0;
}

int conv2k5_64_l(mk5tok5_info_t *mk5info, gr_info_t *gr, 
            m5k5runp_t *runp)
/* conversion for 64track non-parity and MarkIV  2004-09-21 */
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;
    int ih;
    int i1st;
    int ipick;
    int i,n,k,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    int64_t know64;
    IPBOARD_T ipb;
    FILE *fo;
    mk5_read_t mk5;

    printf("  conv2k5_64_l : \n");
    ierr=0;

    /* initialize mk5_read_t type variable */
	mk5.parity=mk5info->parity;

    mk5.nbuf=800;
    mk5.nbitbyte=8; 
    mk5.nbitword=32; 
    mk5.nbithead=160; 

    mk5.nrzm=mk5info->nrzm;
	mk5.ntrack=mk5info->ntrack;
	mk5.ibufsize=mk5.nbuf*mk5.ntrack/32; // ibuf size in 32bit-word (int) 
	mk5.ibuf=(unsigned int *)malloc(mk5.ibufsize*sizeof(int));
    mk5.sbuf=(unsigned short *)mk5.ibuf; 
    mk5.cbuf=(unsigned char *)mk5.ibuf;
    mk5.lbuf=(int64_t *)mk5.ibuf;
 	mk5.fsampl=mk5info->samplrate/mk5info->fanout;  // track sampling rate

    mk5.vlba=mk5info->vlba;
    mk5.bitsframe=mk5info->bitsframe;


    wordsec=(int)mk5info->samplrate/32;   // number of 32bit-words in 1 sec

    mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* setup bit position table corresponding to channel track */
    memcpy(mk5.bit_trk,gr->bit_trk,mk5.tot_track*sizeof(int));

    /* Mark 5 data file open */
    if( (mk5.fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5 Data File : %s\n",runp->mk5name);
 
    printf("  %dch conversion mode pick up bit position :\n   ",gr->numch);
	for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1 run parameters
       if(runp->k5name_given[0]=='\0'){
          k5name_make(runp->mk5name, runp->outdir, gr->channel, 
              gr->group, runp->k5name);
          } else {
             strcpy(runp->k5name,runp->k5name_given);
          }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */
    kk=0;
    mk5_sync_detect(&mk5);
    memcpy(istart,mk5.itime,6*sizeof(int));

    printf("  Mark 5 data : Sync block detected\n");
    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

    /* atamadashi for just seconds start */

    if(mk5_atamadashi(&mk5, runp->soffset)< 0){
        i=fclose(mk5.fin) ;
        i=fclose(fo) ;
        ierr=-3;
        goto exit2return;
    }

    printf("   Atamadashi finished.\n");
    printf("\n");

    /* K5 header data creation */

    ipb.adbit=mk5info->adbit;
    ipb.numch=gr->numch;
    ipb.sfreq=(int)(mk5info->samplrate/1000);
    ipb.seconds=mk5.itime[2]*3600+mk5.itime[3]*60+mk5.itime[4];
    ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

    opos=0;   // out buffer writing position
    obuf[opos]=conv4byte2int(ipb.hd);
    opos++;
    obuf[opos]=conv4byte2int(ipb.hd+4);
    opos++;
 
    nbit=0;  // bit# counter

    /* copy 1st Mark5 header block to K5 */

    if(mk5info->vlba==0){  // in case of only mark-IV mode.
                             // in case of VLBA mode, header block
                             // is not necessary to be copied.
       // head180
       //
       //  wbuf   tr#0,tr#1,..,tr#tot_track-1,tr#0,tr#1,....
       //                 k=0                          k=1     

       for (i=0; i<5; i++){  // word position in header block
          for (kk=0; kk<4; kk++){  // byte position in a header word
             for (k=0; k< mk5.nbitbyte; k++){  // bit position in a byte 
                                                 //  0-7 for parity=0
                for(ch=0; ch<mk5.tot_track; ch++){  // track#
                   wbuf[nbit]
                     =mk5.head180[(i*mk5.nbitword+kk*mk5.nbitbyte+k)*mk5.tot_track+ch];
                   nbit++;
                   if(nbit==32){
                       // wbuf contains data with parity bit already removed
                       obuf[opos]=conv2intn(wbuf,ipb.numch);
                       nbit=0;
                       icount++;
                       opos++;
                   }
                }
             } 
          } 
       } 
    } 

    ih=mk5.nbithead-1;  // bit# in a second counter

    /* 4ch header monitor */
    printf("  *****************************************************\n");
    printf("               Mark5 Header Block Data\n");
    printf("   bit#     Word#1   Word#2   Word#3   Word#4   Word#5\n");
    printf("  -----------------------------------------------------\n");
    for(ch=0; ch<gr->tot_track; ch++){
       printf("  %5d   ",gr->bit_trk[ch]);
       for (i=0; i<5; i++){
          printf("%08x ",rvsint(mk5.header[ch*5+i]));
       }
       printf("\n");
    }
    printf("  *****************************************************\n");


    printf("\n");
    printf("     sec   Mark 5 Header Time\n"); 
    printf("   in day    (1sec before)\n"); 
    printf("   ------ ----------------------\n"); 


       
    i1st=1;
    ikai=0;   // wbuf wtiting pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
    while(1){

       is=0;
       if(i1st==1){
           // data still remain in buffer
           i1st=0;
           if(mk5.ipos < mk5.iread-1){
               is=mk5.ipos+1;
               iread=mk5.iread;
           } 
       } 
       if(is ==0){
           iread=fread(mk5.lbuf,sizeof(int64_t),mk5.nbuf,mk5.fin);
           if (iread ==0 ) { // EOF
              goto exithere;
           }
       } 
       for (i=is; i< iread; i++){
           know64=mk5.lbuf[i];
           ih++;     // bit position in a second
           for(kk=0; kk<gr->tot_track; kk++){
               ipick= (int)(know64 >> gr->bit_trk[kk]) & 0x1;
               wbuf[nbit]= ipick;
               nbit++;
               if (ih < mk5.nbithead) {
                   mk5.head180[ih*gr->tot_track+kk]=ipick;  // header data save
               } 


           } 

          if(nbit==32){
              obuf[opos]=conv2intn(wbuf,ipb.numch);
              opos++;
              if(opos == nwrite){
                  iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                  opos=0;
              }
              nbit=0;
              icount++;
              if( icount== wordsec*mk5info->adbit*ipb.numch){   // 

                  pcount++;
                  if(runp->period !=0 && pcount > runp->period) goto exithere;
                     ipb.seconds++;
                     ip_hd_make(&ipb);

                     ih=-1;  // reset counter for header save for non parity mode

                     printf("   %6d ", ipb.seconds); // just debug
                     k=head180_to_time(&mk5, 0, itime); // pick up track0 time data 
                                             // This time is taken from 1 sec before data
                     printf(" %d/%03d %02d:%02d:%02d.%03d",
                        itime[0],itime[1],itime[2],itime[3],itime[4],itime[5]);
                     if(k<0){
                         printf(" missing sync!! ");
                     } 
                     printf("\n");

                     // put header data into obuf 
                     obuf[opos]=conv4byte2int(ipb.hd);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }
                     obuf[opos]=conv4byte2int(ipb.hd+4);
                     opos++;
                     if(opos == nwrite){
                        iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                        opos=0;
                     }

                     icount=0;
                 } 
                 ikai=-1;

              }
              ikai++;
              //printf("i,nbit, kcount, ikai %d %d %d %d\n",i,nbit,kcount,ikai);

          }
       }
exithere:
       i=fclose( mk5.fin) ;
       // rest of data write
       if(opos != 0){
          iwrite=fwrite(obuf,sizeof(int),opos,fo);
       }
       i=fclose( fo ) ;
exit2return:
       //printf("  K5 file (%s) has been created.\n\n",runp->k5name);
       free(mk5.header);
       free(mk5.head180);
	   free(mk5.ibuf);
       free(obuf);

       return 0;
}

void shift_bit_trk_table(int *bit_trk, int tot_track)
/*   shift 32  in bit position table */
{
    int n;
    printf(" bit-track table was edited\n from ");
    for(n=0; n<tot_track; n++) printf(" %2d",bit_trk[n]);
    printf("\n");
    for(n=0; n<tot_track; n++){
       bit_trk[n]+=32;
       bit_trk[n]%=64 ;
    } 
    printf(" to   ");
    for(n=0; n<tot_track; n++) printf(" %2d",bit_trk[n]);
    printf("\n");
} 

/* UTC to MJD conversion */
int ut2mjd(double *d, int year, int month, int day,
           int hour, int min, int sec)
/*
  [in]
     year   -- year (yyyy)
     month  -- month (month)
     day    -- day of month (dd)
     hour   -- hour (hh)
     min    -- minute (mm)
     sec    -- second (ss)
  [return]
     ut2mjs -- MJD  (integer day)
     d      -- fractional part of MJD (day)
*/
{
    int  bc, gregory, mnth, y4; 
    long j;

	mnth=month;
	y4=year;
    bc = year <= 0;
    
    /* First day at Gregory's calendar */
    gregory = year > 1582 || (year == 1582 && month > 10) ||
              (year == 1582 && month == 10 && day >= 15);
            
    if (mnth <= 2) {
        y4--;
        mnth += 12;
    }
    
    if (hour < 12) {
        j = 0;
        *d = 0.5;
    } else {
        j = 1;
        *d = -0.5;
    }
    *d += (hour*3600L + min*60 + sec)/86400.0;
    
    j += bc ? (y4-3)/4 : y4/4;
    if (gregory) j+= 2 - y4/100 + y4/400;
    j += 1720994L + y4*365L + (mnth+1)*30 + (mnth+1)*3/5 + day;
    
	/* convert to MJD */
	if (hour < 12) {
		*d -= 0.5;
	} else {
		*d += 0.5;
		j -= 1L;
	}
    j -= 2400000L;
    return j;
}

/* MJD to UTC conversion */
void mjd2ut(int *year, int *month, int *day,
		   int *hour, int *min, int *sec, int j, double d)
/*
  [in]
     j      -- MJD  (integer day)
     d      -- fractional part of MJD (day)
  [return]
     year   -- year (yyyy)
     month  -- month (month)
     day    -- day of month (dd)
     hour   -- hour (hh)
     min    -- minute (mm)
     sec    -- second (ss)
*/
{
    int c, k, e, s;

    /* convert to JD */
    if( d >= 0.5) {
		d -= 0.5;
		j++;
	} else {
		d += 0.5;
	}
	j += 2400000L;

	/* JD to UTC */
    if (d >= 0.5) {
        j++;
        d -= 0.5;
    } else d += 0.5;

    if (j >= 2299161) j = j + 1 + (j-1867216)/36524 - (j-1867216)/146096;
    j += 1524;

    c = (long)((j - 122.1)/365.25);
    k = 365L*c + c/4;
    e = (long)((j - k)/30.6001);

    *year = c - 4716;
    *month = e - 1;
    if (*month > 12) {
       (*month) -= 12;
       (*year)++;
    }
    *day = j - k - (long)(30.6*e);

    s = (long)(d * 86400 + 0.5);
    *hour = s / 3600;
    *min = (s % 3600) / 60;
    *sec = s % 60;
}

unsigned mod_func(int ini)
{
/* 
  modulation pattern used in VLBA format
                    Ver 2008-07-04

    ini ----  =1  initialize only
              =0  normal operation
*/
    static unsigned ff[16]={1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
    unsigned output,n;
    if (ini == 1) {
        for (n=0; n<16; n++) ff[n]=1;
        return 0;
    }
    output=ff[10]^ff[12]^ff[13]^ff[15];
    for (n=15; n>0; n--) ff[n]=ff[n-1];
    ff[0]=output;
    return output;
}


int mk5b_dat_anal(char *mk5name, int mode, int nmax, int *istart,
				  int *iuid, int *tvgflag, double *fbps)
{
/*
     Mark5B data file structure analysis
     modified from mk5_dat_anal
     2009-06-06

    [in]
	  mk5name  ---  Mark5 data file name
      mode     ---- Display style  0: old style   1: new including 1st data block
                                   2: modified new style
      nmax     ---- Maximum frame # to read  ( if 0, all data are read)

	[out]
	  istart[6] ----- 1st sync block time  
                         -1,mjd(3digits),hh,mm,ss,msec*10  for VLBA format
	  uid       ----- User specified data (lower 16bits are valid)
	  tvgflag   ----- TVG flag (1: TVG data  0: real data)
	  fbps      ----- Supposed data rate (bps)

    [return]
       1  ---  This is Mark 5B format
       0  ---  This is not Mark 5B format

*/
       int i,nkai;
       unsigned int ibuf[2504];  // for 1 Frame data 2504 X 32 bits
	   unsigned int *head;   // for header 4 X 32 bits
	   unsigned int *idata;  // for data field 2500 X 32 bits
       int nbuf=2504;   // @
       int iread; 
       int itime[6];
	   unsigned short uid, icrc16;
	   int iok;
	   int nframe;
	   int tvg;
       //int data1st[2]; // First 2 word data just after headder at given bit position
       int mk5B_mode;   // =1  Mark 5B format   =0 No
       double bps, mbps;  // supposed data rate
	   double dtime,dtime0,dt;
       FILE *fin;
       //mk5_aux_t aux;

       mk5B_mode=0;
	   head=ibuf;
	   idata=ibuf+4;

       /* Data file open */
       if( (fin  = fopen( mk5name, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",mk5name);
		   exit(1);
       }
       printf("Data File : %s\n",mk5name);


       printf("Now analizing the data file is Mark5B or not .....\n"); 

	   nkai=0;

	   dtime0=0.0;

       while(1){
          iread = fread( ibuf, sizeof( unsigned int ), nbuf, fin );
          if (iread ==0 ) { // EOF
			 printf("\n");
             goto exitreturn; 
          }

          mk5b_sync_detect(head, itime, &uid, &tvg, &nframe, 
		                &icrc16, &iok);

          if(iok > 0) { //  this is Mark 5B header
             if (nkai ==0) 
		     mk5B_mode=1;
		  } else {
             //printf(" not Mark 5B header\n");
             goto exitreturn; 
		  }
          if (nkai ==0) {
              printf("\n OK this is Mark 5B Format data\n");
			  printf("\n");
              for (i=0; i<6; i++) istart[i]=itime[i];
			  *iuid=(int)uid;
			  *tvgflag=tvg;

			  if(mode==0){//old style 
				 printf("                           TIME CODE   CRC  \n");
				 printf("FRAME#   SYNC   UUUUTFFF MJDsssss.ssss  16  MJD HH:MM:SS.SSSS   Mbps\n");
        
			  } else if (mode==1){// new style
				 printf("                  TIME CODE  CRC-  \n");
				 printf("FRAME#   SYNC   MJDsssss.ssss  16  DATA#1   DATA#2     Mbps\n");
			  } else {// modified style for mode =2
				 printf("                           TIME CODE  CRC-  \n");
				 printf("FRAME#   SYNC   UUUUTFFF MJDsssss.ssss  16  DATA#1   DATA#2     Mbps\n");
			  }
		  }
       
		  nkai++;

		  dtime=((double)itime[5])/10000.0;
		  dt=dtime-dtime0;
		  if(dt <= 0.0) dt+=1.0;

		  bps=2500.0*32.0/dt;
		  mbps=bps/1.0e6;

		  /* bps cleaner */
		     // Mark5B time label does not have sufficient digit
		     // so bps is sometimes not correct
		     // now only 2^N Mbps starting from 4 Mbps is assumed
		     // as corrected value

		  if(mbps > 0.0){
			  mbps=pow(2.0, (double)((int)(log(mbps)/log(2.0) +0.5)));
			  bps=mbps*1.0e6;
		  }

		  dtime0=dtime;

		  if(nkai==1) bps=0.0;  // because unable to estimate for first
          /* print header info */
          if(nkai > 1) {
			  if(nkai==2) *fbps=bps;
			  printf(" %6.1f",bps/1.0e6);
             //       printf(" %5d",syncpos);
              printf("\n");
		     if ((nmax > 0) && (nkai > nmax)) goto exitreturn;
		  }
          printf(" %05d ",nframe);
          if(mode==0) { // old style
			  for (i=0; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head[i]);
			  }
              printf(" %03d %02d:%02d:%02d.%04d",
                    itime[1],itime[2],itime[3],itime[4],itime[5]);
		  } else if(mode==1){     // new style
		      printf("%08X ",head[0]);
			  for (i=2; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head[i]);
			  }
              for (i=0; i<2; i++){           
                 printf("%08X ",idata[i]);
			  } 
		  } else {     // modified style mode=2
			  for (i=0; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head[i]);
			  }
              for (i=0; i<2; i++){           
                 printf("%08X ",idata[i]);
			  } 
		  }
		  //if ((nmax > 0) && (nkai >= nmax)) goto exitreturn;
	   }
exitreturn:
       i=fclose(fin) ;
       return mk5B_mode;
}

void mk5b_sync_detect(int *head, int *itime, unsigned short int *uid,
					  int *tvgflag, int *nframe, unsigned short int *icrc16,
					  int *iok)
/*
     Mark5B sync detection and analysis
     2009-06-06  

  [in]
      head  ---- pointer to headder array (4 X 32 bits) buffer 

  [out]
      itime[6] --  time in header
                   -1,mjd(3digits),hh,mm,ss,msec*10  (like a VLBA format)
	  uid      --   User-specified field  (16-bit integer)
	  tvgflag  --   Tvg data flag   1: tvg data  0: not tvg data
	  nframe   --   Frame# within second (starting at 0)
	  icrc16   --   CRCC for time field  (16-bit integer)
	  iok      --   1: This is Mark5B header   0: Not Mark5B header

      header[5*tot_track] ---   header block ( 160 bit X tot_track) (parity is removed)
      head180[180*tot_tarck] ---   header block 1bit data X 180 X tot_tarck (with parity)

*/
{
    int sync_pat=0xABADDEED;   // Mark5B sync pattern

	*iok=0;


    /* sync detection */
	if( head[0]==sync_pat ){ //  sync detected
	   *iok=1;

		/* retrieve user-specified field */
       
		*uid= (unsigned short int)( (head[1] & 0xFFFF0000) >> 16);
		   
		/* retrieve Tvg flag */

		*tvgflag=0;

		if( (head[1] & 0x8000) != 0) *tvgflag=1;

		/*  retrieve frame# */

		*nframe = (int)( head[1] & 0x7FFF);

		/* retrieve time information */

		conv2time_vlba(head+2, itime);

		/* retrieve CRCC */

		*icrc16= (unsigned short int)(head[3] & 0xFFFF);
	}
}


void m5b_hd_make(int *head, int mjd, int seconds, int msec10,
				 int uid, int tvgflag, int nframe)
/*
     Mark5B header block creation
     2009-06-07 

  [in]
      mjd      --  MJD (3digits)
	  seconds  --- seconds in day
	  msec10   --- msec*10
	  uid      --   User-specified field  (lower 16-bits are valid)
	  tvgflag  --   Tvg data flag   1: tvg data  0: not tvg data
	  nframe   --   Frame# within second (starting at 0)]
  [out]
      head  ---- pointer to header array (4 X 32 bits) buffer 
*/
{
    /* Mark 5B header block creation */
    int n,a;
    int crc16;
    //int64_t lw1;
    int idat[12];
    unsigned char bdat[6];
    int sync_pat=0xABADDEED;   // Mark5B sync pattern


    /* Sync word   */
	head[0]=sync_pat;

	/* User-specfied field (16 bits) */

	head[1]=(uid & 0xFFFF) << 16;

	/* TVG flag */
	head[1]=((head[1] & 0xFFFF0000) | ((tvgflag & 0x1) << 15));

	/* Frame # field */
	head[1]=((head[1] & 0xFFFF8000) | (nframe & 0x7FFF));

    /* Time field */

    //  JJJSSSSSssss

    a=100;
    for(n=0; n<3; n++){
        idat[n]=(mjd/a) % 10;
        a/=10;
    }
    a=10000;
    for(n=3; n<8; n++){
        idat[n]=(seconds/a) % 10;
        a/=10;
    }
    a=1000;
    for(n=8; n<12; n++){
        idat[n]=(msec10/a) % 10;
        a/=10;
    }

	head[2]=idat[0] & 0xF;
    for(n=1; n<8; n++){  // 4 bit step
		head[2]=((head[2] << 4) | (idat[n] & 0xF));
	}

	head[3]=idat[8] & 0xF;
    for(n=9; n<12; n++){  // 4 bit step
		head[3]=((head[3] << 4) | (idat[n] & 0xF));
	}
	head[3] <<= 16;

    /* CRC16 */   
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

	// Mark5B bit order is reversal 
	crc16=rvsint(crc16) >> 16;


	head[3] |= (crc16 & 0xFFFF);
    
}


int convM5B2K5(m5k5runp_t *runp)
/*
    Mark5B to K5 conversion  2009-06-08 
*/
{
    int kcount,pcount,bpfram,tc,ikai,icount,opos,is,isync,nbit,ch;

    unsigned int ibuf[2504];  // for 1 Frame data 2504 X 32 bits
	unsigned int *head;   // for header 4 X 32 bits
	unsigned int *idata;  // for data field 2500 X 32 bits
    int nbuf=2504;   // @# of 32bit words in a Mark5B frame
	int nword=2500;  //   # of 32bit data words in a Mark5B frame 
	unsigned short uid, icrc16;
	unsigned int iw;
	int iok;
	int nframe;
	int tvg;
    int mk5B_mode;   // =1  Mark 5B format   =0 No
	int nbitstream;  // # of bits per bit-stream in Mark5B (1,2,4,8,16,32)

    double bps;  // supposed data rate
	double fsample;  // sampling frequency

	int iwk5;
    int ih;
    int know;
    int i1st;
    int ipick;
    int i,n,k,m,kk;
    int iread,iwrite;
    int wordsec;
    int wbuf[36];
    int istart[6],itime[6];
	int numch;
	int adbit;       // AD bits in a Mark5B (1 or 2)
	int k5numch;     // # of ch in a K5 format data (1 or 4)
	int k5ch;        // work k5 ch# in a file ( <= k5numch )
	int m5chmax;     // total mark5 ch # in a Mark5 format (1,2,4,8,16,32)
	int m5ch;        // work mark5 ch # in a Mark5 format ( <= m5chmax)
	                 //  or pick-up mark5B ch in case of K5 1ch mode
    int ierr;
    int *obuf;
    int nwrite=1024;  // obuf size to write
    int64_t iwork64,know64;
    IPBOARD_T ipb;
    FILE *fo;
	FILE *fin;
    mk5_read_t mk5;

	int *conv2bit;
	int conv2bit_1[]={0,1,2,3};   // offset binary (no conversion 2 bit)
    int conv2bit_2[]={0,2,1,3};   //  conversion 2 bit (default)
    int conv2bit_3[]={1,0,2,3};   //  conversion 2 bit singed integer

	int chtabl[4];  // ch pick up table  contents must be prepare from out side

    printf("  convM5BtoK5 : \n");
    ierr=0;

	if(runp->e2bit==1){
	   conv2bit=conv2bit_1;
	} else if(runp->e2bit==3){
	   conv2bit=conv2bit_3;
	} else {
	   conv2bit=conv2bit_2;
	}

	memcpy(chtabl,runp->chtable,4*sizeof(int));

	head=ibuf;
	idata=ibuf+4;



    /* initialize mk5_read_t type variable */


    //printf("wordsec %d\n",wordsec);
    //if(mk5info->vlba==1){
    //    offset=5;
    //} else {
    //    offset=0;
    //}

	fsample=runp->fmhz*1.0e6;
	adbit=runp->adbit;
	numch=runp->numch;
	k5numch=runp->k5numch;

	m5ch=runp->m5ch;

	m5chmax=runp->nbitstream/runp->adbit;


    //wordsec=(int)fsample/32;   // number of 32bit-words in 1 sec

    //mk5.tot_track=gr->tot_track;

    /* allocate memories */
	obuf=(int *)malloc(nwrite*sizeof(int));
	//mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
	//mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));


    /* Mark 5 data file open */
    if( (fin  = fopen( runp->mk5name, "rb" )) == NULL ){
           printf("  Data File %s   Open Error! \n",runp->mk5name);
           ierr=-1;
		   goto exit2return;
    }
    printf("  Mark-5B Data File : %s\n",runp->mk5name);
 
    printf("  conversion condition :\n");
	printf("  Numch = %d  AD bits = %d  Fsample (MHz) = %f\n",numch,adbit,fsample/1.0e6);
	//for(n=0; n< gr->tot_track; n++) printf(" %d",gr->bit_trk[n]);
	printf("\n");

    /* output file create */

    if(runp->ptype==1){ // in case of Mode 1
       if(runp->k5name_given[0]=='\0'){
           k5name_make(runp->mk5name, runp->outdir, runp->channel, 
              runp->group, runp->k5name);
	   } else {
           strcpy(runp->k5name,runp->k5name_given);
	   }
    }

    if( (fo  = fopen( runp->k5name, "wb" )) == NULL ){
       printf("  File  %s  Open Error! \n",runp->k5name);
       ierr= -2;
	   goto exit2return;
    }  
    //printf("  K5 Data File : %s\n",runp->k5name);

    pcount=0; // converted data period counter (in sec)
    bpfram=0; // counter for bit# in a frame

    tc=-1;
    icount=0;  // 32bit word counter
    is=0;
    isync=0;  // sync detection flag


    /* first sync detection */

//    iread = fread( ibuf, sizeof( unsigned int ), nbuf, fin );
//    if (iread ==0 ) { // EOF
//	   printf("\n");
//       goto exit2return; 
//	}

//    mk5b_sync_detect(head, itime, &uid, &tvg, &nframe, 
//	                &icrc16, &iok);

//   kk=0;
//   memcpy(istart,itime,6*sizeof(int));

//    printf("  Mark 5B data : 1st Sync block detected\n");
//    printf("   1st header time is %d/%03d %02d:%02d:%02d.%03d\n",
//               istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);

    /* atamadashi for just seconds start */

	  // not yet implemented

    //printf("   Atamadashi finished.\n");
    //printf("\n");

    /* K5 header data creation */

    ipb.adbit=adbit;
    //ipb.numch=gr->numch;
    ipb.numch=k5numch;
    ipb.sfreq=(int)(fsample/1000);
    //ipb.seconds=itime[2]*3600+itime[3]*60+itime[4];
    //ip_hd_make_ini(&ipb);

    printf("\n  K5 data parameters : ");
    if(ipb.sfreq >= 1000){
        printf("%d MHz %d bit sampling  %d ch data\n",
               ipb.sfreq/1000, ipb.adbit, ipb.numch);
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               ipb.sfreq, ipb.adbit, ipb.numch);
    }
    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* put K5 header data into obuf */

//    opos=0;   // out buffer writing position
//    obuf[opos]=conv4byte2int(ipb.hd);
//    opos++;
//    obuf[opos]=conv4byte2int(ipb.hd+4);
//    opos++;
 
//    nbit=0;  // bit# counter

    //printf("tot_track,nbit,opos  %d %d  %d\n",mk5.tot_track,nbit,opos);
    //printf("icount1st = %d\n",icount);
    //printf("bitsframe = %d\n",mk5.bitsframe);
//    bpfram=mk5.nbithead-1;  // current bit position in a frame  (starting from 0)
    //printf("bpfram = %d\n",bpfram);

//    ih=mk5.nbithead-1;  // bit# in a second counter

    /* header monitor */
    printf("  ****************************************************************\n");
    printf("               Mark5B Header Data\n");
    printf("                           TIME CODE  CRC-  \n");
    printf("   HH:MM:SS   SYNC   UUUUTFFF MJDsssss.ssss  16  DATA#1   DATA#2 \n");
    printf("  ----------------------------------------------------------------\n");
       
    i1st=1;
    ikai=0;   // wbuf wtiting pointer    0<= ikai <32

    kcount=0; // bit position+1 in a byte
	nbit=0;

    while(1){

		/* Mark5B 1 frame read */

        iread = fread( ibuf, sizeof( unsigned int ), nbuf, fin );
        if (iread ==0 ) { // EOF
           goto exithere; 
		}

        mk5b_sync_detect(head, itime, &uid, &tvg, &nframe, 
	                     &icrc16, &iok);

		if(iok != 1) goto exit2return; // not Mark 5B data


		if(itime[5]==0){ // start of seconds

			if(i1st==1){
				memcpy(istart,itime,6*sizeof(int));
				i1st=0;
		    	opos=0;   // out buffer writing position
			}

            /* info display */
		    printf("   %02d:%02d:%02d ",itime[2],itime[3],itime[4]);
			for (i=0; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head[i]);
			}
            for (i=0; i<2; i++){           
                 printf("%08X ",idata[i]);
			} 
		    printf("\n");

			/* K5 header generate */
            ipb.seconds=itime[2]*3600+itime[3]*60+itime[4];
            ip_hd_make_ini(&ipb);

			/* put K5 header data into obuf */

			//opos=0;   // out buffer writing position
			obuf[opos]=conv4byte2int(ipb.hd);
			opos++;
            if(opos == nwrite){
               iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
               opos=0;
			}
			obuf[opos]=conv4byte2int(ipb.hd+4);
			opos++;
            if(opos == nwrite){
               iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
               opos=0;
			}
		}

		/* conver to K5 data */

		iwk5=obuf[opos];
		for(n=0; n<nword; n++){   // nword = 2500 
			iw=idata[n];
		    iwk5=obuf[opos];
			if(k5numch ==1){

			   /* K5 1ch mode */
                conv_m5b2k5_1ch(runp->nbitstream, runp->adbit, m5chmax,
					            iw, m5ch, conv2bit, &nbit, &iwk5);
				obuf[opos]=iwk5;
				if(nbit==32){
				    nbit=0;
				    opos++;
				}
			} else {
               /* convert to K5 4ch mode */
                 conv_m5b2k5_4ch(runp->nbitstream, runp->adbit, m5chmax,
					             iw, chtabl, conv2bit, &nbit, &iwk5);
			 	 obuf[opos]=iwk5;
				 if(nbit==32){
				    nbit=0;
				    opos++;
				 }
			}

            if(opos == nwrite){
               iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
               opos=0;
			}
		}
	}
exithere:
    i=fclose( fin) ;
    // rest of data write
    if(opos != 0){
       iwrite=fwrite(obuf,sizeof(int),opos,fo);
	}
    i=fclose( fo ) ;
    printf("  ****************************************************************\n");
	printf("\n");
exit2return:
       //printf("  K5 file (%s) has been created.\n\n",runp->k5name);
    free(obuf);

    return 0;
}


void conv_m5b2k5_1ch(int nbitstream, int adbit, int chmax,
					int iwM5B, int m5ch, int *conv2bit,
					int *nbit, int *iwK5)
/*
    Mark5B to K5 conversion 1ch mode engine 2009-06-08 

  [in]
     nbitstream  --- Mark5B   # of bit-stream  (1,2,4,8,16,32)
	 adbit       --- Mark5B   AD bits  (1,2)
	 chmax       --- Mark5B   available maximum # of channels bit)
	                          (=nbitstream/adbit)
	 iwM5B       --- Mark5B   a 32bit word data
     m5ch        --- Mark5B   ch# to be picked up (starting at 1)
	                    in case of non-existing ch#, zero data is
						filled in K5 data
	 conv2bit    --- 2bit data conversion table
	 nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 
     iwK5        --- K5 a 32bit word data

  [return]
	 nbit        --- already filled out bits in iwK5 (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
	int i,m;
	int iw, ik5, nb, ipick;
	int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
		         0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
		         0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
		         0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
		         0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
		         0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
		         0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
				 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
	int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

	int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
		          0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
		          0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
				  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

	//chmax=nbitstream/adbit;

	iw=iwM5B;
	ik5= *iwK5;
	nb=*nbit;

	switch(nbitstream){  
	   case 1:
		   ik5=iw;
		   nb+=32;
    	   break;
	   case 2:
		   if(adbit == 2){  // M5 ch# is 1
			  for(i=0; i< 16; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
				       m=2*i;
				       ipick= (int)(iw >> m) & 0x3;
					   ipick=conv2bit[ipick];
				   }
				   ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
				   nb+=2;
			  }
		   } else {  // adbit=1,  pick up M5 ch# (starting at 1 to 2) 1bit data
			   for(i=0; i< 16; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
				       m=2*i+(m5ch-1);
					   ipick= (int)(iw >> m) & 0x1;
				   }
				   ik5 = (ik5 & imask[nb]) | (ipick << nb);
   				   nb++;
			   }
		   }
		   break;
	   case 4:
		   if(adbit == 2){ // pick up M5 ch# (1 or 2) data
			   for(i=0; i< 8; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
					   m=4*i+(m5ch-1)*2;
					   ipick= (int)(iw >> m) & 0x3;
					   ipick=conv2bit[ipick];
				   }
				   ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
				   nb+=2;
			   }
		   } else {    // pick up M5 ch# (1-4) 1bit data
			  for(i=0; i< 8; i++){
				   if(m5ch > chmax){
					  ipick=0;
				   } else {
					  m=4*i+(m5ch-1);
					  ipick= (int)(iw >> m) & 0x1;
				   }
				   ik5 = (ik5 & imask[nb]) | (ipick << nb);
				   nb++;
			  }
		   }
		   break;
	   case 8:
		   if(adbit == 2){ // pick up M5 ch# (1 -4) data
			   for(i=0; i< 4; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
					   m=8*i+(m5ch-1)*2;
					   ipick= (int)(iw >> m) & 0x3;
					   ipick=conv2bit[ipick];
				   }
				   ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
				   nb+=2;
			   }
		   } else {    // pick up M5 ch# (1-8) 1bit data
			   for(i=0; i< 4; i++){
				   if(m5ch > chmax){
					  ipick=0;
				   } else {
					   m=8*i+(m5ch-1);
					   ipick= (int)(iw >> m) & 0x1;
				   }
				   ik5 = (ik5 & imask[nb]) | (ipick << nb);
				   nb++;
			   }
		   }
		   break;
	   case 16:
		   if(adbit == 2){ // pick up M5 ch# (1 -8) 2 bit data
			   for(i=0; i< 2; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
				       m=16*i+(m5ch-1)*2;
					   ipick= (int)(iw >> m) & 0x3;
					   ipick=conv2bit[ipick];
				   }
				   ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
				   nb+=2;
			   }
		   } else {    // pick up M5 ch# (1-16) 1bit data
			   for(i=0; i< 2; i++){
				   if(m5ch > chmax){
					   ipick=0;
				   } else {
  					   m=16*i+(m5ch-1);
					   ipick= (int)(iw >> m) & 0x1;
				   }
				   ik5 = (ik5 & imask[nb]) | (ipick << nb);
				   nb++;
			   }
		   }
		   break;
	   case 32:
		   if(adbit == 2){ // pick up M5 ch# (1 -16) 2 bit data
			   if(m5ch > chmax){
				   ipick=0;
			   } else {
				   m=(m5ch-1)*2;
				   ipick= (int)(iw >> m) & 0x3;
				   ipick=conv2bit[ipick];
			   }
			   ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
			   nb+=2;
		   } else {    // pick up M5 ch# (1-32) 1bit data
			   if(m5ch > chmax){
				   ipick=0;
			   } else {
			       m=(m5ch-1);
				   ipick= (int)(iw >> m) & 0x1;
			   }
			   ik5 = (ik5 & imask[nb]) | (ipick << nb);
			   nb++;
		   }
		   break;
	   default: ;
	}
	*nbit=nb;
	*iwK5=ik5;
}


void conv_m5b2k5_4ch(int nbitstream, int adbit, int chmax,
					int iwM5B, int *chtable, int *conv2bit,
					int *nbit, int *iwK5)
/*
    Mark5B to K5 conversion 4ch mode engine 2009-06-08 
	Only  MAximum Mark5B channel exceeds 4 is supported

  [in]
     nbitstream  --- Mark5B   # of bit-stream  (1,2,4,8,16,32)
	 adbit       --- Mark5B   AD bits  (1,2)
	 chmax       --- Mark5B   available maximum # of channels bit)
	                          (=nbitstream/adbit)
	 iwM5B       --- Mark5B   a 32bit word data
     chtable[4]  --- Mark5B   ch# to be picked up for K5 4ch data
	                    (starting at 1)
	                    in case of non-existing ch#, zero data is
						filled in K5 data
	 conv2bit    --- 2bit data conversion table
	 nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
	 nbit        --- already filled out bits in iwo (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
	int i,m;
	int iw, ik5, nb, ipick;
	int k5ch,m5ch;
	int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
		         0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
		         0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
		         0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
		         0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
		         0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
		         0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
				 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
	int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

	int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
		          0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
		          0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
				  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

	//chmax=nbitstream/adbit;

	iw=iwM5B;
	ik5= *iwK5;
	nb=*nbit;

	if(chmax < 4) goto exitreturn; // nothing to do

	switch(nbitstream){  
	   case 4:     // only 1 bit AD is allowded for this mode
				   // pick up all 4 ch data
  	       for(i=0; i< 8; i++){
		       for(k5ch=0; k5ch<4; k5ch++){
		  	       m5ch=chtable[k5ch];
				   m=4*i+(m5ch-1);
				   ipick= (int)(iw >> m) & 0x1;
				   ik5 = (ik5 & imask[nb]) | (ipick << nb);
				   nb++;
			   }
		   }
		   break;
	   case 8:
		   if(adbit == 2){ // pick up M5 ch# (1 -4) data
			   for(i=0; i< 4; i++){
			       for(k5ch=0; k5ch<4; k5ch++){
			           m5ch=chtable[k5ch];
				       if(m5ch > chmax){
					      ipick=0;
					   } else {
					      m=8*i+(m5ch-1)*2;
					      ipick= (int)(iw >> m) & 0x3;
				          ipick=conv2bit[ipick];
					   }
			           ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
			           nb+=2;
				   }
			   }
		   } else {    // pick up M5 ch# (1-8) 1bit data
			   for(i=0; i< 4; i++){
			       for(k5ch=0; k5ch<4; k5ch++){
					   m5ch=chtable[k5ch];
				       if(m5ch > chmax){
					      ipick=0;
					   } else {
						   m=8*i+(m5ch-1);
				  	       ipick= (int)(iw >> m) & 0x1;
					   }
			           ik5 = (ik5 & imask[nb]) | (ipick << nb);
			           nb++;
				   }
			   }
		   }
		   break;
	   case 16:
		   if(adbit == 2){ // pick up M5 ch# (1 -8) 2 bit data
			   for(i=0; i< 2; i++){
			       for(k5ch=0; k5ch<4; k5ch++){
			           m5ch=chtable[k5ch];
				       if(m5ch > chmax){
					      ipick=0;
					   } else {
						   m=16*i+(m5ch-1)*2;
						   ipick= (int)(iw >> m) & 0x3;
				           ipick=conv2bit[ipick];
					   }
			           ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
			           nb+=2;
				   }
			   }
		   } else {    // pick up M5 ch# (1-16) 1bit data
			   for(i=0; i< 2; i++){
				   for(k5ch=0; k5ch<4; k5ch++){
					   m5ch=chtable[k5ch];
				       if(m5ch > chmax){
					      ipick=0;
					   } else {
						  m=16*i+(m5ch-1);
						  ipick= (int)(iw >> m) & 0x1;
					   }
			           ik5 = (ik5 & imask[nb]) | (ipick << nb);
			           nb++;
				   }
			   }
		   }
		   break;
	   case 32:
		   if(adbit == 2){ // pick up M5 ch# (1 -16) 2 bit data
			   for(k5ch=0; k5ch<4; k5ch++){
				   m5ch=chtable[k5ch];
				   if(m5ch > chmax){
				      ipick=0;
				   } else {
					  m=(m5ch-1)*2;
					  ipick= (int)(iw >> m) & 0x3;
				      ipick=conv2bit[ipick];
				   }
			       ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
			       nb+=2;
			   }
		   } else {    // pick up M5 ch# (1-32) 1bit data
			   for(k5ch=0; k5ch<4; k5ch++){
				   m5ch=chtable[k5ch];
				   if(m5ch > chmax){
				      ipick=0;
				   } else {
		 	          m=m5ch-1;
				      ipick= (int)(iw >> m) & 0x1;
				   }
			       ik5 = (ik5 & imask[nb]) | (ipick << nb);
			       nb++;
			   }
		   }
		   break;
	   default: ;
	}
exitreturn:
	*nbit=nb;
	*iwK5=ik5;
}
